//-----------------------------------------------------------------------------
// <copyright file="IfxMetrics.h" company="Microsoft">
// Copyright (c) Microsoft Corporation.  All rights reserved.
// </copyright>
//
// The Azure multi-dimention metrics publishing APIs in IFx package.
//-----------------------------------------------------------------------------

#pragma once

#include <windows.h>


// Constants

// The maximum number of user defined dimensions for each metric.
#define MAX_NUMBER_DIMENSIONS 64

// The maximum number of default dimensions for each metric. Total number of supported dimension is 74 (user defined + default dimension)
#define MAX_NUMBER_DEFAULT_DIMENSIONS 10

// The maximum length of the metric namespace string.
#define MAX_NAMESPACE_SIZE 1024

// The maximum length of the name string, currently this applies to metric name as well as dimension names.
#define MAX_NAME_SIZE 256
#define MAX_DIMENSION_NAME_SIZE MAX_NAME_SIZE

// The maximum length of the dimension value string.
#define MAX_DIMENSION_VALUE_SIZE 1024

// C-style APIs

extern "C" {

#if defined(_WIN32)

#define AttachThreadToIfx()
#define DetachThreadFromIfx()

#else
    BOOL AttachThreadToIfx();
    BOOL DetachThreadFromIfx();
#endif

}

// Return error code, NO_ERROR if no error.
uint32_t IfxStartup();
void IfxCleanup();

// The APIs for count metric.

// The API to create one measure metric.
//
// The input parameters:
// - The monitoringAccount name; in Azure world, this usually is the tenant name.
// - The metric namespace; similar to the counter object name in the Windows performance counter terminology.
// - The metric name; similar to the counter name in the Windows performance counter terminology.
// - The number of dimensions the metric has.
// - The names of all the dimensions.
// - The boolean flag that specifies whether to add default dimension or not
//
// The output parameter:
// - The context handle to the created metric.
//
// The result:
// - HRESULT whether the operation succeed.
extern "C" HRESULT __stdcall CreateIfxMeasureMetric(
    __out HANDLE* hMetric,
    __in_z LPCSTR monitoringAccount,
    __in_z LPCSTR metricNamespace,
    __in_z LPCSTR metricName,
    __in DWORD countDimension,
    __in_ecount_opt(countDimension) LPCSTR* listDimensionNames,
    __in bool addDefaultDimension = false
    );

// There are 2 variations of measure metric set APIs, the variations come whether the timestamp is provided
// explicitly or implicitly using the ETW timestamp.
//
// The input parameters:
// - The context handle of the measure metric.
// - The number of the dimensions; this should match the number of the dimensions when the measure metric created.
// - The values of all dimensions.
//
// The optional input parameters:
// - The UTC timestamp when the count metric increment happens; if not provided, use the timestmap in ETW event header.
//
// The result:
// - HRESULT whether the operation succeeds.

// The API to set the measure metric; use the implicit timestamp.
extern "C" HRESULT __stdcall SetIfxMeasureMetric(
    __in HANDLE hMetric,
    __in LONG64 rawData,
    __in DWORD countDimension,
    __in_ecount_opt(countDimension) LPCSTR* listDimensionValues
    );

// The API to set the measure metric; use the explicit UTC timestamp.
extern "C" HRESULT __stdcall SetIfxMeasureMetricWithTimestamp(
    __in HANDLE hMetric,
    __in ULONG64 timestampUtc,
    __in LONG64 rawData,
    __in DWORD countDimension,
    __in_ecount_opt(countDimension) LPCSTR* listDimensionValues
    );

// The API to set the default monitoring account name
extern "C" HRESULT __stdcall SetDefaultMonitoringAccount(
    __in_z LPCSTR monitoringAccount
    );

// The API to get the default monitoring account name
extern "C" HRESULT __stdcall GetDefaultMonitoringAccount(
    __out_ecount_z(monitoringAccountBufferSize) LPSTR monitoringAccount,
    __in DWORD monitoringAccountBufferSize
    );

// The API to set the default dimensions names-values pair
// Only one call per process is allowed for SetDefaultDimension. 
// Default dimension once set can't be overwritten by further call to SetDefaultDimension
// As all further calls would throw an invalid argument exception.
extern "C" HRESULT __stdcall SetDefaultDimensionNamesValues(
    __in DWORD countDefaultDimension,
    __in_ecount_opt(countDefaultDimension) LPCSTR* listDefaultDimensionNames,
    __in_ecount_opt(countDefaultDimension) LPCSTR* listDefaultDimensionValues
    );

// The C++ style public classes.

// The generic class for measure metric; the number of dimensions is determined by the dimension name
// string array passed in the Create call.
class MeasureMetric
{
private:
    HANDLE _hMetric;
    int    _numDimension;

    MeasureMetric(HANDLE hMetric, int countDimension)
    {
        _hMetric = hMetric;
        _numDimension = countDimension;
    }

public:
    // Special case static create metric static function for 0D MDM metrics.
    static MeasureMetric* Create(
        __in_z LPCSTR monitoringAccount,
        __in_z LPCSTR metricNamespace,
        __in_z LPCSTR metricName)
    {
        HANDLE hMetric = NULL;

        if (SUCCEEDED(CreateIfxMeasureMetric(
                &hMetric, monitoringAccount, metricNamespace, metricName, 0, NULL, FALSE)))
        {
            return new MeasureMetric(hMetric, 0);
        }
        else
        {
            return NULL;
        }
    }

    static MeasureMetric* Create(
        __in bool addDefaultDimension,
        __in_z LPCSTR monitoringAccount,
        __in_z LPCSTR metricNamespace,
        __in_z LPCSTR metricName)
    {
        HANDLE hMetric = NULL;

        if (SUCCEEDED(CreateIfxMeasureMetric(
            &hMetric, monitoringAccount, metricNamespace, metricName, 0, NULL, addDefaultDimension)))
        {
            return new MeasureMetric(hMetric, 0);
        }
        else
        {
            return NULL;
        }
    }

    // Create metric static function for MDM metrics with dimentions.
    static MeasureMetric* Create(
        __in_z LPCSTR monitoringAccount,
        __in_z LPCSTR metricNamespace,
        __in_z LPCSTR metricName,
        __in int countDimension,
        ...)
    {
        LPCSTR dimensionNames[MAX_NUMBER_DIMENSIONS];
        HANDLE hMetric = NULL;

        if (countDimension > MAX_NUMBER_DIMENSIONS)
        {
            // Exceed max number of dimensions allowed.
            return NULL;
        }
        else
        {
            va_list dimensionNameList;
            int i;

            va_start(dimensionNameList, countDimension);
            for (i = 0; i < countDimension; i++)
            {
                dimensionNames[i] = (LPCSTR) va_arg(dimensionNameList, PSTR);
            }
            va_end(dimensionNameList);
        }

        if (SUCCEEDED(CreateIfxMeasureMetric(
               &hMetric, monitoringAccount, metricNamespace, metricName, countDimension, dimensionNames, FALSE)))
        {
            return new MeasureMetric(hMetric, countDimension);
        }
        else
        {
            return NULL;
        }
    }

    static MeasureMetric* Create(
        __in bool addDefaultDimension,
        __in_z LPCSTR monitoringAccount,
        __in_z LPCSTR metricNamespace,
        __in_z LPCSTR metricName,
        __in int countDimension,
        ...)
    {
        LPCSTR dimensionNames[MAX_NUMBER_DIMENSIONS];
        HANDLE hMetric = NULL;

        if (countDimension > MAX_NUMBER_DIMENSIONS)
        {
            // Exceed max number of dimensions allowed.
            return NULL;
        }
        else
        {
            va_list dimensionNameList;
            int i;

            va_start(dimensionNameList, countDimension);
            for (i = 0; i < countDimension; i++)
            {
                dimensionNames[i] = (LPCSTR) va_arg(dimensionNameList, PSTR);
            }
            va_end(dimensionNameList);
        }

        if (SUCCEEDED(CreateIfxMeasureMetric(
                &hMetric, monitoringAccount, metricNamespace, metricName, countDimension, dimensionNames, addDefaultDimension)))
        {
            return new MeasureMetric(hMetric, countDimension);
        }
        else
        {
            return NULL;
        }
    }

    // Set MDM value functions for MDM metrics with dimensions.
    HRESULT LogCurrentValue(
        __in LONG64 rawData,
        ...)
    {
        if (_numDimension <= 0)
        {
            return SetIfxMeasureMetric(_hMetric, rawData, 0, NULL);
        }
        else
        {
            LPCSTR dimensionValues[MAX_NUMBER_DIMENSIONS];
            va_list dimensionValueList;
            int i;

            va_start(dimensionValueList, rawData);
            for (i = 0; i < _numDimension; i++)
            {
                dimensionValues[i] = (LPCSTR)va_arg(dimensionValueList, PSTR);
            }
            va_end(dimensionValueList);

            return SetIfxMeasureMetric(_hMetric, rawData, _numDimension, dimensionValues);
        }
    }

    HRESULT LogValue(
        __in ULONG64 timestampUtc,
        __in LONG64 rawData,
        ...)
    {
        if (_numDimension <= 0)
        {
            return SetIfxMeasureMetricWithTimestamp(_hMetric, timestampUtc, rawData, 0, NULL);
        }
        else
        {
            LPCSTR dimensionValues[MAX_NUMBER_DIMENSIONS];
            va_list dimensionValueList;
            int i;

            va_start(dimensionValueList, rawData);
            for (i = 0; i < _numDimension; i++)
            {
                dimensionValues[i] = (LPCSTR)va_arg(dimensionValueList, PSTR);
            }
            va_end(dimensionValueList);

            return SetIfxMeasureMetricWithTimestamp(_hMetric, timestampUtc, rawData, _numDimension, dimensionValues);
        }
    }
};

// The short-cut class for measure metric with 0 dimension.
class MeasureMetric0D
{
private:
    HANDLE _hMetric;

    MeasureMetric0D(HANDLE hMetric)
    {
        _hMetric = hMetric;
    }

public:
    static MeasureMetric0D* Create(
        __in_z LPCSTR monitoringAccount,
        __in_z LPCSTR metricNamespace,
        __in_z LPCSTR metricName,
        __in bool addDefaultDimension = false)
    {
        HANDLE hMetric = NULL;

        if (SUCCEEDED(CreateIfxMeasureMetric(
            &hMetric, monitoringAccount, metricNamespace, metricName, 0, NULL, addDefaultDimension)))
        {
            return new MeasureMetric0D(hMetric);
        }
        else
        {
            return NULL;
        }
    }

    HRESULT LogValue(
        __in LONG64 rawData)
    {
        return SetIfxMeasureMetric(_hMetric, rawData, 0, NULL);
    }

    HRESULT LogValue(
        __in ULONG64 timestampUtc,
        __in LONG64 rawData)
    {
        return SetIfxMeasureMetricWithTimestamp(_hMetric, timestampUtc, rawData, 0, NULL);
    }
};

// The short-cut class for measure metric with 1 dimension.
class MeasureMetric1D
{
private:
    HANDLE _hMetric;

    MeasureMetric1D(HANDLE hMetric)
    {
        _hMetric = hMetric;
    }

public:
    static MeasureMetric1D* Create(
        __in_z LPCSTR monitoringAccount,
        __in_z LPCSTR metricNamespace,
        __in_z LPCSTR metricName,
        __in_z LPCSTR dimensionName,
        __in bool addDefaultDimension = false)
    {
        HANDLE hMetric = NULL;

        if (SUCCEEDED(CreateIfxMeasureMetric(
            &hMetric, monitoringAccount, metricNamespace, metricName, 1, &dimensionName, addDefaultDimension)))
        {
            return new MeasureMetric1D(hMetric);
        }
        else
        {
            return NULL;
        }
    }

    HRESULT LogValue(
        __in LONG64 rawData,
        __in_z LPCSTR dimensionValue)
    {
        return SetIfxMeasureMetric(_hMetric, rawData, 1, &dimensionValue);
    }

    HRESULT LogValue(
        __in ULONG64 timestampUtc,
        __in LONG64 rawData,
        __in_z LPCSTR dimensionValue)
    {
        return SetIfxMeasureMetricWithTimestamp(_hMetric, timestampUtc, rawData, 1, &dimensionValue);
    }
};

// The short-cut class for measure metric with 2 dimensions.
class MeasureMetric2D
{
private:
    HANDLE _hMetric;

    MeasureMetric2D(HANDLE hMetric)
    {
        _hMetric = hMetric;
    }

public:
    static MeasureMetric2D* Create(
        __in_z LPCSTR monitoringAccount,
        __in_z LPCSTR metricNamespace,
        __in_z LPCSTR metricName,
        __in_z LPCSTR dimensionName1,
        __in_z LPCSTR dimensionName2,
        __in bool addDefaultDimension = false)
    {
        HANDLE hMetric = NULL;
        LPCSTR listDimensions[2];

        listDimensions[0] = dimensionName1;
        listDimensions[1] = dimensionName2;

        if (SUCCEEDED(CreateIfxMeasureMetric(
            &hMetric, monitoringAccount, metricNamespace, metricName, 2, listDimensions, addDefaultDimension)))
        {
            return new MeasureMetric2D(hMetric);
        }
        else
        {
            return NULL;
        }
    }

    HRESULT LogValue(
        __in LONG64 rawData,
        __in_z LPCSTR dimensionValue1,
        __in_z LPCSTR dimensionValue2)
    {
        LPCSTR listDimensions[2];

        listDimensions[0] = dimensionValue1;
        listDimensions[1] = dimensionValue2;

        return SetIfxMeasureMetric(_hMetric, rawData, 2, listDimensions);
    }

    HRESULT LogValue(
        __in ULONG64 timestampUtc,
        __in LONG64 rawData,
        __in_z LPCSTR dimensionValue1,
        __in_z LPCSTR dimensionValue2)
    {
        LPCSTR listDimensions[2];

        listDimensions[0] = dimensionValue1;
        listDimensions[1] = dimensionValue2;

        return SetIfxMeasureMetricWithTimestamp(_hMetric, timestampUtc, rawData, 2, listDimensions);
    }
};

// The short-cut class for measure metric with 3 dimensions
class MeasureMetric3D
{
private:
    HANDLE _hMetric;

    MeasureMetric3D(HANDLE hMetric)
    {
        _hMetric = hMetric;
    }

public:
    static MeasureMetric3D* Create(
        __in_z LPCSTR monitoringAccount,
        __in_z LPCSTR metricNamespace,
        __in_z LPCSTR metricName,
        __in_z LPCSTR dimensionName1,
        __in_z LPCSTR dimensionName2,
        __in_z LPCSTR dimensionName3,
        __in bool addDefaultDimension = false)
    {
        HANDLE hMetric = NULL;
        LPCSTR listDimensions[3];

        listDimensions[0] = dimensionName1;
        listDimensions[1] = dimensionName2;
        listDimensions[2] = dimensionName3;

        if (SUCCEEDED(CreateIfxMeasureMetric(
            &hMetric, monitoringAccount, metricNamespace, metricName, 3, listDimensions, addDefaultDimension)))
        {
            return new MeasureMetric3D(hMetric);
        }
        else
        {
            return NULL;
        }
    }

    HRESULT LogValue(
        __in LONG64 rawData,
        __in_z LPCSTR dimensionValue1,
        __in_z LPCSTR dimensionValue2,
        __in_z LPCSTR dimensionValue3)
    {
        LPCSTR listDimensions[3];

        listDimensions[0] = dimensionValue1;
        listDimensions[1] = dimensionValue2;
        listDimensions[2] = dimensionValue3;

        return SetIfxMeasureMetric(_hMetric, rawData, 3, listDimensions);
    }

    HRESULT LogValue(
        __in ULONG64 timestampUtc,
        __in LONG64 rawData,
        __in_z LPCSTR dimensionValue1,
        __in_z LPCSTR dimensionValue2,
        __in_z LPCSTR dimensionValue3)
    {
        LPCSTR listDimensions[3];

        listDimensions[0] = dimensionValue1;
        listDimensions[1] = dimensionValue2;
        listDimensions[2] = dimensionValue3;

        return SetIfxMeasureMetricWithTimestamp(_hMetric, timestampUtc, rawData, 3, listDimensions);
    }
};

// The short-cut class for measure metric with 4 dimensions.
class MeasureMetric4D
{
private:
    HANDLE _hMetric;

    MeasureMetric4D(HANDLE hMetric)
    {
        _hMetric = hMetric;
    }

public:
    static MeasureMetric4D* Create(
        __in_z LPCSTR monitoringAccount,
        __in_z LPCSTR metricNamespace,
        __in_z LPCSTR metricName,
        __in_z LPCSTR dimensionName1,
        __in_z LPCSTR dimensionName2,
        __in_z LPCSTR dimensionName3,
        __in_z LPCSTR dimensionName4,
        __in bool addDefaultDimension = false)
    {
        HANDLE hMetric = NULL;
        LPCSTR listDimensions[4];

        listDimensions[0] = dimensionName1;
        listDimensions[1] = dimensionName2;
        listDimensions[2] = dimensionName3;
        listDimensions[3] = dimensionName4;

        if (SUCCEEDED(CreateIfxMeasureMetric(
            &hMetric, monitoringAccount, metricNamespace, metricName, 4, listDimensions, addDefaultDimension)))
        {
            return new MeasureMetric4D(hMetric);
        }
        else
        {
            return NULL;
        }
    }

    HRESULT LogValue(
        __in LONG64 rawData,
        __in_z LPCSTR dimensionValue1,
        __in_z LPCSTR dimensionValue2,
        __in_z LPCSTR dimensionValue3,
        __in_z LPCSTR dimensionValue4)
    {
        LPCSTR listDimensions[4];

        listDimensions[0] = dimensionValue1;
        listDimensions[1] = dimensionValue2;
        listDimensions[2] = dimensionValue3;
        listDimensions[3] = dimensionValue4;

        return SetIfxMeasureMetric(_hMetric, rawData, 4, listDimensions);
    }

    HRESULT LogValue(
        __in ULONG64 timestampUtc,
        __in LONG64 rawData,
        __in_z LPCSTR dimensionValue1,
        __in_z LPCSTR dimensionValue2,
        __in_z LPCSTR dimensionValue3,
        __in_z LPCSTR dimensionValue4)
    {
        LPCSTR listDimensions[4];

        listDimensions[0] = dimensionValue1;
        listDimensions[1] = dimensionValue2;
        listDimensions[2] = dimensionValue3;
        listDimensions[3] = dimensionValue4;

        return SetIfxMeasureMetricWithTimestamp(_hMetric, timestampUtc, rawData, 4, listDimensions);
    }
};

// The short-cut class for measure metric with 5 dimensions.
class MeasureMetric5D
{
private:
    HANDLE _hMetric;

    MeasureMetric5D(HANDLE hMetric)
    {
        _hMetric = hMetric;
    }

public:
    static MeasureMetric5D* Create(
        __in_z LPCSTR monitoringAccount,
        __in_z LPCSTR metricNamespace,
        __in_z LPCSTR metricName,
        __in_z LPCSTR dimensionName1,
        __in_z LPCSTR dimensionName2,
        __in_z LPCSTR dimensionName3,
        __in_z LPCSTR dimensionName4,
        __in_z LPCSTR dimensionName5,
        __in bool addDefaultDimension = false)
    {
        HANDLE hMetric = NULL;
        LPCSTR listDimensions[5];

        listDimensions[0] = dimensionName1;
        listDimensions[1] = dimensionName2;
        listDimensions[2] = dimensionName3;
        listDimensions[3] = dimensionName4;
        listDimensions[4] = dimensionName5;

        if (SUCCEEDED(CreateIfxMeasureMetric(
            &hMetric, monitoringAccount, metricNamespace, metricName, 5, listDimensions, addDefaultDimension)))
        {
            return new MeasureMetric5D(hMetric);
        }
        else
        {
            return NULL;
        }
    }

    HRESULT LogValue(
        __in LONG64 rawData,
        __in_z LPCSTR dimensionValue1,
        __in_z LPCSTR dimensionValue2,
        __in_z LPCSTR dimensionValue3,
        __in_z LPCSTR dimensionValue4,
        __in_z LPCSTR dimensionValue5)
    {
        LPCSTR listDimensions[5];

        listDimensions[0] = dimensionValue1;
        listDimensions[1] = dimensionValue2;
        listDimensions[2] = dimensionValue3;
        listDimensions[3] = dimensionValue4;
        listDimensions[4] = dimensionValue5;

        return SetIfxMeasureMetric(_hMetric, rawData, 5, listDimensions);
    }

    HRESULT LogValue(
        __in ULONG64 timestampUtc,
        __in LONG64 rawData,
        __in_z LPCSTR dimensionValue1,
        __in_z LPCSTR dimensionValue2,
        __in_z LPCSTR dimensionValue3,
        __in_z LPCSTR dimensionValue4,
        __in_z LPCSTR dimensionValue5)
    {
        LPCSTR listDimensions[5];

        listDimensions[0] = dimensionValue1;
        listDimensions[1] = dimensionValue2;
        listDimensions[2] = dimensionValue3;
        listDimensions[3] = dimensionValue4;
        listDimensions[4] = dimensionValue5;

        return SetIfxMeasureMetricWithTimestamp(_hMetric, timestampUtc, rawData, 5, listDimensions);
    }
};

// The short-cut class for measure metric with 6 dimensions.
class MeasureMetric6D
{
private:
    HANDLE _hMetric;

    MeasureMetric6D(HANDLE hMetric)
    {
        _hMetric = hMetric;
    }

public:
    static MeasureMetric6D* Create(
        __in_z LPCSTR monitoringAccount,
        __in_z LPCSTR metricNamespace,
        __in_z LPCSTR metricName,
        __in_z LPCSTR dimensionName1,
        __in_z LPCSTR dimensionName2,
        __in_z LPCSTR dimensionName3,
        __in_z LPCSTR dimensionName4,
        __in_z LPCSTR dimensionName5,
        __in_z LPCSTR dimensionName6,
        __in bool addDefaultDimension = false)
    {
        HANDLE hMetric = NULL;
        LPCSTR listDimensions[6];

        listDimensions[0] = dimensionName1;
        listDimensions[1] = dimensionName2;
        listDimensions[2] = dimensionName3;
        listDimensions[3] = dimensionName4;
        listDimensions[4] = dimensionName5;
        listDimensions[5] = dimensionName6;

        if (SUCCEEDED(CreateIfxMeasureMetric(
            &hMetric, monitoringAccount, metricNamespace, metricName, 6, listDimensions, addDefaultDimension)))
        {
            return new MeasureMetric6D(hMetric);
        }
        else
        {
            return NULL;
        }
    }

    HRESULT LogValue(
        __in LONG64 rawData,
        __in_z LPCSTR dimensionValue1,
        __in_z LPCSTR dimensionValue2,
        __in_z LPCSTR dimensionValue3,
        __in_z LPCSTR dimensionValue4,
        __in_z LPCSTR dimensionValue5,
        __in_z LPCSTR dimensionValue6)
    {
        LPCSTR listDimensions[6];

        listDimensions[0] = dimensionValue1;
        listDimensions[1] = dimensionValue2;
        listDimensions[2] = dimensionValue3;
        listDimensions[3] = dimensionValue4;
        listDimensions[4] = dimensionValue5;
        listDimensions[5] = dimensionValue6;

        return SetIfxMeasureMetric(_hMetric, rawData, 6, listDimensions);
    }

    HRESULT LogValue(
        __in ULONG64 timestampUtc,
        __in LONG64 rawData,
        __in_z LPCSTR dimensionValue1,
        __in_z LPCSTR dimensionValue2,
        __in_z LPCSTR dimensionValue3,
        __in_z LPCSTR dimensionValue4,
        __in_z LPCSTR dimensionValue5,
        __in_z LPCSTR dimensionValue6)
    {
        LPCSTR listDimensions[6];

        listDimensions[0] = dimensionValue1;
        listDimensions[1] = dimensionValue2;
        listDimensions[2] = dimensionValue3;
        listDimensions[3] = dimensionValue4;
        listDimensions[4] = dimensionValue5;
        listDimensions[5] = dimensionValue6;

        return SetIfxMeasureMetricWithTimestamp(_hMetric, timestampUtc, rawData, 6, listDimensions);
    }
};

// The short-cut class for measure metric with 7 dimensions.
class MeasureMetric7D
{
private:
    HANDLE _hMetric;

    MeasureMetric7D(HANDLE hMetric)
    {
        _hMetric = hMetric;
    }

public:
    static MeasureMetric7D* Create(
        __in_z LPCSTR monitoringAccount,
        __in_z LPCSTR metricNamespace,
        __in_z LPCSTR metricName,
        __in_z LPCSTR dimensionName1,
        __in_z LPCSTR dimensionName2,
        __in_z LPCSTR dimensionName3,
        __in_z LPCSTR dimensionName4,
        __in_z LPCSTR dimensionName5,
        __in_z LPCSTR dimensionName6,
        __in_z LPCSTR dimensionName7,
        __in bool addDefaultDimension = false)
    {
        HANDLE hMetric = NULL;
        LPCSTR listDimensions[7];

        listDimensions[0] = dimensionName1;
        listDimensions[1] = dimensionName2;
        listDimensions[2] = dimensionName3;
        listDimensions[3] = dimensionName4;
        listDimensions[4] = dimensionName5;
        listDimensions[5] = dimensionName6;
        listDimensions[6] = dimensionName7;

        if (SUCCEEDED(CreateIfxMeasureMetric(
            &hMetric, monitoringAccount, metricNamespace, metricName, 7, listDimensions, addDefaultDimension)))
        {
            return new MeasureMetric7D(hMetric);
        }
        else
        {
            return NULL;
        }
    }

    HRESULT LogValue(
        __in LONG64 rawData,
        __in_z LPCSTR dimensionValue1,
        __in_z LPCSTR dimensionValue2,
        __in_z LPCSTR dimensionValue3,
        __in_z LPCSTR dimensionValue4,
        __in_z LPCSTR dimensionValue5,
        __in_z LPCSTR dimensionValue6,
        __in_z LPCSTR dimensionValue7)
    {
        LPCSTR listDimensions[7];

        listDimensions[0] = dimensionValue1;
        listDimensions[1] = dimensionValue2;
        listDimensions[2] = dimensionValue3;
        listDimensions[3] = dimensionValue4;
        listDimensions[4] = dimensionValue5;
        listDimensions[5] = dimensionValue6;
        listDimensions[6] = dimensionValue7;

        return SetIfxMeasureMetric(_hMetric, rawData, 7, listDimensions);
    }

    HRESULT LogValue(
        __in ULONG64 timestampUtc,
        __in LONG64 rawData,
        __in_z LPCSTR dimensionValue1,
        __in_z LPCSTR dimensionValue2,
        __in_z LPCSTR dimensionValue3,
        __in_z LPCSTR dimensionValue4,
        __in_z LPCSTR dimensionValue5,
        __in_z LPCSTR dimensionValue6,
        __in_z LPCSTR dimensionValue7)
    {
        LPCSTR listDimensions[7];

        listDimensions[0] = dimensionValue1;
        listDimensions[1] = dimensionValue2;
        listDimensions[2] = dimensionValue3;
        listDimensions[3] = dimensionValue4;
        listDimensions[4] = dimensionValue5;
        listDimensions[5] = dimensionValue6;
        listDimensions[6] = dimensionValue7;

        return SetIfxMeasureMetricWithTimestamp(_hMetric, timestampUtc, rawData, 7, listDimensions);
    }
};

// The short-cut class for measure metric with 8 dimensions.
class MeasureMetric8D
{
private:
    HANDLE _hMetric;

    MeasureMetric8D(HANDLE hMetric)
    {
        _hMetric = hMetric;
    }

public:
    static MeasureMetric8D* Create(
        __in_z LPCSTR monitoringAccount,
        __in_z LPCSTR metricNamespace,
        __in_z LPCSTR metricName,
        __in_z LPCSTR dimensionName1,
        __in_z LPCSTR dimensionName2,
        __in_z LPCSTR dimensionName3,
        __in_z LPCSTR dimensionName4,
        __in_z LPCSTR dimensionName5,
        __in_z LPCSTR dimensionName6,
        __in_z LPCSTR dimensionName7,
        __in_z LPCSTR dimensionName8,
        __in bool addDefaultDimension = false)
    {
        HANDLE hMetric = NULL;
        LPCSTR listDimensions[8];

        listDimensions[0] = dimensionName1;
        listDimensions[1] = dimensionName2;
        listDimensions[2] = dimensionName3;
        listDimensions[3] = dimensionName4;
        listDimensions[4] = dimensionName5;
        listDimensions[5] = dimensionName6;
        listDimensions[6] = dimensionName7;
        listDimensions[7] = dimensionName8;

        if (SUCCEEDED(CreateIfxMeasureMetric(
            &hMetric, monitoringAccount, metricNamespace, metricName, 8, listDimensions, addDefaultDimension)))
        {
            return new MeasureMetric8D(hMetric);
        }
        else
        {
            return NULL;
        }
    }

    HRESULT LogValue(
        __in LONG64 rawData,
        __in_z LPCSTR dimensionValue1,
        __in_z LPCSTR dimensionValue2,
        __in_z LPCSTR dimensionValue3,
        __in_z LPCSTR dimensionValue4,
        __in_z LPCSTR dimensionValue5,
        __in_z LPCSTR dimensionValue6,
        __in_z LPCSTR dimensionValue7,
        __in_z LPCSTR dimensionValue8)
    {
        LPCSTR listDimensions[8];

        listDimensions[0] = dimensionValue1;
        listDimensions[1] = dimensionValue2;
        listDimensions[2] = dimensionValue3;
        listDimensions[3] = dimensionValue4;
        listDimensions[4] = dimensionValue5;
        listDimensions[5] = dimensionValue6;
        listDimensions[6] = dimensionValue7;
        listDimensions[7] = dimensionValue8;

        return SetIfxMeasureMetric(_hMetric, rawData, 8, listDimensions);
    }

    HRESULT LogValue(
        __in ULONG64 timestampUtc,
        __in LONG64 rawData,
        __in_z LPCSTR dimensionValue1,
        __in_z LPCSTR dimensionValue2,
        __in_z LPCSTR dimensionValue3,
        __in_z LPCSTR dimensionValue4,
        __in_z LPCSTR dimensionValue5,
        __in_z LPCSTR dimensionValue6,
        __in_z LPCSTR dimensionValue7,
        __in_z LPCSTR dimensionValue8)
    {
        LPCSTR listDimensions[8];

        listDimensions[0] = dimensionValue1;
        listDimensions[1] = dimensionValue2;
        listDimensions[2] = dimensionValue3;
        listDimensions[3] = dimensionValue4;
        listDimensions[4] = dimensionValue5;
        listDimensions[5] = dimensionValue6;
        listDimensions[6] = dimensionValue7;
        listDimensions[7] = dimensionValue8;

        return SetIfxMeasureMetricWithTimestamp(_hMetric, timestampUtc, rawData, 8, listDimensions);
    }
};

// The short-cut class for measure metric with 9 dimensions.
class MeasureMetric9D
{
private:
    HANDLE _hMetric;

    MeasureMetric9D(HANDLE hMetric)
    {
        _hMetric = hMetric;
    }

public:
    static MeasureMetric9D* Create(
        __in_z LPCSTR monitoringAccount,
        __in_z LPCSTR metricNamespace,
        __in_z LPCSTR metricName,
        __in_z LPCSTR dimensionName1,
        __in_z LPCSTR dimensionName2,
        __in_z LPCSTR dimensionName3,
        __in_z LPCSTR dimensionName4,
        __in_z LPCSTR dimensionName5,
        __in_z LPCSTR dimensionName6,
        __in_z LPCSTR dimensionName7,
        __in_z LPCSTR dimensionName8,
        __in_z LPCSTR dimensionName9,
        __in bool addDefaultDimension = false)
    {
        HANDLE hMetric = NULL;
        LPCSTR listDimensions[9];

        listDimensions[0] = dimensionName1;
        listDimensions[1] = dimensionName2;
        listDimensions[2] = dimensionName3;
        listDimensions[3] = dimensionName4;
        listDimensions[4] = dimensionName5;
        listDimensions[5] = dimensionName6;
        listDimensions[6] = dimensionName7;
        listDimensions[7] = dimensionName8;
        listDimensions[8] = dimensionName9;

        if (SUCCEEDED(CreateIfxMeasureMetric(
            &hMetric, monitoringAccount, metricNamespace, metricName, 9, listDimensions, addDefaultDimension)))
        {
            return new MeasureMetric9D(hMetric);
        }
        else
        {
            return NULL;
        }
    }

    HRESULT LogValue(
        __in LONG64 rawData,
        __in_z LPCSTR dimensionValue1,
        __in_z LPCSTR dimensionValue2,
        __in_z LPCSTR dimensionValue3,
        __in_z LPCSTR dimensionValue4,
        __in_z LPCSTR dimensionValue5,
        __in_z LPCSTR dimensionValue6,
        __in_z LPCSTR dimensionValue7,
        __in_z LPCSTR dimensionValue8,
        __in_z LPCSTR dimensionValue9)
    {
        LPCSTR listDimensions[9];

        listDimensions[0] = dimensionValue1;
        listDimensions[1] = dimensionValue2;
        listDimensions[2] = dimensionValue3;
        listDimensions[3] = dimensionValue4;
        listDimensions[4] = dimensionValue5;
        listDimensions[5] = dimensionValue6;
        listDimensions[6] = dimensionValue7;
        listDimensions[7] = dimensionValue8;
        listDimensions[8] = dimensionValue9;

        return SetIfxMeasureMetric(_hMetric, rawData, 9, listDimensions);
    }

    HRESULT LogValue(
        __in ULONG64 timestampUtc,
        __in LONG64 rawData,
        __in_z LPCSTR dimensionValue1,
        __in_z LPCSTR dimensionValue2,
        __in_z LPCSTR dimensionValue3,
        __in_z LPCSTR dimensionValue4,
        __in_z LPCSTR dimensionValue5,
        __in_z LPCSTR dimensionValue6,
        __in_z LPCSTR dimensionValue7,
        __in_z LPCSTR dimensionValue8,
        __in_z LPCSTR dimensionValue9)
    {
        LPCSTR listDimensions[9];

        listDimensions[0] = dimensionValue1;
        listDimensions[1] = dimensionValue2;
        listDimensions[2] = dimensionValue3;
        listDimensions[3] = dimensionValue4;
        listDimensions[4] = dimensionValue5;
        listDimensions[5] = dimensionValue6;
        listDimensions[6] = dimensionValue7;
        listDimensions[7] = dimensionValue8;
        listDimensions[8] = dimensionValue9;

        return SetIfxMeasureMetricWithTimestamp(_hMetric, timestampUtc, rawData, 9, listDimensions);
    }
};

// The short-cut class for measure metric with 10 dimensions, which is the allowed maximum number of dimensions.
class MeasureMetric10D
{
private:
    HANDLE _hMetric;

    MeasureMetric10D(HANDLE hMetric)
    {
        _hMetric = hMetric;
    }

public:
    static MeasureMetric10D* Create(
        __in_z LPCSTR monitoringAccount,
        __in_z LPCSTR metricNamespace,
        __in_z LPCSTR metricName,
        __in_z LPCSTR dimensionName1,
        __in_z LPCSTR dimensionName2,
        __in_z LPCSTR dimensionName3,
        __in_z LPCSTR dimensionName4,
        __in_z LPCSTR dimensionName5,
        __in_z LPCSTR dimensionName6,
        __in_z LPCSTR dimensionName7,
        __in_z LPCSTR dimensionName8,
        __in_z LPCSTR dimensionName9,
        __in_z LPCSTR dimensionName10,
        __in bool addDefaultDimension = false)
    {
        HANDLE hMetric = NULL;
        LPCSTR listDimensions[10];

        listDimensions[0] = dimensionName1;
        listDimensions[1] = dimensionName2;
        listDimensions[2] = dimensionName3;
        listDimensions[3] = dimensionName4;
        listDimensions[4] = dimensionName5;
        listDimensions[5] = dimensionName6;
        listDimensions[6] = dimensionName7;
        listDimensions[7] = dimensionName8;
        listDimensions[8] = dimensionName9;
        listDimensions[9] = dimensionName10;

        if (SUCCEEDED(CreateIfxMeasureMetric(
            &hMetric, monitoringAccount, metricNamespace, metricName, 10, listDimensions, addDefaultDimension)))
        {
            return new MeasureMetric10D(hMetric);
        }
        else
        {
            return NULL;
        }
    }

    HRESULT LogValue(
        __in LONG64 rawData,
        __in_z LPCSTR dimensionValue1,
        __in_z LPCSTR dimensionValue2,
        __in_z LPCSTR dimensionValue3,
        __in_z LPCSTR dimensionValue4,
        __in_z LPCSTR dimensionValue5,
        __in_z LPCSTR dimensionValue6,
        __in_z LPCSTR dimensionValue7,
        __in_z LPCSTR dimensionValue8,
        __in_z LPCSTR dimensionValue9,
        __in_z LPCSTR dimensionValue10)
    {
        LPCSTR listDimensions[10];

        listDimensions[0] = dimensionValue1;
        listDimensions[1] = dimensionValue2;
        listDimensions[2] = dimensionValue3;
        listDimensions[3] = dimensionValue4;
        listDimensions[4] = dimensionValue5;
        listDimensions[5] = dimensionValue6;
        listDimensions[6] = dimensionValue7;
        listDimensions[7] = dimensionValue8;
        listDimensions[8] = dimensionValue9;
        listDimensions[9] = dimensionValue10;

        return SetIfxMeasureMetric(_hMetric, rawData, 10, listDimensions);
    }

    HRESULT LogValue(
        __in ULONG64 timestampUtc,
        __in LONG64 rawData,
        __in_z LPCSTR dimensionValue1,
        __in_z LPCSTR dimensionValue2,
        __in_z LPCSTR dimensionValue3,
        __in_z LPCSTR dimensionValue4,
        __in_z LPCSTR dimensionValue5,
        __in_z LPCSTR dimensionValue6,
        __in_z LPCSTR dimensionValue7,
        __in_z LPCSTR dimensionValue8,
        __in_z LPCSTR dimensionValue9,
        __in_z LPCSTR dimensionValue10)
    {
        LPCSTR listDimensions[10];

        listDimensions[0] = dimensionValue1;
        listDimensions[1] = dimensionValue2;
        listDimensions[2] = dimensionValue3;
        listDimensions[3] = dimensionValue4;
        listDimensions[4] = dimensionValue5;
        listDimensions[5] = dimensionValue6;
        listDimensions[6] = dimensionValue7;
        listDimensions[7] = dimensionValue8;
        listDimensions[8] = dimensionValue9;
        listDimensions[9] = dimensionValue10;

        return SetIfxMeasureMetricWithTimestamp(_hMetric, timestampUtc, rawData, 10, listDimensions);
    }
};
