//
// This header file defines common definitions that are used by clients and PAL.
// It is included by both rtcpal_apis_video.h and VirtualDeviceCommon.h
//

#ifndef __MEDIAVIDEOFORMATCOMMON_H__
#define __MEDIAVIDEOFORMATCOMMON_H__

// -----------------------------------------------------------------------------
// MediaVideoRotation_e enumeration
//
// Enumeration of supported rotation angles, including a wildcard value that
// is only used to represent that a component can take any rotation factor
// and execute its own rotation.
// -----------------------------------------------------------------------------

typedef enum _MediaVideoRotation_e
{
    MediaVideoRotation_Any             = -1,
    MediaVideoRotation_0               = 0,
    MediaVideoRotation_90Clockwise     = 90,
    MediaVideoRotation_180             = 180,
    MediaVideoRotation_270Clockwise    = 270,
} MediaVideoRotation_e;


// ----------------------------------------------------------------------------
// Video Format
//
// Type used to describe a video format (uncompressed or CBR). This type can be 
// used in enumeration of camera capabilities, and for getting and setting the 
// video format for capture or preview.
//-----------------------------------------------------------------------------
typedef struct _MediaVideoFormat_t
{
    // Output type
    uint32_t fccVideoFormat;
    
    // Resolution
    uint16_t usFrameWidth;
    uint16_t usFrameHeight;
    
    // Rotation
    MediaVideoRotation_e eRotation;
    
    // Supplemental information
    int32_t  lStride;
    uint32_t ulBitsPerPixel;
    
    // Frame rate
    float fltFrameRate;
    
    // Bit rate
    uint32_t ulBitRate;
    
} MediaVideoFormat_t;


#define SCREEN_MAX_INFOCOUNT   32
#define SCREEN_MAX_STRINGMSG   64

//
// Meta data info type for screen shared content
//
typedef enum _ScreenInfoMode_t {
    ScreenInfo_Unavailable = 0,
    ScreenInfo_DirtyRects,     // Use dirty/move rects.
    ScreenInfo_WindowsLayout   // Use windows layout.
} ScreenInfoMode_t;

//
// Move rectangles for screen captured content.
//
typedef struct _ScreenMoveRect_t {
  POINT SourcePoint;                // SourcePoint - The point in the previous frame from which the rect is moved.
  RECT DestinationRect;             // DestinationRect - the rect in the current frame.
} ScreenMoveRect_t;

//
// Window layout information
// Notes:
//  The scraper should output layout info for all visible windows and indicate which ones are being shared.
//
typedef struct _ScreenWindowLayout_t {
    LPVOID id;                                      // HWND of the window.
    wchar_t wszProcessName[SCREEN_MAX_STRINGMSG];   // Parent process's name string.
    RECT Rect;                                      // Size and location of the window.
    int bShared;                                    // Whether the window is shared.
} ScreenWindowLayout_t;

// 
// Meta data for screen captured content.
// Notes: 
// 1. Video pipeline assumes move rects are not covered in dirty rects and will always process both.
// 2. Windows layout info and dirty/move rects info are currently assumed to be mutually exclusive. 
//    Caller should use ScreenInfoMode to indicate which one should be used.
//
typedef struct _ScreenInfo_t {
    ScreenInfoMode_t Mode;
    uint32_t         iFrameOrderCount;                              // Frame count in temporal order (restart from 0 if 0xFFFFFFFF is reached).

    // Dirty/move rects.
    uint32_t         iNumDirtyRects;                                // Number of dirty rects.
    RECT             DirtyRects[SCREEN_MAX_INFOCOUNT];              // Dirty rectangles.
    uint32_t         iNumMoveRects;                                 // Number of move rects.
    ScreenMoveRect_t MoveRects[SCREEN_MAX_INFOCOUNT];               // Move rectangles.
    uint32_t         iNumMouseRects;                                // Number of mouse rects (up to 2).
    RECT             MouseRects[2];                                 // Mouse rectangles (in current and previous frame).
    
    // Windows layout.
    uint32_t              iNumWinLayouts;                           // Number of windows.
    ScreenWindowLayout_t  WindowsLayout[SCREEN_MAX_INFOCOUNT];      // Windows layouts sorted in z-order (first being the foremost window).
} ScreenInfo_t;


#endif // __MEDIAVIDEOFORMATCOMMON_H__
