#pragma once
/* --

Copyright (c) Microsoft Corporation. All rights reserved.

Module Name:

    VirtualDeviceCommon.h

Abstract:

    Virtual Device Comomon APIs.

--*/
#ifndef __VIRTUAL_DEVICE_COMMON_H
#define __VIRTUAL_DEVICE_COMMON_H

#ifdef __cplusplus

#include "MediaVideoFormatCommon.h"

// -----------------------------------------------------------------------------
// Uncompressed (Raw) Video Frame
//
// Structure used for delivery of uncompressed video frames through
// the preview.
// -----------------------------------------------------------------------------
typedef struct _VirtualVideoRawFrame_t
{          
    // Uncompressed format
    uint32_t fccVideoFormat;
    
    // Coded Resolution
    uint16_t usFrameWidth;
    uint16_t usFrameHeight;

    // Rotation
    MediaVideoRotation_e eRotation;
    
    // Frame data
    uint8_t* pbFrameData;
    uint32_t ulDataLength;
    
    // Supplemental information
    int32_t  lStride;
    uint32_t ulBitsPerPixel;
    
    // Time stamp
    uint64_t ullTimeStamp;
} VirtualVideoRawFrame_t;

//
// Interface of Virtual Frame
//
class IVirtualFrame
{
 public:
    // Lifetime management
    virtual ULONG RTCAPI AddRef() RTCPAL_NOTHROW = 0;
    virtual ULONG RTCAPI Release() RTCPAL_NOTHROW = 0;

    // Get methods
    virtual VirtualVideoRawFrame_t * GetFrame() = 0;
    virtual ScreenInfo_t * GetScreenInfo() = 0;
};

// Set and Get Attribute API for virtual source devices
typedef enum _IVirtualDeviceAttribute
{
    VDFrameRate = 0,
    VDMinFrameWidth,    
} IVirtualDeviceAttribute;

struct IVirtualDeviceArrayInfo {
    size_t    allocLength;  // allocated array length 
    size_t    actualLength; // actual array length
    union {
        void* p;
        char* strData;
        int*  intData;
        RECT* rectData;
    } p;
};

typedef struct _IVirtualDeviceVariant
{
    union{
        bool                        boolData;
        int                         intData;
        float                       floatData;
        void*                       pointerData;
        IVirtualDeviceArrayInfo*    arrayData;
        void*                       externData;  
    } d;    
} IVirtualDeviceVariant;

//
// Interface for Virtual Video Source provider.
// Normally client does the implementation of provider.
//
class IVirtualVideoSourceProvider
{
 public:
    // Lifetime management
//    STDMETHOD_(ULONG, AddRef)() RTCPAL_NOTHROW = 0;    
//    STDMETHOD_(ULONG, Release)() RTCPAL_NOTHROW = 0;

    virtual ULONG RTCAPI AddRef() RTCPAL_NOTHROW = 0;    
    virtual ULONG RTCAPI Release() RTCPAL_NOTHROW = 0;


    virtual HRESULT Start() = 0;
    virtual HRESULT Stop() = 0;
    virtual HRESULT GetVideoFormatSupportedList(__out MediaVideoFormat_t * pFormats, __out uint32_t* pulCount) = 0;
    virtual HRESULT GetVideoFormat(__out MediaVideoFormat_t* pFormatOut) = 0;
    virtual HRESULT SetVideoFormat(__in const MediaVideoFormat_t * pFormat) = 0;
    virtual HRESULT DeliverEvent(__in void * pEvent) = 0;
    virtual HRESULT GetSourceName(__out wchar_t ** pSourceName) = 0;    
    virtual HRESULT GetValue(__in const IVirtualDeviceAttribute attribute, __out IVirtualDeviceVariant *value) {UNREFERENCED_PARAMETER(attribute); UNREFERENCED_PARAMETER(value); return E_NOTIMPL;};
    virtual HRESULT SetValue(__in const IVirtualDeviceAttribute attribute, __in  IVirtualDeviceVariant *value) {UNREFERENCED_PARAMETER(attribute); UNREFERENCED_PARAMETER(value); return E_NOTIMPL;}
    virtual HRESULT SetFramerateDynamic (__in float fNewRate) { UNREFERENCED_PARAMETER(fNewRate); return E_NOTIMPL; }    
} ;


//
// Interface for Virtual Video Source provider.
// Normally PAL does the implementation.
// 
class IVirtualVideoSink
{
public:
    // Lifetime management

    virtual HRESULT DeliverFrame(__in IVirtualFrame* pFrame) = 0;
    virtual ULONG GetSourceID() = 0;
    virtual HRESULT DeliverEvent(__in void * pEvent) = 0;
} ;


typedef enum _PalMediaExtensionType
{
    PMET_DefaultType,
    PMET_VirtualVideoSource,
    PMET_ORTC,
    PMET_ORTC_Test,
    PMET_MaxType
} PalMediaExtensionType;


typedef enum _VideoSourceProviderType
{
    VSPT_DefaultType,
    VSPT_ScreenScraper,
    VSPT_MaxType
} VideoSourceProviderType;

//
// Interface for Virtual Video Source extentsion.
// This is the entry point for client to reach PAL and create virtual video devices.
// 
class IVirtualVideoSourceExtension
{
public:
    virtual ~IVirtualVideoSourceExtension() {}
    virtual ULONG RTCAPI AddRef() RTCPAL_NOTHROW = 0;    
    virtual ULONG RTCAPI Release() RTCPAL_NOTHROW = 0;

    virtual HRESULT RegisterVideoSourceProvider
        (__in  VideoSourceProviderType type, 
         __in  IVirtualVideoSourceProvider * pSourceProvider, 
         __out IVirtualVideoSink ** pSink) = 0;

    virtual HRESULT UnregisterVideoSourceProvider
        (__in IVirtualVideoSink *pSink) = 0;
};

#endif

#endif // __VIRTUAL_DEVICE_COMMON_H
