#pragma once

#ifndef _CONSUMERRT_H__
#define _CONSUMERRT_H__

#include <rtcpal.h>
#include <vector>
#include <string>
#include <unordered_map>
#include <memory>
#include <mutex>

#include "winport.h"

namespace etw
{


class IOCache {
public:
    IOCache() {}
    ~IOCache();

    /// <summary>
    /// Get the global cache file offset, which is the same value
    /// for the etl file's offset.
    /// </summary>
    ULONGLONG GetFileOffset() const {
        return m_filledOffset;
    }

    /// <summary>
    /// Save data to the cache buffers. It will expand cache capacity if needed.
    /// Return error code.
    /// </summary>
    ULONG SaveData(void* srcdata, ULONG dataSize);

    /// <summary>
    /// Read data from cache into destdata.
    /// Return true if success, false if any error.
    /// </summary>
    bool ReadData(void* destdata,
        DWORD NumberOfBytesToRead,
        LPDWORD NumberOfBytesRead,
        ULONGLONG ReadPosition);

    /// <summary>
    /// Free the full memory buffer which holds a given offset position.
    /// </summary>
    void FreeBuffer(ULONGLONG offset);

    /// <summary>
    /// Create a new event fd if not exists. If exists, just return it.
    /// Return the event fd, or -1 if any error.
    /// </summary>
    int CreateEventFDIfNotExists();

    /// <summary>
    /// Write something to event fd. This is used to signal the event fd on
    /// the read() side to abort further reading.
    /// Also mark the consumer to be in stopping state.
    /// Return true if succeed, return false if any error.
    /// </summary>
    bool SignalReaderToClose();

    /// <summary>
    /// close the event fd.
    /// </summary>
    void CloseEventFD();

    /// <summary>
    /// Mark consumer to be in a stopping state. No further reading will be performed
    /// once it is in stopping state.
    /// </summary>
    void MarkAsStopping() { m_isStopping = true; }

    /// <summary>
    /// Return true if consumer is in Stopping state, return false if not.
    /// </summary>
    bool IsStopping() const { return m_isStopping; }

private:
    /// <summary>
    /// Create a new void* buffer in number of bytes.
    /// Return the index of buffer in the list, or -1 if any error.
    /// </summary>
    int CreateBuffer(ULONG uSize);

    /// <summary>
    /// Create new buffers if needed in global cache to hold new data size.
    /// Return error code.
    /// </summary>
    ULONG CreateBuffers(ULONG nNewBytes);

    /// <summary>
    /// Save the srcdata to the global cache. It assumes
    /// the global cache is large enough to hold the data.
    /// return error code.
    /// </summary>
    ULONG FillBuffers(void *srcdata, ULONG nbytes);

    /// A list of pointers. Each pointer is either nullptr
    /// or points to an allocated data buffer.
    /// The memory should be freed using free().
    std::vector<void*> m_bufList;

    /// Buffer size in number of bytes. Each allocated buffer
    /// in m_bufList uses this size.
    const ULONG BUFFER_SIZE = DEFAULT_LOG_BUFFER_SIZE;

    /// data offset in number of bytes
    ULONGLONG m_filledOffset = 0;

    /// A event fd used to send event to consumer side process
    int m_eventfd = -1;

    /// If true, the consumer is in stopping state. It should be
    /// terminated and no more data read should be done.
    bool m_isStopping = false;
};

class ConsumerRT {
public:
    static ConsumerRT& GetInstance();
    static void DeleteInstance();

    ~ConsumerRT()
    {
        m_cacheTable.clear();
    }

    std::shared_ptr<IOCache> GetIOCache(const std::wstring & guid);
    std::shared_ptr<IOCache> GetIOCache(LPCWSTR guid);
    std::shared_ptr<IOCache> GetIOCache(const GUID& guid);

    void ClearCache() {
        m_cacheTable.clear();
    }

    /// <summary>
    /// Notify consumer process to stop reading
    /// <summary>
    void StopTrace(const GUID& guid)
    {
        auto iocache = GetIOCache(guid);
        iocache->SignalReaderToClose();
        iocache->CloseEventFD();
    }

    /// <summary>
    /// disable copy and move constructors
    /// </summary>
    ConsumerRT(ConsumerRT&& c) = delete;
    ConsumerRT& operator=(ConsumerRT&& c) = delete;

    ConsumerRT(const ConsumerRT& c) = delete;
    ConsumerRT& operator=(const ConsumerRT& c) = delete;

private:
    ConsumerRT() {}

    std::unordered_map<std::wstring, std::shared_ptr<IOCache>> m_cacheTable;

    // Use static object allocated dynamically so that its lifetime can last.
    // If the object is a static local object, it will be dellocated before
    // attribute(destructor) functions are called, which will cause issue for
    // functions called in the attribute(destructor) functions.
    static ConsumerRT* s_instance;

    // used to make sure it is thread-safe.
    static std::once_flag s_getFlag;
    static std::once_flag s_delFlag;
};


}; // namespace etw

#endif // _CONSUMERRT_H__
