/** \file provider.h

    This file defines the Provider class.
 */
#pragma once

#include "std.h"

#include "rtcpal.h"
#include "rtcpal_thread_safe.h"
#include "evntrace.h"
#include "evntprov.h"

#include "util.h"
#include <set>
#include <mutex>
    
namespace etw
{
class Session;

/** This class defines the provider object. Provider is the producer of the event.
    A session can enable multiple providers. If the provider is enabled by a session,
    the provider will output the event to the session's buffer.
 */
class Provider
{
public:
    Provider(
        const GUID& guid, 
        PENABLECALLBACK callback, 
        void* context);
        
        Provider(
            const GUID& guid, 
            WMIDPREQUEST wppCallback,
            void* context);
            
    ~Provider() noexcept;
    
    ULONG WriteEvent(
        const PCEVENT_DESCRIPTOR pEventDescriptor,
        ULONG uUserDataCount,
        const PEVENT_DATA_DESCRIPTOR pUserData,
        USHORT eventHeaderFlag = 0
        ) noexcept;
    
    GUID GetGuid() const { return m_guid; }

    static void EnableInSession(
            const GUID& guid, 
            bool fEnable, 
            Session* pSession,
            ULONGLONG uEnableFlag,
            ULONG uEnableLevel);

    static void DeleteProviders();

private:
    void _EnableInSession(
            bool fEnable, 
            Session* pSession, 
            ULONGLONG uEnableFlag, 
            ULONG uEnableLevel);

    void _RegisterProvider(const GUID& guid);

    static RtcPalThreadSafe<set<Provider*>>* GetProviders();

    /// Static member which will hold all registed Providers.
    static RtcPalThreadSafe<set<Provider*>>* s_providers;

    static std::once_flag s_getFlag;
    static std::once_flag s_delFlag;

    /// Hold all sessions which enables this provider.
    RtcPalThreadSafe<vector<Session*> > m_enableSessions;
    
    /// The provider's guid.
    GUID m_guid;
    
    /// The enable/disable callback for this provider.
    PENABLECALLBACK m_callback;

    /// The enable/disable callback for this WPP provider.
    WMIDPREQUEST m_WppCallback;
    
    /// The enable/disable callback context.
    void* m_callbackContext;
};
}
