#pragma once

#ifndef __PROVIDERRT_H__
#define __PROVIDERRT_H__

#include <rtcpal.h>
#include <evntrace.h>
#include "winport.h"
#include <string>
#include <atomic>
#include <map>
#include <memory>
#include <mutex>

namespace etw
{
    class ShmManager;

    struct ShmMgrItem {
        std::unique_ptr<ShmManager> m_shmmgr;
        int32_t m_index;      // Valid ps index should >= 1; Invalid ps index <= 0.

        ShmMgrItem();

        ShmMgrItem(std::unique_ptr<ShmManager>&& shmmgr, int32_t index);

        ShmMgrItem(ShmMgrItem&& other);

        ~ShmMgrItem();
    };


/// This class implements ETW Realtime provider support.
/// Each process can run multiple providers, with each provider has its unique GUID.
/// Each provider GUID can be shared by multiple processes.
class ProviderRT {
    using ShmMgrIter = std::map<GUID, ShmMgrItem>::iterator;
    using EventTracePtr = std::unique_ptr<EVENT_TRACE_PROPERTIES, decltype(free)*>;

public:
    // get singleton instance as reference
    static ProviderRT& GetInstance();

    // Delete the instance and delete it once if in multi-threads.
    static void DeleteInstance();

    ~ProviderRT() {}

    /// <summary>
    /// delete copy and move constructors
    /// </summary>
    ProviderRT(ProviderRT&& p) = delete;
    ProviderRT& operator=(ProviderRT&& p) = delete;

    ProviderRT(const ProviderRT& p) = delete;
    ProviderRT& operator=(const ProviderRT& p) = delete;

    /// <summary>
    /// Start provider given its GUID.
    /// If the provider is not enabled yet, just register it and
    /// then return ERROR_SUCCESS.
    /// </summary>
    /// Return ERROR_SUCCESS if success, error code if any error.
    ULONG StartProvider(const GUID& guid);

    /// <summary>
    /// Stop the provider given its GUID.
    /// Also remove the provider from global shared memory.
    /// <summary>
    ULONG StopProvider(const GUID& guid);

    /// <summary>
    /// Return ERROR_SUCCESS if success, error code if any error.
    /// </summary>
    ULONG Register(const GUID& guid);

    /// <summary>
    /// Return ERROR_SUCCESS if success, error code if any error.
    /// </summary>
    ULONG Unregister(const GUID& guid);

    /// <summary>
    /// Set signal handlers
    /// </summary>
    void SetSignalCatchers() const;

    void SetCallback(const GUID& guid, WMIDPREQUEST callback)
    {
        m_callbackTable[guid] = callback;
    }

    WMIDPREQUEST GetCallback(const GUID& guid) const
    {
        auto iter = m_callbackTable.find(guid);
        if (iter != m_callbackTable.end()) {
            return iter->second;
        }
        return nullptr;
    }

    TRACEHANDLE GetSessionHandle() const
    {
        return m_sessionHandle;
    }

private:
    ProviderRT() {}

    /// <summary>
    /// Flip all provider processes:
    /// - If the provider GUID is enabled, start the provider.
    /// - If the provider GUID is disabled, stop the provider.
    /// </summary>
    /// Return true if success, return false if any error.
    bool FlipAllProviders();

    /// <summary>
    /// Pause all the providers. The provider process entries are not
    /// removed from global shared memory. So that they can be started
    /// at a later time if needed.
    /// <summary>
    /// Return trueif success, return false if any error.
    bool PauseAllProviders();

    /// <summary>
    /// Remove a provider information item from global shared memory.
    /// <summary>
    /// Return ERROR_SUCCESS if success, return error code if any error.
    ULONG Unregister(ShmMgrIter iter);

    /// <summary>
    /// Define the signal handler which handles the special signal.
    /// The special signal is used by controller to start/stop providers.
    /// </summary>
    static void CatchSignal(int signo);

    /// <summary>
    /// Handle signal SIGPIPE.
    /// This signal can be received in I/O of FIFO. When read end is closed,
    /// write-end process will receive this signal.
    /// </summary>
    static void CatchSigPIPE(int signo);

    /// <summary>
    /// Create a new EVENT_TRACE_PROPERTIES object to be used by trace session.
    /// Return the EVENT_TRACE_PROPERTIES pointer or nullptr if any error.
    /// </summary>
    EventTracePtr CreateEvtProp(LPCTSTR sessionName, const GUID & guid) const;

    /// <summary>
    /// Start and enable a new trace session.
    /// When session is started, it will be enabled.
    /// If enabling failed, it will be stoppped and session handle be set to be 0.
    /// Return error code, or ERROR_SUCCESS if no error.
    /// </summary>
    ULONG StartTraceRT(const GUID & guid);

    /// <summary>
    /// Stop the trace session if it exists.
    /// Return error code, or ERROR_SUCCESS if no error.
    /// </summary>
    ULONG StopTraceRT(const GUID & guid);

    /// <summary>
    /// Create the configuration dir if not exists yet.
    /// Return true for success, false for any error.
    /// </summary>
    bool CreateConfigDir() const;

    /// <summary>
    /// Remove zero or more files in the config dir.
    /// Return true for success, false for error.
    /// </summary>
    bool CleanUpConfigDir() const;

private:
    // Use static object allocated dynamically so that its lifetime can last.
    // If the object is a static local object, it will be dellocated before
    // attribute(destructor) functions are called, which will cause issue for
    // functions called in the attribute(destructor) functions.
    static ProviderRT* s_instance;

    // used to make sure it is thread-safe.
    static std::once_flag s_getFlag;
    static std::once_flag s_delFlag;

    /// Cache multiple providers with their GUIDs.
    std::map<GUID, ShmMgrItem> m_shmTable;

    /// Cache multiple providers' callbacks with GUIDs.
    std::map<GUID, WMIDPREQUEST> m_callbackTable;

    /// Handle of trace session used by provider to write events.
    std::atomic<TRACEHANDLE> m_sessionHandle{0};
};



} // namespace etw

#endif // __PROVIDERRT_H__
