/** \file session.h

    This file defines the Session class.
 */
#pragma once

#include "std.h"

#include "rtcpal.h"
#include "evntprov.h"
#include "evntrace.h"

#include "util.h"
#include "winport.h"
#include <atomic>
#include <string>

struct ETW_LOGFILE_HEADER;
namespace etw
{
class Session
{
    static ETW_BUFFER_HEADER* _CreateBuffer(ULONG uSize);

    static void _DeleteBuffer(ETW_BUFFER_HEADER* pBuffer) noexcept;

public:
    Session(const wchar_t* name, const EVENT_TRACE_PROPERTIES& properties);
    ~Session() throw();
    
    char* RequireBuffer(ULONG uRequireSize, ETW_BUFFER_HEADER** ppBufferHeader, LARGE_INTEGER* pTimeStamp) throw();
    void ReleaseBuffer(ETW_BUFFER_HEADER* pBufferHeader) noexcept;

    LONG IncrementSeqenceNumber() {return InterlockedIncrement(&m_lSeqNum);};
    LONG IncrementEventsLost() {return InterlockedIncrement(&m_lEventsLost);};

    void FlushBuffer() 
    {
        m_fRequireFlushFile = true;
        SetEvent(m_hFlushEvent.get());
    };
        
    ULONG GetEnableLevel() {return m_uEnableLevel;};
    void SetEnableLevel(ULONG uEnableLevel) {m_uEnableLevel = uEnableLevel;};
    
    ULONG GetEnableFlags() {return m_uEnableFlags;};
    void SetEnableFlags(ULONG uEnableFlags) {m_uEnableFlags = uEnableFlags;};

    void FillProperties(PEVENT_TRACE_PROPERTIES properties);

    void EnableProvider(bool fEnable, const GUID& guid);
    
    int GetSessionError() const { return m_sessionError; }

private:
    static uint32_t RTCAPI _LogThread(void* parameter);
    void _LogThread();
    
    ULONG _CaculateBufferSize(const EVENT_TRACE_PROPERTIES& properties);
    ETW_BUFFER_HEADER* _GetFreeBuffer();                    // can return NULL if no buffer is avaliable
    void _EnqueueFullBuffer(ETW_BUFFER_HEADER* pBuffer);    // insert a buffer in the full buffer queue
    ULONG _FreeBuffers(ETW_BUFFER_HEADER* pBuffer);         // return the number of buffers freed
    void _InitBuffers(ULONG uBuffers, ULONG uBufferSize);
    void _FlushBuffer(bool bFlushAll=false);
    void _ResetBuffer(ETW_BUFFER_HEADER& buffer);
    void _InitLogFile(const wchar_t* name, const EVENT_TRACE_PROPERTIES& properties);
    void _FinalizeLogFile();
    void _StartLogThread();
    void _StopLogThread();
    void _DisableAllProviders();
    
    void _AcquireLock()
    {
        (void)EnterCriticalSection(&m_lock);
    }
    
    void _ReleaseLock()
    {
        (void)LeaveCriticalSection(&m_lock);
    }

    /// Hold the handle of the logger thread.
    RtcPalThreadHandle m_hLogThread;
    
    /// Hold the handle of flush event.
    auto_handle m_hFlushEvent;
    
    /// Hold the handle of the log file.
    auto_handle m_hLogFile;
    
    /// Hold the pointer of the ETW_LOGFILE_HEADER.
    auto_free<ETW_LOGFILE_HEADER> m_pLogHeader;

    /// The session's current buffer
    ETW_BUFFER_HEADER * volatile m_pCurrBuffer;
    
    ETW_BUFFER_HEADER* m_pFreeBuffersHead;

    /// Full buffer list head
    ETW_BUFFER_HEADER* m_pFullBuffersHead;

    /// Full buffer list tail
    ETW_BUFFER_HEADER* m_pFullBuffersTail;

    /// A lock for the session.
    CRITICAL_SECTION m_lock;

    /// Free buffer list head
    /// Hold the buffer size
    ULONG m_uBufferSize;

    /// Hold the total buffer Count
    ULONG m_uBuffers;

    /// Hold the free buffer Count
    ULONG m_uFreeBuffers;

    /// Hold the min buffer count
    ULONG m_uMinBuffers;

    /// Hold the max buffer count
    ULONG m_uMaxBuffers;
    
    /// Hold the current file size
    ULONG m_uCurrentFileSize;
    
    /// Hold the max file size
    ULONG m_uMaxFileSize;
    
    /// If the value is true, means the logger thread should be exited.
    volatile bool m_fRequireExit;

    /// If the value is true, flush the log file.
    volatile bool m_fRequireFlushFile;

    /// Sequence number
    volatile LONG m_lSeqNum;

    /// Events lost
    volatile LONG m_lEventsLost;

    /// The enable level
    ULONG m_uEnableLevel;

    /// The enable flags
    ULONG m_uEnableFlags;

    /// The clock type
    ULONG m_uClockType;

    /// Hold all providers (GUID) which are enabled in this session.
    RtcPalThreadSafe<vector<GUID> > m_enabledProviders;
    
    std::atomic<int> m_sessionError;

    bool m_addLogHeader;

    const uint32_t RT_RETRY_TIMEOUT_MS = 5000; // in milli-seconds
    const uint32_t RT_RETRY_INTERVAL_MS = 100; // in milli-seconds

    /// Flush timer in seconds
    DWORD m_flushTimer;
    /// RealTime ETW or not
    bool m_realTimeMode;
};
}
