#pragma once

#ifndef __SHMMANAGER_H__
#define __SHMMANAGER_H__

#include <string>
#include <atomic>
#include <cassert>
#include <functional>
#include <vector>
#include <unordered_map>

#include <boost/interprocess/mapped_region.hpp>
#include <boost/interprocess/sync/named_mutex.hpp>
#include <boost/interprocess/sync/scoped_lock.hpp>

#include "etwrtcommon.h"

namespace etw {


class ShmManager {
    using BoostRegion = boost::interprocess::mapped_region;
    using BoostMutex = boost::interprocess::named_mutex;
    using BoostLock = boost::interprocess::scoped_lock<boost::interprocess::named_mutex>;

public:
    ShmManager(const std::string & name = "");

    ~ShmManager() {}

    ShmManager(const ShmManager& other) = delete;
    ShmManager(ShmManager&& other) = delete;

    ShmManager& operator=(const ShmManager& other) = delete;
    ShmManager& operator=(ShmManager&& other) = delete;

    ProviderPSInfo* OpenWithReadOnly();
    ProviderPSInfo* OpenWithRW();
    ProviderPSInfo* CreateWithRWIfNotExist();

    /// Given pid and check whether it exists in shm or not.
    /// this function assumes the shm is already opened. If shm
    /// is not opened when calling this function, it will return false.
    /// Return true if found, false if not.
    bool FindPs(int32_t pid) const;

    /// Save current process's information to shm.
    /// NOTE: this function will create shm if not exist yet. It
    /// will then open the shm if not opened yet.
    /// Return shm slot Index where the information is saved.
    int32_t SavePsInfo();

    /// Clear the process information from shm at given index location.
    /// NOTE: Open shm if not opened yet. It won't create shm.
    /// Return true if cleared successfully, return false if not cleared.
    bool ClearPsInfo(int32_t psIndex);

    /// Enable the provider, then apply given function on each process.
    /// If the pid is invalid or function fails, the process entry
    /// will be removed from shm.
    /// NOTE: this function will create shm if not exist yet. It
    /// will then open the shm if not opened yet.
    /// Return true if success, false if any error.
    bool EnableAll(const std::function<bool(int32_t pid)>& fn);

    /// Disable the provider, then apply given function on each process.
    /// If the pid is invalid or function fails, the process entry
    /// will be removed from shm.
    /// NOTE: this function won't create shm. It will open shm if not openned yet.
    /// Return true if success, false if any error.
    bool DisableAll(const std::function<bool(int32_t pid)>& fn);

    bool IsTraceEnabled() const {
        if (m_addr) {
            BoostLock lock(m_bmutex);
            return (1 == m_addr[0].isEnabled);
        }
        return false;
    }

    int32_t GetMaxItems() const {
        if (m_addr) {
            BoostLock lock(m_bmutex);
            return m_addr[0].maxIndex;
        }
        return -1;
    }

private:
    ProviderPSInfo* GetShmDataArray(bool openOnly, bool readOnly);

    /// Clear shm process entry with given index. If the index equals to the maxIndex,
    /// decrease maxIndex value.
    /// return the new maxIndex if it is decreased, or -1 if not changed.
    int32_t ClearPsEntry(int32_t psIndex);

    /// Apply given function pointer on each process entry in shm.
    /// For each entry where function call fails, the entry will be cleared from shm.
    /// The psTickTable contains a table of <pid, process starttime in ticks>. It is
    /// used to check whether a process is valid or not. The table should be constructed
    /// outside of the mutex lock to minimize lock time.
    ///
    /// NOTE: this function must be called with m_bmutex be locked by caller function.
    ///
    /// Return list of index that's cleared from shm.
    std::vector<int32_t> ForeachPS(const std::function<bool(int32_t pid)>& fn,
        const std::unordered_map<int32_t, int64_t>& psTickTable);

    /// Query the process's start time and save it in a table. Because ps startTime query
    /// involves file I/O, this function should be run before the mutex lock so that the 
    /// mutex lock time can be shorter.
    /// return the cache table with: key=pid, value=ps_startTime_In_Ticks.
    std::unordered_map<int32_t, int64_t> GetPsStartTicks(bool stopAtFirstInvalid);

    /// Check whether a process is valid or not in the shm entry.
    /// It will search the cache table first, if not found, then read process startTime from 
    /// /proc file.
    /// Return true if valid, false if not.
    bool IsPidValid(const std::unordered_map<int32_t, int64_t> & cacheTable, const ProviderPSInfo& ps);


private:
    std::string m_name;
    ProviderPSInfo* m_addr;

    // mapped region must keep alive to make sure the address is valid.
    // it is moveable but not copyable.
    BoostRegion m_region;

    // To store previous umask value.
    // NOTE: it must be declared before m_bmutex because m_bmutex
    // initialization will use it.
    mode_t m_prevUmask;
    mutable BoostMutex m_bmutex;
};

} // namespace etw

#endif // __SHMMANAGER_H__
