/** \file util.h

    Defines some helpper functions.
 */
#pragma once

#ifndef __UTIL_H__
#define __UTIL_H__

#include "std.h"
#include "rtcpal.h"
#include "error.h"

namespace etw
{
template<typename T, typename FreeOp>
class UniquePtr
{
    FreeOp m_deleter;
    T* m_p;
    
    UniquePtr(const UniquePtr& p) {}
    UniquePtr& operator=(UniquePtr& p) {};
    
public:
    explicit UniquePtr(T* p = NULL) : m_p(p) {}
    
    ~UniquePtr()
    {
        if (get())
        {
            m_deleter(release());
        }
    }
    
    void reset(T* p)
    {
        if (m_p)
        {
            m_deleter(m_p);
        }
        m_p = p;
    }
    
    T* get()
    {
        return m_p;
    }
    
    T* operator->()
    {
        return get();
    }
    
    T* release()
    {
        T* p = m_p;
        m_p = NULL;
        return p;
    }
};

struct CheckAndCloseHandleOp
{
    void operator() (HANDLE h) 
    {
        if(h) CloseHandle(h);
    }
};
typedef UniquePtr<remove_pointer<HANDLE>::type, CheckAndCloseHandleOp> auto_handle;

struct FreeOp
{
    void operator() (void* p)
    {
        free(p);
    }
};
template<typename T>
class auto_free : public UniquePtr<T, FreeOp>
{
};

static inline int DumpError(const throwable::ErrorT& e)
{
#if defined(__linux)
    printf("Error: unexpected exception in %s, line: %u, err: %u: %s \n",
           e.file.c_str(), e.line, e.error, e.msg.c_str());
    fflush(stdout);
#endif

#if defined(ANDROID)
    TRACE_COMPID(
        PALDEFAULT_GENERIC, 
    printf(
        "exception in %s, line: %u, err: %u: %s \n"
        "callstack: %p  \n" // #0
        "           %p  \n" // #1
        "           %p  \n" // #2
        "           %p  \n" // #3
        "           %p  \n" // #4
        "           %p  \n" // #5
        "           %p  \n" // #6
        "           %p  \n" // #7
        "           %p  \n" // #8
        "           %p    ",// #9
        e.file.c_str(), e.line, e.error, e.msg.c_str(),
        e.callstack[0], e.callstack[1], e.callstack[2], e.callstack[3], e.callstack[4],
        e.callstack[5], e.callstack[6], e.callstack[7], e.callstack[8], e.callstack[9]
        );
#endif

    return e.error;
}
}

static inline bool operator<(const GUID& g1, const GUID& g2)
{
    return memcmp(&g1, &g2, sizeof(g1)) < 0;
}

// Return a 64-bit value representing the number of 100-nanosecond intervals since January 1, 1601 (UTC).
static inline uint64_t RtcPalGetSystemTimeIn100ns()
{
    ULARGE_INTEGER uliTime = {{0}};
    FILETIME fileTime;

#if defined(WINPHONE)
    GetSystemTimeAsFileTime(&fileTime);
#else
    SYSTEMTIME sysTime;

    GetSystemTime(&sysTime);

    if (SystemTimeToFileTime(&sysTime, &fileTime))
#endif
    {
        uliTime.u.LowPart = fileTime.dwLowDateTime;
        uliTime.u.HighPart = fileTime.dwHighDateTime;
    }
    
    return uliTime.QuadPart;
}


uint64_t GetSystemBootTimeIn100ns();


#endif // __UTIL_H__
