/*++

Copyright (C) Microsoft Corporation. All rights reserved.

Module Name:

    rtcpal_aio.h

Abstract:

    Definitions for asynchronous file I/O. PAL AIO uses POSIX AIO API signature.

        rtcpal_aio_open
        rtcpal_aio_wopen
        rtcpal_aio_close
        rtcpal_aio_cancel
        rtcpal_aio_error
        rtcpal_aio_read
        rtcpal_aio_write
        rtcpal_aio_return
        rtcpal_aio_suspend

    rtcpal_aio_read and rtcpal_aio_write are used for submitting asynchronous
    I/O requests. They could return WSAEWOULDBLOCK if the underlying platform
    doesn't have enough resource to queue 'cb'. When this happens, the application
        1) must wait or periodically poll until one of the I/O request completes.
        2) then the application must use rtcpal_aio_return to get the result
           of the completed I/O request.
           Note: this is important. The underlying platform won't reset
           WSAEWOULDBLOCK contidioon unless rtcpal_aio_return has been called.
        3) then the application could retry the I/O requests which are failed
           with WSAEWOULDBLOCK error.

Author:

Revision History:

--*/

#ifndef __RTCPAL_AIO_H__
#define __RTCPAL_AIO_H__

#ifdef __cplusplus
extern "C" {
#endif

#if defined(_WIN32) || defined(ANDROID)
    struct aiocb {
        int aio_fildes;          // file handle
        uint64_t aio_offset;        // file offset
        volatile void * aio_buf;    // location of buffer
        size_t aio_nbytes;          // length of transfer

        // struct sigevent aio_sigevent;
        // int aio_lio_opcode;

        // specific on WINDOWS NT
        OVERLAPPED Overlapped;

#if defined(ANDROID )
        int aio_type;
        ssize_t ret;
        int active;
        int aio_done;
#endif
    };

    #include <fcntl.h>

#ifdef _WIN32
    #define O_RDONLY    _O_RDONLY
    #define O_WRONLY    _O_WRONLY
    #define O_RDWR      _O_RDWR
    #define O_CREAT     _O_CREAT
    #define O_EXCL      _O_EXCL
    #define O_TRUNC     _O_TRUNC
#endif

#else
    #include <aio.h>
#endif

// open file for asynchronous I/O
// return file decriptor
int
RTCPAL_CDECL
rtcpal_aio_open(
    const char * pszName,
    int oflags,
    ...
    );

int
RTCPAL_CDECL
rtcpal_aio_wopen(
    const WCHAR * pszName,
    int oflags,
    ...
    );

void
RTCAPI
rtcpal_aio_close(
    int fd
    );

// Cancel an asynchronous I/O request
RTCPAL_ERROR
RTCAPI
rtcpal_aio_cancel(
    int fd,
    struct aiocb * cb
    );

// return NO_ERROR if the I/O request has already been completed.
// return ERROR_IO_PENDIGN if the I/O request hasn't been completed
RTCPAL_ERROR
RTCAPI
rtcpal_aio_error(
    const struct aiocb * cb
    );

// Issue an aysnchronous I/O read request
RTCPAL_ERROR
RTCAPI
rtcpal_aio_read(
    __inout_ecount(1) struct aiocb * cb
    );

// Issue an aysnchronous I/O write request
RTCPAL_ERROR
RTCAPI
rtcpal_aio_write(
    struct aiocb * cb
    );

// free resource allocated by rtcpal_aio_read and rtcpal_aio_write.
//
// Return
//      >= 0    the number of bytes transferred on success.
//      -1      error happens. Use RtcPalGetLastError to retrieve the error code.
int
RTCAPI
rtcpal_aio_return(
    struct aiocb * cb
    );

// Wait for at least one I/O request to complete.
// On Windows Platform, the maximum number of I/O requests which can be waited
// is same as the maximum number of HANDLES that can be passed to WaitForMultipleObjectsEx
//
// Return NO_ERROR if at least one I/O request has completed
RTCPAL_ERROR
RTCAPI
rtcpal_aio_suspend(
    __in_ecount(cnt) const struct aiocb * const [],
    int cnt,
    uint32_t dwMilliseconds
    );

#ifdef __cplusplus
};
#endif

#endif // __RTCPAL_AIO_H__
