/*++

Abstract:
    Interface definitionas for the public interface into RTCPAL Audio/Video devices.
    This file (originally composed by sangchoe, revised for initial check-in by alexgim)
    defines the set of public interfaces at the top of the RTC PAL layer that provide
    access to Audio and Video devices.

--*/

#ifndef __rtcpal_apis_AV_devices__
#define __rtcpal_apis_AV_devices__

#include <rtcpal_device_handles.h>

// at this layer, all interfaces are C-styled
#ifdef __cplusplus
extern "C"
{
#endif

//////////////////////////////////////////////////////////////////////////////////////////////
// Data types for platform information.
//
// Platform capability bits 
// $$REVIEW - Will 64-bits be enough for all the platform specific capabilities?  If not, we may need to
//            extend this in another way.
#define RPDPLATFORM_LOOPBACK                        0x0000000000000001          /* Loopback stream is available. */
#define RPDPLATFORM_VOLUMECHANGE_CALLBACK           0x0000000000000008          /* Supports volume change notification */
#define RPDPLATFORM_TYPINGDETECT_CALLBACK           0x0000000000000010          /* Supports typing detecting callback */
#define RPDPLATFORM_FORCE_OSVERSION                 0x0000000000000020          /* Used during platform open, forces platform to a specific version of OS */
#define RPDPLATFORM_DEVICECHANGE_CALLBACK           0x0000000000000040          /* Supports device change notification */


// Device capability bits
// $$TODO - Define audio device capabilities.
#define RPDDEVICE_CAPABILITY_VIRTUALDEVICE          0x0000000000000001          /* This is a virtual device */         
#define RPDDEVICE_CAPABILITY_LOOPBACK               0x0000000000000002          /* This device supports loopback. */

// The platform type defines the major platform flavor.  In the RtcPalDevicePlatformOpen call, the RtcPalDevicePlatformInfo_t
// structure will have both the type and version populated.  For example, under Windows 7, the RtcPalDevicePlatformInfo_t
// structure's type would be RtcPalDevicePlatformType_Windows with the version set to 6.1.7600.0.
typedef enum _RtcPalDevicePlatformType_e
{
    RtcPalDevicePlatformType_Start = 0,
    RtcPalDevicePlatformType_Unknown = RtcPalDevicePlatformType_Start,
    RtcPalDevicePlatformType_Windows_Classic,       // x86 or AMD64
    RtcPalDevicePlatformType_Windows_Classic_ARM,
    RtcPalDevicePlatformType_Windows_Modern,        // x86 or AMD64
    RtcPalDevicePlatformType_Windows_Modern_ARM,
    RtcPalDevicePlatformType_Windows_Phone,
    RtcPalDevicePlatformType_Windows_CE,
    RtcPalDevicePlatformType_IPhone,
    RtcPalDevicePlatformType_OSX,
    RtcPalDevicePlatformType_XBox360,
    RtcPalDevicePlatformType_Android,
    RtcPalDevicePlatformType_Windows_Classic_ORTC,
    RtcPalDevicePlatformType_End
} RtcPalDevicePlatformType_e;

// Enum flags indicating whether the platform is in a call or no longer in a call.
// This flag is only meaningful under Windows 8 Modern implementation at this time
// to ensure we "fake" out a call to keep us from being suspended when we're in the
// background:  ie., during a call hold.
typedef enum _RtcPalDevicePlatformState_e
{
    RtcPalDevicePlatformState_Start = 0,
    RtcPalDevicePlatformState_Unknown = RtcPalDevicePlatformState_Start,
    RtcPalDevicePlatformState_InCall,
    RtcPalDevicePlatformState_NotInCall,
    RtcPalDevicePlatformState_End
} RtcPalDevicePlatformState_e;

// Structures below should be at least 32-bit aligned.  Ideally, we should keep the structures 64-bit aligned
// as Win64 uses 8 byte packs for structures.
typedef struct _RtcPalDevicePlatformVersion_t
{
    uint32_t                            nMajor;
    uint32_t                            nMinor;
    uint32_t                            nBuild;
    uint32_t                            nQFE;
} RtcPalDevicePlatformVersion_t;

typedef struct _RtcPalDevicePlatformInfo_t
{
    size_t                              cbSize;         // Always the size of this structure.
    uint64_t                            capabilities;   // Bitmask of platform capabilities (see above)
    RtcPalDevicePlatformType_e          eType;          // Platform type (Windows, OSX, iPhone, etc...)
    RtcPalDevicePlatformVersion_t       version;        // Platform version (XP = 5.1.0.0, Vista = 6.0.0.0, etc..)
} RtcPalDevicePlatformInfo_t;

//////////////////////////////////////////////////////////////////////////////////////////////
// Data types for audio format.
// NOTE:  It is intentional to make this structure match the WAVEFORMATEX structure.  We can certainly
// extend the structure as platform needs arise, but we should not lose any existing fields.  As far as
// RtcPal's audio subsystem is concerned, RVDAudioFormatDesc_t is the structure used to 
typedef struct _RVDAudioFormatDesc_t
{
    uint16_t  wFormatTag;
    uint16_t  nChannels;
    uint32_t  nSamplesPerSec;
    uint32_t  nAvgBytesPerSec;
    uint16_t  nBlockAlign;
    uint16_t  wBitsPerSample;
    uint16_t  cbSize;
} RVDAudioFormatDesc_t;

RTCPAL_FORCEINLINE
RTCPAL_BOOL 
RtcPalIsAudioFormatsSame(const RVDAudioFormatDesc_t *pAudioFmt1, const RVDAudioFormatDesc_t *pAudioFmt2)
{
    return (
        pAudioFmt1->wFormatTag == pAudioFmt2->wFormatTag &&
        pAudioFmt1->nChannels == pAudioFmt2->nChannels &&
        pAudioFmt1->nSamplesPerSec == pAudioFmt2->nSamplesPerSec &&
        pAudioFmt1->nAvgBytesPerSec == pAudioFmt2->nAvgBytesPerSec &&
        pAudioFmt1->nBlockAlign == pAudioFmt2->nBlockAlign &&
        pAudioFmt1->wBitsPerSample == pAudioFmt2->wBitsPerSample &&
        pAudioFmt1->cbSize == pAudioFmt2->cbSize
        );
}

// This structure is created to feed in the information from the ADSP into the PAL layer.
//Currently, ADSP only pass the rawstream usage information, but in future, if we need to 
// pass more information, we can extend this structure and feed the data from ADSP into PAL, 
// without adding more and more parameters to the public API
typedef struct _RVDAudioStreamProperties_t
{
    RTCPAL_BOOL bUseRawStream;
} RVDAudioStreamProperties_t;

//////////////////////////////////////////////////////////////////////////////////////////////
// Data types for audio buffer.
typedef struct _RtcPalDeviceAudioBufferInfo_t
{
    size_t                  cbSize;
    size_t                  cbBytesOffset;  // Used to indicate how many bytes to offset before capture/render.  Used only in DSound.
    size_t                  cbBytesToLock;
    unsigned char*          pbBuffer1;
    size_t                  cbBuffer1;
    unsigned char*          pbBuffer2;      // optional - some platforms may choose to always use a single buffer.
    size_t                  cbBuffer2;      // having two buffers allows plaforms to avoid memcpy if they internally use circular buffers.
    uint32_t                uiTSFlags;      // 0 - Only QPC timestamp is valid.  1 - both QPC and DEV timestamps are valid.
    uint64_t                rtQPCTS;        // This is system timestamp
    uint64_t                rtDevTS;        // This is device timestamp    
    uint32_t                uiGlitchCount;  // Glitch count in samples (AlignBlocks)
    void *                  pContext;       // May be unused.  Keep this for now to see if we need some sort of context information.
} RtcPalDeviceAudioBufferInfo_t;

#define RTCPAL_AUDBUFFLAG_DISCONTINUITY     0x00000001
#define RTCPAL_AUDBUFFLAG_SILENT            0x00000002
#define RTCPAL_AUDBUFFLAG_TS_ERROR          0x00000004

#define RTCPAL_DEVICEFLAG_RESET_ON_AUDIOROUTING     0x00000001

//////////////////////////////////////////////////////////////////////////////////////////////
// Data types for callback information.
// 
// Volume type:  For Vista/Windows 7, the session/system volume is separate.  For other platforms, session/system
// volume may be the same.
// This enum should be kept as a bitmask-able values as we can have mix of session/system/systemdb volumes.
typedef enum _RtcPalVolumeType_e
{
    RPDVolumeType_Unknown       = 0x00000000,
    RPDVolumeType_SessionMute   = 0x00000001,
    RPDVolumeType_Session       = 0x00000002,
    RPDVolumeType_SystemMute    = 0x00000004,
    RPDVolumeType_System        = 0x00000008,
    RPDVolumeType_SystemDb      = 0x00000010,
    RPDVolumeType_SystemMinDb   = 0x00000020,
    RPDVolumeType_SystemMaxDb   = 0x00000040,
    RPDVolumeType_SystemIncDb   = 0x00000080,
} RtcPalVolumeType_e;


// Not yet implemented.
typedef struct _RVDTypingDetected_t
{
    int32_t                     nKey;                   /* The virtual key code corresponding to a specific key being pressed (-1 if we can't figure it out) */
    int32_t                     nKeyDown;               /* 0 - Key Up, 1 - Key Down, -1 if no idea */
    uint64_t                    tsQPC;                  /* QPC time stamp of when the typing was detected */
} RVDTypingDetected_t;


// eType flag indicates which of the volume/mute fields are valid.
typedef struct _RtcPalDeviceVolumeInfo_t
{
    size_t                  cbSize;
    uint32_t                eType;                  // Bitmask indicating which of the following fields are valid.
    float                   fltVolSession;
    uint32_t                fMuteSession;           // 0 - not muted, 1 - muted.
    float                   fltVolSystem;
    uint32_t                fMuteSystem;            
    float                   fltVolSystemDb;
    float                   fltVolSystemMinDb;      // Read-only
    float                   fltVolSystemMaxDb;      // Read-only
    float                   fltVolSystemIncDb;      // Read-only
} RtcPalDeviceVolumeInfo_t;

// $$REVIEW - Do we need more form factors than what we have listed here?  We'll need input from the ADSP folks to
//            resolve this question.
typedef enum _RtcPalAudioDeviceFormFactor_e
{
    RPADFormFactor_Start = 0,
    RPADFormFactor_RemoteNetworkDevice = RPADFormFactor_Start,
    RPADFormFactor_Speakers,
    RPADFormFactor_LineLevel,
    RPADFormFactor_Headphones,
    RPADFormFactor_Microphone,
    RPADFormFactor_Headset,
    RPADFormFactor_Handset,
    RPADFormFactor_UnknownDigitalPassthrough,
    RPADFormFactor_SPDIF,
    RPADFormFactor_DigitalAudioDisplayDevice,
    RPADFormFactor_UnknownFormFactor,
    RPADFormFactor_Speakerphone,
    RPADFormFactor_End,
} RtcPalAudioDeviceFormFactor_e;

// $$REVIEW - Same as above.  Do we need to more connection type information?
typedef enum _RtcPalAudioDeviceConnection_e
{
    RPADConnection_Start = 0,
    RPADConnection_USB = RPADConnection_Start,
    RPADConnection_PCI,
    RPADConnection_HDAudio,
    RPADConnection_Bluetooth,
    RPADConnection_XBoxWirelessHeadset,
    RPADConnection_XBoxWirelessController,
    RPADConnection_XBoxWiredController,
    RPADConnection_XBoxNatal,
    RPADConnection_XBoxSpeakers,
    RPADConnection_End
} RtcPalAudioDeviceConnection_e;

// The device type defines whether it's a source/sink and whether the modality is audio/video.  To extend this,
// simply add new device types before the RVDDeviceType_End.
typedef enum _RtcPalDeviceType_e
{
    RtcPalDeviceType_Start = 0,
    RtcPalDeviceType_AudioCapture = RtcPalDeviceType_Start,
    RtcPalDeviceType_AudioRender,
    RtcPalDeviceType_AudioLoopback,
    RtcPalDeviceType_End
} RtcPalDeviceType_e;

// The audio endpoint type defines endpoints that are used by audio routing.
typedef enum _RtcPalAudioEndpointType_e
{
    RPAEndpointType_None            = 0x00000000,
    RPAEndpointType_Loudspeaker     = 0x00000001,
    RPAEndpointType_NonLoudspeaker  = 0x00000002,
    RPAEndpointType_Bluetooth       = 0x00000004    
} RtcPalAudioEndpointType_e;

// The audio processing feature type defines underlying audio processing
// features defined by audio.

typedef enum _AudioOffloadCapabilities_e
{
    AudioOffload_None                         = 0x00000000,

    // Device specific offload
    AudioOffload_AcousticEchoCancellation     = 0x00000001,
    AudioOffload_AutomaticGainControl         = 0x00000002,
    AudioOffload_NoiseSuppression             = 0x00000004,
    AudioOffload_SpeakerProtection            = 0x00000008,
    AudioOffload_BeamForming                  = 0x00000010,
    AudioOffload_ConstantToneRemoval          = 0x00000020,
    AudioOffload_Equalizer                    = 0x00000040,
    AudioOffload_LoudnessEqualizer            = 0x00000080,
    AudioOffload_BassBoost                    = 0x00000100,
    AudioOffload_VirtualSurround              = 0x00000200,
    AudioOffload_VirtualHeadphones            = 0x00000400,
    AudioOffload_SpeakerFill                  = 0x00000800,
    AudioOffload_RoomCorrection               = 0x00001000,
    AudioOffload_BassManagement               = 0x00002000,
    AudioOffload_EnvironmentalEffects         = 0x00004000,
    AudioOffload_SpeakerCompensation          = 0x00008000,
    AudioOffload_DynamicRangeCompression      = 0x00010000,
    AudioOffload_Other                        = 0x00020000,

    // Platform specific offload
    AudioOffload_SILK                         = 0x00100000,
} AudioOffloadCapabilities_e;

typedef enum _AudioCertifiedListVersion_e
{
    // different table
    AudioCertifiedListVersionBasic               = 0x00000000,//basic table, expected column: 1: certified list id. 2: audio effects
    AudioCertifiedListVersionWithRouting         = 0x00010000,//table with routing information. possible columns: 1: certified list id; 2: audio routing informtion, 3: audio effects.    
} AudioCertifiedListVersion_e;


typedef struct _RVDDeviceInfo_t
{
    size_t                          cbSize;
    RtcPalAudioDeviceFormFactor_e   eFormFactor;
    uint32_t                        ulWaveId;              // may not apply to some platforms
    RtcPalDeviceType_e              eType;
    union
    {
        // The internal ID should not be parsed.  It's an opaque
        // data structure.
        WCHAR                       wszDeviceId[RVDMAX_STRINGMSG]; // Internal ID, do not attempt to parse this.
        GUID                        guid;
    } id;
    WCHAR                           wszPnpName[RVDMAX_STRINGMSG]; // we also call this "moniker name".
    WCHAR                           wszFriendlyName[RVDMAX_STRINGMSG];
    WCHAR                           wszDisplayName[RVDMAX_STRINGMSG];
    WCHAR                           wszDriverVersion[RVDMAX_STRINGMSG];
    RVDAudioFormatDesc_t            deviceFormat;
    RtcPalAudioDeviceConnection_e   eConnection;
    uint32_t                        bIsEmulated;
    uint32_t                        fIsDAD;
    uint32_t                        fIsDCD;
    uint32_t                        fGainFieldValid;
    uint32_t                        fGainControlSupport;
    uint32_t                        fHardwareGainSupport;
    uint32_t                        fDBGainSupport;
    float                           fltMinGainDb;
    float                           fltMaxGainDb;
    float                           fltGainStepDb;
    GUID                            guidContainerID;
    uint64_t                        capabilities;
    int32_t                         nVendorId;          // 0 indicate invalid vendor ID.
    int32_t                         nProductId;         // 0 indicate invalid product ID.
    uint32_t                        uiRenderTickRateInMilliseconds;
    uint32_t                        fIsAudioRoutingCapable;
    uint32_t                        uAvailableAudioEndpoints;       // bit map of RtcPalAudioEndpointType_e
    RtcPalAudioEndpointType_e       eActiveAudioEndpoint;           // audio endpoint that is being used
    uint32_t                        uFlags;
    uint32_t                        fRawStreamSupport;
    WCHAR                           wszDeviceIdWinrt[RVDMAX_STRINGMSG]; //winrt from of deviceid to query the device offload effects. Needed in both classic and modern
    WCHAR                           wszExtendedDeviceId[RVDMAX_STRINGMSG]; // device id populated in exact way Skype audio library used
} RVDDeviceInfo_t;


typedef enum _RPDDeviceChangeType_e
{
    RPDDeviceChangeType_Unknown         = 0x00000000,
    RPDDeviceChangeType_DeviceState     = 0x00000001,
    RPDDeviceChangeType_Property        = 0x00000002,
    RPDDeviceChangeType_Add             = 0x00000004,
    RPDDeviceChangeType_Remove          = 0x00000008,
    RPDDeviceChangeType_DefaultDevice   = 0x00000010,
    RPDDeviceChangeType_RemoteDesktop   = 0x00000020,
}RPDDeviceChangeType_e;

typedef struct _RPDDeviceChangeInfo_t
{
    uint32_t            uiFlag;         // Bit mask flag indicating which type of change.
    RTCPAL_HANDLE       hDeviceId; 
} RPDDeviceChangeInfo_t;

typedef struct _RPDDeviceEffectsChangeInfo_t
{
    uint32_t            uiFlag;         // Bit mask flag indicating  audio effects.
} RPDDeviceEffectsChangeInfo_t;

typedef int32_t (WINAPI *PRVDDEVICECHANGECALLBACK)(HRVDPLAT hrvdplat, RPDDeviceChangeInfo_t* pInfo, void* pvContext);
typedef int32_t (WINAPI *PRVDTYPINGDETECTEDCALLBACK)(HRVDPLAT hrvdplat, void* pvContext, RVDTypingDetected_t* pKeys);
typedef int32_t (WINAPI *PRVDDEVICEVOLUMECHANGECALLBACK)(HRVDPLAT hrvdplat, HRVDDEVICE hDevice, void* pvContext, RtcPalDeviceVolumeInfo_t* pVolInfo);
typedef int32_t (WINAPI *PRVDDEVICEEFFECTSCHANGECALLBACK)(HRVDPLAT hrvdplat, void* pvContext, RPDDeviceEffectsChangeInfo_t* pEffectsInfo);

typedef struct _RtcPalDevicePlatformCallback_t
{
    void*                               pvContext;
    PRVDDEVICECHANGECALLBACK            pfnDeviceChangeCallback;
    PRVDTYPINGDETECTEDCALLBACK          pfnTypingDetectedCallback;
} RtcPalDevicePlatformCallback_t;

typedef struct _RtcPalDeviceCallback_t
{
    void*                               pvContext;
    PRVDDEVICEVOLUMECHANGECALLBACK      pfnDeviceVolumeChangeCallback;
    PRVDDEVICEEFFECTSCHANGECALLBACK     pfnDeviceEffectsChangeCallback;
} RtcPalDeviceCallback_t;


//////////////////////////////////////////////////////////////////////////////////////////////
// Data types for HID information.
typedef enum _RtcPalHIDType_e
{
    RtcPalHIDType_Unknown       = 0x00000000,
    RtcPalHIDType_FeatureReport = 0x00000001,
    RtcPalHIDType_UsageReport   = 0x00000002,
} RtcPalHIDType_e;

#define UCMR_DEVICENUMBER_MAXCOUNT              0x03

// Need to pack this into 1 byte alignment since this represents on the
// wire USB data payload.  Do not change the structure without coordination
// with the ADSP team.  And also don't move the pshpack1.h/poppack.h below.
#include <pshpack1.h>
typedef struct _UCMR_BASE_DESCRIPTOR
{
    BYTE            bLength;
    BYTE            bDescriptorType;
} UCMR_BASE_DESCRIPTOR, *PUCMR_BASE_DESCRIPTOR;

typedef struct _UCMR_CONFIG_DESCRIPTOR
{
    BYTE            bLength;
    BYTE            bDescriptorType;
    USHORT          bcdUCMedia;
    WORD            wTotalLength;
    WORD            Reserved;
} UCMR_CONFIG_DESCRIPTOR, *PUCMR_CONFIG_DESCRIPTOR;

typedef struct _UCMR_HIDUSAGE_DESCRIPTOR
{
    BYTE            bLength;
    BYTE            bDescriptorType;
    WORD            bmAttributes;
} UCMR_HIDUSAGE_DESCRIPTOR, *PUCMR_HIDUSAGE_DESCRIPTOR;

typedef struct _UCMR_AGC_INFO
{
    BYTE            bAudioDev;
    SHORT           wAudioDevInitGain;
    SHORT           wAudioDevMinGain;
    SHORT           wAudioDevMaxGain;
} UCMR_AGC_INFO, *PUCMR_AGC_INFO;

typedef struct _UCMR_ERL_INFO
{
    BYTE            bAudioDev;
    SHORT           wAudioDevERL;
} UCMR_ERL_INFO, *PUCMR_ERL_INFO;

///////////////////////////////////////////////////////////////////////
// Usage reports.
typedef struct _UCMR_USAGE_DATA
{
    BYTE            bDRC:4;
    BYTE            bAudioDeviceMode:4;
    USHORT          nPlaybackVolume;
    BYTE            bmStatus1;
}UCMR_USAGE_DATA, *PUCMR_USAGE_DATA;
#include<poppack.h>

// DO NOT pack the following structure!
typedef struct _UCMediaFeatureReportHID
{
    RTCPAL_BOOL                 fIsValid;
    UCMR_HIDUSAGE_DESCRIPTOR    Info;         
} UCMediaFeatureReportHID, *PUCMediaFeatureReportHID;

typedef struct _UCMediaFeatureReportAGC
{
    RTCPAL_BOOL                 fIsValid;
    UCMR_AGC_INFO               Info;         
} UCMediaFeatureReportAGC, *PUCMediaFeatureReportAGC;

typedef struct _UCMediaFeatureReportERL
{
    RTCPAL_BOOL                 fIsValid;
    UCMR_ERL_INFO               Info;
} UCMediaFeatureReportERL, *PUCMediaFeatureReportERL;

typedef struct _UCMediaFeatureReportData_t
{
    UCMediaFeatureReportHID     Hid;
    UCMediaFeatureReportAGC     Agc[UCMR_DEVICENUMBER_MAXCOUNT];
    UCMediaFeatureReportERL     Erl[UCMR_DEVICENUMBER_MAXCOUNT];
} UCMediaFeatureReportData_t;

typedef struct _UCMediaUsageReportData_t
{
    BYTE                        bAudioDeviceMode;
    BYTE                        bDRC;
    USHORT                      usPlaybackVolume;
    BYTE                        bmStatus1;
} UCMediaUsageReportData_t;

typedef struct _RtcPalHIDInfo_t
{
    uint32_t                    eType;              // bitmask flag of HIDType.
    uint32_t                    fFeatureValid;
    UCMediaFeatureReportData_t  hidFeature;
    uint64_t                    tsFeatureReport;
    uint32_t                    fUsageValid;
    UCMediaUsageReportData_t    hidUsage;
    uint64_t                    tsUsageReport;
} RtcPalHIDInfo_t;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Platform
/*
    Open the RTC Audio/Video Device platform handle.  This handle must be closed with RtcPalDevicePlatformClose API when done.
    This is the entry point for all RVD operations.  A valid platform handle must be passed in with all RtcPalDevice functions.
    The RtcPalDevicePlatformInfo_t structure contains the OS type and version information along with the capabilities.

    NOTE:  If RPDPLATFORM_FORCE_OSVERSION flag is set in the platforminfo structure on input, the version stored in the
           platforminfo structure is used to determine the type of subsystem created.
*/
RTC_CLASS_LINKAGE
HRESULT
RtcPalDevicePlatformOpen(
    __out           HRVDPLAT* phrvdplat,
    __inout         RtcPalDevicePlatformInfo_t* pPltInfo
    );

/*
    All successful calls to RtcPalDevicePlatformOpen must have a corresponding RtcPalDevicePlatformClose.  Calling RtcPalDevicePlatformClose
    with an invalid HRVDPLAT handle will result in an error (and an assert in checked builds):  E_INVALIDARG.
*/
RTC_CLASS_LINKAGE
HRESULT
RtcPalDevicePlatformClose(
    __in            HRVDPLAT hrvdplat
    );

/*
    To register for all platform level notifications.
    To subscribe to a specific callback, simply set the call back function pointer in the RtcPalDevicePlatformCallback_t
    structure to a valid function pointer.
*/
RTC_CLASS_LINKAGE
HRESULT
RtcPalDevicePlatformRegisterNotification(
    __in            HRVDPLAT hrvdplat,
    __in            RtcPalDevicePlatformCallback_t* pCBInfo
    );

/*
    Unregister platform level notification.
    All calls to register notification must have a corresponding unregister notification.
*/
RTC_CLASS_LINKAGE
HRESULT
RtcPalDevicePlatformUnregisterNotification(
    __in            HRVDPLAT hrvdplat
    );

/*
    Enable audio processing on this platform.  Attempts to enumerate audio devices without first
    enabling the audio subsystem will result in a failure.
*/
RTC_CLASS_LINKAGE
HRESULT
RtcPalDevicePlatformEnableAudio(
    __in            HRVDPLAT hrvdplat
    );

/*
    This will enumerate all the available devices specified by eType.  This can be called at anytime, but should
    always be called after a device change notification is fired.

    The pulCount will contain the number of RtcPalDeviceInfo_t structures needed to contain the whole list.
    Caller should allocate this buffer and call RtcPalDevicePlatformGetDeviceList() with it.
*/
RTC_CLASS_LINKAGE
HRESULT
RtcPalDevicePlatformEnumDevices(
    __in            HRVDPLAT hrvdplat,
    __in            RtcPalDeviceType_e eType,
    __in            uint32_t ulCategoryMask,
    __out           uint32_t* pulCount
    );

/*
    The actual enumeration happens with the call of RtcPalDevicePlatformEnumDevices().  This API will collect
    the device info structures from that enumeration.  On return, the pulCount will contain the number of
    device info structures written into the prvddevInfo buffer.  If there are more device info structures 
    available than space provided, the API will return ERROR_INSUFFICIENT_MEMORY error code.

    NOTE:  Calling this API prior to calling EnumDevices may result in an invalid list of devices.  This is
    because the internal platform device state is reset when a device change takes place.
*/
RTC_CLASS_LINKAGE
HRESULT
RtcPalDevicePlatformGetDeviceList(
    __in            HRVDPLAT hrvdplat,
    __in            RtcPalDeviceType_e eType,
    __inout        RVDDeviceInfo_t* prvddevinfo,
    __inout        uint32_t* pulCount
    );


/*
    Create a uniquely identifiable device ID for a given device described in the RVDDeviceInfo_t structure.
    All device ID resulting from this call should have a corresponding RtcPalDevicePlatformDisposeDeviceId call
    or will result in a memory leak.

    The resulting device ID should not be parsed.  It should be treated as an opaque data structure.
*/
RTC_CLASS_LINKAGE
HRESULT
RtcPalDevicePlatformCreateDeviceId(
    __in            HRVDPLAT hrvdplat,
    __in            RVDDeviceInfo_t* prvddevinfo,
    __out           RTCPAL_HANDLE* pdevid
    );

/*
    Duplicate the device ID.  The resulting duplicated device ID must be freed using RtcPalDevicePlatformDisposeDeviceId
    API.
*/
RTC_CLASS_LINKAGE
HRESULT
RtcPalDevicePlatformDuplicateDeviceId(
    __in            RTCPAL_HANDLE devId,
    __out           RTCPAL_HANDLE* pdevid
    );

/*
    Compare the two device ID to determine if both device IDs are referring to the same device.
*/
RTC_CLASS_LINKAGE
HRESULT
RtcPalDevicePlatformCompareDeviceId(
    __in            RTCPAL_HANDLE devId1,
    __in            RTCPAL_HANDLE devid2,
    __out           uint32_t* pfResult          // 0 - If IDs are different, !0 if IDs are the same.
    );

/*
    Compare the two device ID to determine if both device IDs are referring to the same device.
*/
RTC_CLASS_LINKAGE
HRESULT
RtcPalDevicePlatformIsSamePhysicalDevice(
    __in            RTCPAL_HANDLE devId1,
    __in            RTCPAL_HANDLE devid2,
    __out           uint32_t* pfResult          // 0 - If IDs are different, !0 if IDs are the same.
    );


/*
    Dispose of the device ID resulting from either the create or duplicate device ID API.
    Once disposed, the device ID should not be reused.
*/
RTC_CLASS_LINKAGE
HRESULT
RtcPalDevicePlatformDisposeDeviceId(
    __in            RTCPAL_HANDLE devid
    );

/*
    This will open the device represented by the devid.  On success, a valid HRVDDEVICE handle will
    be returned.  This handle MUST be freed by calling RtcPalDevicePlatformCloseDevice().
*/
RTC_CLASS_LINKAGE
HRESULT
RtcPalDevicePlatformOpenAudioDevice(
    __in            HRVDPLAT hrvdplat,
    __in            RTCPAL_HANDLE devid,
    __in            RtcPalEventHandle hEvent,
    __in            RVDAudioFormatDesc_t *pAudioFormat,
    __out           HRVDDEVICE* prvdd,
    __in            RVDAudioStreamProperties_t *pStreamProperties
    );

/*
    This will open the device represented by the devid in loopback mode.  The device represented by
    the devid MUST be an audio render device.  Any attempts to use a device id that does not correspond
    to an audio render device will result in an E_INVALIDARG failure.

    Resulting device handle must be freed using RtcPalDevicePlatformCloseDevice call.

    NOTE:  If the RPDPLATFORM_LOOPBACK capability bit is not set in the platform open API, calling this
    API will result in an E_NOTIMPL error.
*/
RTC_CLASS_LINKAGE
HRESULT
RtcPalDevicePlatformOpenAudioLoopbackDevice(
    __in            HRVDPLAT hrvdplat,
    __in            RTCPAL_HANDLE devid,
    __inout         RVDAudioFormatDesc_t *pAudioFormat,
    __out           HRVDDEVICE* prvdd
    );

/*
    Register for device related callback (currently only volume change notification is supported).
    Closing the device handle will result in the callback being unregistered.
*/
RTC_CLASS_LINKAGE
HRESULT
RtcPalDevicePlatformDeviceRegisterNotification(
    __in            HRVDDEVICE hdevice,
    __in            RtcPalDeviceCallback_t* pcallback
    );

/*
    Close/free the device handle obtained from either the open or openloopback call above.
*/
RTC_CLASS_LINKAGE 
HRESULT
RtcPalDevicePlatformCloseDevice(
    __in            HRVDPLAT hrvdplat,
    __in            HRVDDEVICE rvdd
    );

/*
    Obtain the current volume information of a given device represented by the devid.
    The eType flag of the RtcPalDeviceVolumeInfo_t will indicate which volume information
    is obtained.
*/
RTC_CLASS_LINKAGE
HRESULT
RtcPalDevicePlatformGetVolumeInfo(
    __in            HRVDPLAT hrvdplat,
    __in            RTCPAL_HANDLE devid,
    __in            RtcPalDeviceVolumeInfo_t* pVolInfo
    );

/*
    Set the volume information of a given device represented by the devid.  The eType flag
    of the RtcPalDeviceVolumeInfo_t structure will indicate which volume/mute to set.
*/
RTC_CLASS_LINKAGE
HRESULT
RtcPalDevicePlatformSetVolumeInfo(
    __in            HRVDPLAT hrvdplat,
    __in            RTCPAL_HANDLE devid,
    __in            RtcPalDeviceVolumeInfo_t* pVolInfo
    );

/*
    Sets the PAL device platform state to the eState indicated.  Media stack is
    responsible for making sure the platform is in the correct state to keep our
    application from being suspended when in background (this API is a NO-OP for
    not Windows 8 Modern PAL).
 
    NOTE:  This API MUST be called before any audio device is activated and MUST
    be called after all the activated devices are closed.
*/
RTC_CLASS_LINKAGE
HRESULT
RtcPalDevicePlatformSetState(
    __in            HRVDPLAT hrvdplat,
    __in            RtcPalDevicePlatformState_e eState
    );


/*
    Start the capture/render audio device opened.
*/
RTC_CLASS_LINKAGE
HRESULT
RtcPalDeviceAudioStart(
    __in            HRVDDEVICE rvdd
    );

/*
    Stop the capture/render audio device opened.
*/
RTC_CLASS_LINKAGE
HRESULT
RtcPalDeviceAudioStop(
    __in            HRVDDEVICE rvdd
    );

/*
    Reset the capture/render audio device.
*/
RTC_CLASS_LINKAGE
HRESULT
RtcPalDeviceAudioReset(
    __in            HRVDDEVICE rvdd
    );

/*
    Determine if the audio render device is ready for additional samples to render.
    The result of this call is the number of bytes currently queued on the render
    device.
*/
RTC_CLASS_LINKAGE
HRESULT
RtcPalDeviceAudioGetQueuedBytes(
    __in            HRVDDEVICE rvdd,                // Must be a render device.
    __out           uint32_t* pui                   // Number of queued bytes.
    );

/*
    For render device:  Locks the buffer based on the cbBytesToLock field.  On return, the pbBuffer1/pbBuffer2
    parameters will be set to a memory buffer of cbBuffer1/cbBuffer2 length into which can be written the audio
    samples.  To render the written audio, unlock the buffer with RtcPalDeviceAudioUnlockBuffer.

    For capture device:  Locks the buffer based on cbBytesToLock field.  On return the pbBuffer1/pbBuffer2 parameters
    will be set to a memory buffer where the captured audio sample is stored.  The amount of captured audio is
    determined by cbBuffer1/cbBuffer2 (NOTE:  It may NOT be the same amount as cbBytesToLock).  Once the captured
    frames are copied out of the memory, unlock the buffer with RtcPalDeviceAudioUnlockBuffer.
*/
RTC_CLASS_LINKAGE
HRESULT
RtcPalDeviceAudioLockBuffer(
    __in            HRVDDEVICE rvdd,
    __in            RtcPalDeviceAudioBufferInfo_t* pBufInfo
    );

/*
    For render device:  Unlocks the buffer and renders the audio frames copied inot the buffer data.

    For capture device:  Unlocks the buffer and allows the audio subsystem to reuse the memory.  Do NOT
    attempt to touch the pbBuffer1/pbBuffer2 buffers after calling unlock.
*/
RTC_CLASS_LINKAGE
HRESULT
RtcPalDeviceAudioUnlockBuffer(
    __in            HRVDDEVICE rvdd,
    __in            RtcPalDeviceAudioBufferInfo_t* pBufInfo
    );

/*
    Obtain the current volume information of a given device represented by the devid.
    The eType flag of the RtcPalDeviceVolumeInfo_t will indicate which volume information
    is obtained.
*/
RTC_CLASS_LINKAGE
HRESULT
RtcPalDevicePlatformGetVolumeInfoByDeviceHandle(
    __in            HRVDDEVICE rvdd,
    __in            RtcPalDeviceVolumeInfo_t* pVolInfo,
    __out_opt       uint64_t* pui64VolumeTS
    );

RTC_CLASS_LINKAGE
HRESULT
RtcPalDevicePlatformOpenHIDContext(
    __in HRVDPLAT hrvdplat,
    __in const WCHAR* pwzMonikerName,
    __out HRVDHID* phhid
    );

RTC_CLASS_LINKAGE
HRESULT
RtcPalDevicePlatformCloseHIDContext(
    __in HRVDHID hhid
    );

/*
    Set the HID context information corresponding to a given device described by the HID handle.
    The eType field of RtcPalHIDInfo_t structure will determine which type(s) of HID information
    is being set to the device.
*/
RTC_CLASS_LINKAGE
HRESULT
RtcPalDevicePlatformSetHIDInformation(
    __in            HRVDHID hhid,
    __in            RtcPalHIDInfo_t* pHidInfo
    );

/*
    Get the HID context information corresponding to a given device described by the HID handle.
    The eType field of RtcPalHIDInfo_t structure will determine which type(s) of HID information
    is being read from the device.
*/
RTC_CLASS_LINKAGE
HRESULT
RtcPalDevicePlatformGetHIDInformation(
    __in            HRVDHID hhid,
    __out           RtcPalHIDInfo_t* ppHidInfo
    );

/*
    The amount of padding (in bytes) queued to the render queue.  If the padding value
    is less than 0, the value represents the glitch duration.
*/
RTC_CLASS_LINKAGE
HRESULT
RtcPalDeviceAudioGetGlitchInfo(
    __in            HRVDDEVICE rvdd,
    __in            uint32_t uiThresholdInMilliseconds,
    __out           uint64_t* pui64Position,
    __out           uint64_t* pui64QPCPosition,
    __out           int32_t* piPadding
    );

/*
    Set the audio endpoint to use for audio routing. 
*/
RTC_CLASS_LINKAGE
HRESULT
RtcPalDeviceAudioSetActiveEndpoint(
    __in            HRVDPLAT hrvdplat,
    __in            RTCPAL_HANDLE devid,
    __in            RtcPalAudioEndpointType_e eAudioEndpoint
    );

/*
     Get current active/enabled audio processing features 
*/
RTC_CLASS_LINKAGE
HRESULT
RtcPalDeviceAudioGetProcessingFeatures(
    __in            HRVDPLAT hrvdplat,
    __in            RTCPAL_HANDLE devid,
    __out           uint32_t *puFeatureFlags
    );

/*
     Set new active audio processing features.
*/
RTC_CLASS_LINKAGE
HRESULT
RtcPalDeviceAudioSetProcessingFeatures(
    __in           HRVDPLAT hrvdplat,
    __in           RTCPAL_HANDLE devid,
    __in           uint32_t uFeatureFlags
    );

/*
     Get supported audio processing flags by OS and the features that can be overwritten.
*/
RTC_CLASS_LINKAGE
HRESULT
RtcPalDeviceAudioGetSupportedProcessingCapacity(
    __in           HRVDPLAT hrvdplat,
    __in           RTCPAL_HANDLE devid,
    __out          uint32_t* puCapacityFlags,
   __out           uint32_t *puWritableCapacityFlags
    );

/*
     Constructs a unique device whiltelist string based on certain hueristics per platform.
*/
RTC_CLASS_LINKAGE
HRESULT
RtcPalDevicePlatformGetAudioCertifiedListId(
    __in          HRVDPLAT hrvdplat,
    __in          RTCPAL_HANDLE devId,
    __out         uint32_t* puVersion,
    __out         wchar_t* pwzUniqueDeviceId,
    __inout       uint32_t* puSize
    );

/*
     Get media extension from audio platform and then application can access it directly.
*/
RTC_CLASS_LINKAGE
HRESULT
RtcPalDevicePlatformGetMediaExtension(
    __in          HRVDPLAT hrvdplat,
    __in          int iMediaExtensionType,
    __out         void ** ppMediaExtension
);


// at this layer, all interfaces are C-styled
#ifdef __cplusplus
}
#endif

#endif  /*  __rtcpal_apis_AV_devices__ */






