#pragma once
/* --

Copyright (c) Microsoft Corporation. All rights reserved.

Module Name:

    rtcpal_apis_video.h

Abstract:

    Video PAL types and defines

Specification:

    http://livecommteam/sites/main/media/W15/Media%20Stack%20W15%20Plan%20Tracking/W15%20Video%20PAL%20Interface%20Design%20Doc.docx

--*/

#ifndef __RTCPAL_APIS_VIDEO_H__
#define __RTCPAL_APIS_VIDEO_H__

#ifdef __cplusplus
extern "C" {
#endif


// =============================================================================
// Video DSP MLE Types
// =============================================================================

// Use relative path to avoid having to add /videodsp/codecs/inc in INCLUDES
#include "../../videodsp/MLEinterface/include/mleinterface_types.h"
#include "../../videodsp/MLEinterface/include/vainterface_types.h"

// =============================================================================
// Handle Definitions
// =============================================================================

#include "MediaVideoFormatCommon.h"
#include "rtcpal_device_handles.h"

// =============================================================================
// Video renderer API dependencies
// =============================================================================

#include "rtcpal_apis_video_renderer_dependencies.h"

// =============================================================================
// Macro definitions
// =============================================================================

// -----------------------------------------------------------------------------
// RTCPAL_FCC(x) macro
//
// RTC PAL version of FCC macro. Ensures correct endian-ness.
// e.g. RTCPAL_FCC('I420')
// -----------------------------------------------------------------------------
#ifndef RTCPAL_FCC
#ifdef RTCPAL_LITTLE_ENDIAN
#define RTCPAL_FCC(ch4)    ((((uint32_t) (ch4) & 0xFF) << 24) |     \
                            (((uint32_t) (ch4) & 0xFF00) << 8) |    \
                            (((uint32_t) (ch4) & 0xFF0000) >> 8) |  \
                            (((uint32_t) (ch4) & 0xFF000000) >> 24))
#else // RTCPAL_LITTLE_ENDIAN
#define RTCPAL_FCC(ch4)     (ch4)
#endif // RTCPAL_LITTLE_ENDIAN
#endif // RTCPAL_FCC


// -----------------------------------------------------------------------------
// RTCPAL_MAKE_DWORD(a,b,c,d) and RTCPAL_MAKE_WORD(a,b) macros
//
// Macros to pack 2 bytes into a word or 4 bytes into a dword, with correct
// endianness to match memory layout
// -----------------------------------------------------------------------------
#ifdef RTCPAL_BIG_ENDIAN
#define RTCPAL_MAKE_DWORD(a,b,c,d) (((a) << 24) | ((b) << 16) | ((c) << 8) | (d))
#define RTCPAL_MAKE_WORD(a,b) (((a) << 8) | (b))
#else // RTCPAL_BIG_ENDIAN
#define RTCPAL_MAKE_DWORD(a,b,c,d) (((d) << 24) | ((c) << 16) | ((b) << 8) | (a))
#define RTCPAL_MAKE_WORD(a,b) (((b) << 8) | (a))
#endif // RTCPAL_BIG_ENDIAN

// -----------------------------------------------------------------------------
// The maximum pixel difference between width and height in a square video
// -----------------------------------------------------------------------------
#define RTCPAL_SQUARE_THRESHOLD     5

// =============================================================================
// Forward declarations
// =============================================================================

struct _RtcPalVideoRawFrame_t;
struct _RtcPalVideoEncodedUnit_t;



// =============================================================================
// Enums
// =============================================================================

#define DECODER_E_SLOW MAKE_HRESULT(SEVERITY_ERROR, FACILITY_ITF, 0x0200)
#define DECODER_E_NEED_IFRAME MAKE_HRESULT(SEVERITY_ERROR, FACILITY_ITF, 0x0201)




// -----------------------------------------------------------------------------
// RtcPalDecoderCallbackData_t union
//
// Definitions for optional data that accompanies decoder events
// -----------------------------------------------------------------------------
typedef union _RtcPalDecoderCallbackData_t
{
    struct
    {
        HRESULT  hr;
        uint32_t decoderId;
    } DecoderSwitchData;
} RtcPalDecoderCallbackData_t;


// -----------------------------------------------------------------------------
// RtcPalVideoCodecBenchmarkLevel_e enumeration
//
// Enumeration of available Video Codec Benchmark complexity levels.
// -----------------------------------------------------------------------------
typedef enum _RtcPalVideoCodecBenchmarkLevel_e
{
    RtcPalVideoCodecBenchmarkLevel1 = 1,
    RtcPalVideoCodecBenchmarkLevel2 = 2,
    RtcPalVideoCodecBenchmarkLevel3 = 3,
} RtcPalVideoCodecBenchmarkLevel_e;


// -----------------------------------------------------------------------------
// RtcPalVideoSourceType_e enumeration
//
// Enumeration of Video Source types, used to differentiate between integrated,
// external (e.g. USB), and virtual sources, as well as determine the direction
// the camera is facing (front or back-facing).
//
// NOTE : this type is deprecated and will be supported for backwards compat 
// till all clients move to the new type RtcPalVideoSourceTypeEx_e
// -----------------------------------------------------------------------------
typedef enum _RtcPalVideoSourceType_e
{
    RtcPalVideoSourceType_Unknown,
    RtcPalVideoSourceType_Integrated,
    RtcPalVideoSourceType_IntegratedBackFacing,
    RtcPalVideoSourceType_External,
    RtcPalVideoSourceType_Virtual,
    RtcPalVideoSourceType_VirtualBackFacing,
    RtcPalVideoSourceType_CaptureCard,
    RtcPalVideoSourceType_ScreenScraper,
    RtcPalVideoSourceType_Other,
} RtcPalVideoSourceType_e;

// -----------------------------------------------------------------------------
// RtcPalVideoSourceApi_e enumeration
//
// Enumeration of supported video source API's. Current options are Media 
// Foundation and DirectShow.
// -----------------------------------------------------------------------------
typedef enum _RtcPalVideoSourceApi_e
{
    RtcPalVideoSourceApi_Unknown,
    RtcPalVideoSourceApi_MediaFoundation,
    RtcPalVideoSourceApi_DirectShow,    
} RtcPalVideoSourceApi_e;

// -----------------------------------------------------------------------------
// RtcPalVideoSourceType_e enumeration
//
// Enumeration of Video Source types, used to differentiate between integrated,
// external (e.g. USB), and virtual sources
//
// -----------------------------------------------------------------------------
typedef enum _RtcPalVideoSourceTypeEx_e 
{
    RtcPalVideoSourceTypeEx_Unknown,
    RtcPalVideoSourceTypeEx_Integrated,
    RtcPalVideoSourceTypeEx_External,
    RtcPalVideoSourceTypeEx_Virtual
} RtcPalVideoSourceTypeEx_e;

// -----------------------------------------------------------------------------
// RtcPalPLDPanelLocation_e enumeration
//
// Panel location of a camera as reported by the OEM, per the _PLD entry in the
// ACPI spec.
//
// NOTE : these entries do NOT match 1:1 to the actual bit values in the spec
//
// -----------------------------------------------------------------------------
typedef enum _RtcPalPLDPanelLocation_e
{
    RtcPalPLDPanelLocation_Unknown,
    RtcPalPLDPanelLocation_Top,
    RtcPalPLDPanelLocation_Bottom,
    RtcPalPLDPanelLocation_Left,
    RtcPalPLDPanelLocation_Right,
    RtcPalPLDPanelLocation_Front,
    RtcPalPLDPanelLocation_Back
} RtcPalPLDPanelLocation_e;


// -----------------------------------------------------------------------------
// RtcPalVideoCodecType_e enumeration
//
// Enumeration of Video Codec types, used to describe whether an encoder or
// a decoder is hardware-based, software-based, or mixed (e.g. DXVA). It is 
// also used to differentiate between inbox and RTC encoders/decoders
// -----------------------------------------------------------------------------
typedef enum _RtcPalVideoCodecType_e
{
    RtcPalVideoCodecType_Unknown,
    RtcPalVideoCodecType_RtcSoftware,
    RtcPalVideoCodecType_RtcMixed,
    RtcPalVideoCodecType_RtcHardware,
    RtcPalVideoCodecType_InboxSoftware,
    RtcPalVideoCodecType_InboxMixed,
    RtcPalVideoCodecType_InboxHardware,
    RtcPalVideoCodecType_CameraHardware,
    RtcPalVideoCodecType_Other,
} RtcPalVideoCodecType_e;

#define RTCPAL_VALIDATE_FORMAT_ROTATION(e) \
    RTCPAL_ASSERT(e == RtcPalVideoRotation_Any \
    || e == RtcPalVideoRotation_0 \
    || e == RtcPalVideoRotation_90Clockwise \
    || e == RtcPalVideoRotation_180 \
    || e == RtcPalVideoRotation_270Clockwise)

#define RTCPAL_VALIDATE_FRAME_ROTATION(e) \
    RTCPAL_ASSERT(e == RtcPalVideoRotation_0 \
    || e == RtcPalVideoRotation_90Clockwise \
    || e == RtcPalVideoRotation_180 \
    || e == RtcPalVideoRotation_270Clockwise)

#define RTCPAL_ROTATED_SIDE(e, v1, v2) ((e == RtcPalVideoRotation_90Clockwise \
    || e == RtcPalVideoRotation_270Clockwise) ? v2 : v1)

// -----------------------------------------------------------------------------
// PALVIDEOCAPID_??? values
//
// Values to define buckets of video capability ids
// -----------------------------------------------------------------------------
#define PALVIDEOCAPID_CATEGORY_MASK     (3 << 16)
#define PALVIDEOCAPID_ID_MASK           ((1 << 16) - 1)
#define PALVIDEOCAPID_CATEGORY_RTMCODEC (0 << 16)
#define PALVIDEOCAPID_CATEGORY_RSM      (1 << 16)
#define PALVIDEOCAPID_CATEGORY_SOURCE   (2 << 16)

// -----------------------------------------------------------------------------
// PAL_VIDEODECODERID_??? values
//
// Used to define decoder categories and individual decoders within them
// -----------------------------------------------------------------------------
#define PAL_VIDEODECODER_CATEGORY_MASK     (3 << 16)
#define PAL_VIDEODECODER_ID_MASK           ((1 << 16) - 1)
#define PAL_VIDEODECODER_CATEGORY_RTMCODEC (0 << 16)
#define PAL_VIDEODECODER_CATEGORY_INBOX (1 << 16)
#define PAL_VIDEODECODER_ID_INBOX_VC1_SW 0
#define PAL_VIDEODECODER_ID_INBOX_VC1_HW 1
#define PAL_VIDEODECODER_ID_INBOX_H264_SW 2
#define PAL_VIDEODECODER_ID_INBOX_H264_HW 3


// -----------------------------------------------------------------------------
// Supplemental RGB video types' custom four-char codes
// -----------------------------------------------------------------------------
#define RTCPAL_FOURCC_ANYFORMAT    0xFFFFFFFF
#define RTCPAL_FOURCC_RGBA         RTCPAL_FCC('RGBA')
#define RTCPAL_FOURCC_RGBX         RTCPAL_FCC('RGBX')
#define RTCPAL_FOURCC_BGRA         RTCPAL_FCC('BGRA')
#define RTCPAL_FOURCC_BGRX         RTCPAL_FCC('BGRX')
#define RTCPAL_FOURCC_RGB24        RTCPAL_FCC('RG24')
#define RTCPAL_FOURCC_BGR24        RTCPAL_FCC('BG24')
#define RTCPAL_FOURCC_RGB16        RTCPAL_FCC('RG16')
#define RTCPAL_FOURCC_BGR16        RTCPAL_FCC('BG16')
#define RTCPAL_FOURCC_RGB15        RTCPAL_FCC('RG15')
#define RTCPAL_FOURCC_BGR15        RTCPAL_FCC('BG15')
#define RTCPAL_FOURCC_H264         RTCPAL_FCC('H264')
#define RTCPAL_FOURCC_WVC1         RTCPAL_FCC('WVC1')
#define RTCPAL_FOURCC_VC1S         RTCPAL_FCC('VC1S')

#define RTCPAL_IS_CUSTOM_RGB_FCC(x) ( \
    (x == RTCPAL_FOURCC_RGBA) || \
    (x == RTCPAL_FOURCC_RGBX) || \
    (x == RTCPAL_FOURCC_BGRA) || \
    (x == RTCPAL_FOURCC_BGRX) || \
    (x == RTCPAL_FOURCC_RGB24) || \
    (x == RTCPAL_FOURCC_BGR24) || \
    (x == RTCPAL_FOURCC_RGB16) || \
    (x == RTCPAL_FOURCC_BGR16) || \
    (x == RTCPAL_FOURCC_RGB15) || \
    (x == RTCPAL_FOURCC_BGR15))


// -----------------------------------------------------------------------------
// RtmCodecs-compatible stride and image size calculation
// -----------------------------------------------------------------------------
#define RTCPAL_BIH_STRIDE(w, bpp) ((bpp == 12) ? (w) : (((w * bpp + 31) & ~31) / 8))
#define RTCPAL_BIH_IMAGESIZE(w, h, bpp) ((bpp == 12) ? (w * h * bpp / 8) : (((w * bpp + 31) & ~31) * abs(h) / 8))


// =============================================================================
// Callback Functions
// =============================================================================


// -----------------------------------------------------------------------------
// PVIDEOPLATFORMCALLBACK function prototype
//
// Video Platform event callback
// -----------------------------------------------------------------------------
typedef
HRESULT
(*PVIDEOPLATFORMCALLBACK)(
    __in            HPALVIDEOPLATFORM hVideoPlatform,
    __in            RtcPalVideoEvent_e eEventType,
    __in            void* pvContext
);


// -----------------------------------------------------------------------------
// PVIDEOSOURCECALLBACK function prototype
//
// Video Source event callback
// -----------------------------------------------------------------------------
typedef
HRESULT
(*PVIDEOSOURCECALLBACK)(
    __in            HPALVIDEOSOURCE hVideoSource,
    __in            RtcPalVideoEvent_e eEventType,
    __in            void* pvContext
);


// -----------------------------------------------------------------------------
// PVIDEOPREVIEWCALLBACK function prototype
//
// Video Preview event callback
// -----------------------------------------------------------------------------
typedef
HRESULT
(*PVIDEOPREVIEWCALLBACK)(
    __in            HPALVIDEOPREVIEW hVideoPreview,
    __in            RtcPalVideoEvent_e eEventType,
    __in            void* pvContext
);


// -----------------------------------------------------------------------------
// PVIDEOENCODERCALLBACK function prototype
//
// Video Encoder event callback
// -----------------------------------------------------------------------------
typedef
HRESULT
(*PVIDEOENCODERCALLBACK)(
    __in            HPALVIDEOENCODER hVideoEncoder,
    __in            RtcPalVideoEvent_e eEventType,
    __in            void* pvContext
);


// -----------------------------------------------------------------------------
// PFNRAWFRAMEDECODEDCALLBACK function prototype
//
// Raw frame decoded callback. Invoked by a video decoder upon decoding a frame
// -----------------------------------------------------------------------------
typedef
HRESULT
(*PFNRAWFRAMEDECODEDCALLBACK)(
    __in            struct _RtcPalVideoRawFrame_t *pDecodedFrame,
    __in            void *pvContext
);



// -----------------------------------------------------------------------------
// PFNDESTROYENCODEDUNIT function prototype
//
// Encoded Unit destruction prototype
// -----------------------------------------------------------------------------
typedef
HRESULT
(*PFNDESTROYENCODEDUNIT)(
    __in            struct _RtcPalVideoEncodedUnit_t *pEncodedUnit
);



// =============================================================================
// Data Structures
// =============================================================================

// ----------------------------------------------------------------------------
// Video Source Info
//
// Type used to describe a PAL Video Source
//-----------------------------------------------------------------------------
typedef struct _RtcPalVideoSourceInfo_t
{
    // PAL-owned source identifier
    uint32_t ulPalSourceID;
    
    // Friendly name
    wchar_t wszFriendlyName[RVDMAX_STRINGMSG];
    
    // Platform specific descriptor that is NOT guaranteed
    // to be persistent between app runs or different apps.
    wchar_t wszDescriptor[RVDMAX_STRINGMSG];

    // Platform specific descriptor that is expected
    // to be persistent between app runs or different apps.
    // Can be empty if none is available.
    wchar_t wszPersistentDescriptor[RVDMAX_STRINGMSG];
    
    // Driver version information.
    wchar_t wszDriverVersion[RVDMAX_STRINGMSG];

    // Source type - deprecated
    RtcPalVideoSourceType_e eSourceType;

    // Source type
    RtcPalVideoSourceTypeEx_e eSourceTypeEx;

    // Panel location - meaningful only if eSourceTypeEx is RtcPalVideoSourceType_Integrated or 
    // RtcPalVideoSourceType_Virtual
    RtcPalPLDPanelLocation_e ePanelLocation;
    
    // Panoramic property
    RTCPAL_BOOL bIsPanoramic;

    // Vertical and horizontal offsets of the camera relative to the Panel origin. 
    // Bits 128-143 (vertical) and 144-159 (horizontal) of the _PLD spec
    // The X and Y values are meaningful only if bPositionSupported is true
    RTCPAL_BOOL bPositionSupported;
    int32_t     lPhysicalPositionX;
    int32_t     lPhysicalPositionY;

    // Clockwise rotation degrees of the camera arounds its origin (bits 115-118 of the _PLD spec)
    uint32_t    ulRotationDegreeCW;
    uint64_t    capabilities;
    int32_t     nVendorId;
    int32_t     nProductId;
    uint32_t    uFirmwareVersion;
    
    // Source API
    RtcPalVideoSourceApi_e eSourceApi;
} RtcPalVideoSourceInfo_t;

#define RtcPalVideoRotation_Any MediaVideoRotation_Any 
#define RtcPalVideoRotation_0 MediaVideoRotation_0
#define RtcPalVideoRotation_90Clockwise MediaVideoRotation_90Clockwise 
#define RtcPalVideoRotation_180 MediaVideoRotation_180
#define RtcPalVideoRotation_270Clockwise MediaVideoRotation_270Clockwise 



// There are other controls for both video proc and camera control, but
// to keep things consistent between classic and modern, I've limited this
// list to just what the modern version exposes (i.e., exposed by IVideoDeviceController)
// If we need a control beyond this, the modern version will have to return
// HRESULT_FROM_WIN32(ERROR_NOT_SUPPORTED).
typedef enum _RtcPalVideoControl_e
{
    RtcPalVideoControl_Start = 0,
    RtcPalVideoControl_BacklightCompensation = RtcPalVideoControl_Start,
    RtcPalVideoControl_Brightness,
    RtcPalVideoControl_Contrast,
    RtcPalVideoControl_WhiteBalance,
    RtcPalVideoControl_Hue,
    RtcPalVideoControl_PowerLineFrequency,
    RtcPalVideoControl_Exposure,
    RtcPalVideoControl_Focus,
    RtcPalVideoControl_Zoom,
    RtcPalVideoControl_Pan,
    RtcPalVideoControl_Roll,
    RtcPalVideoControl_Tilt,
    
    RtcPalVideoControl_PreviewFrameRate,                      // This is a non-videoproc/camctrl control.
    RtcPalVideoControl_PreviewAvgFrameRate,                   // This is a non-videoproc/camctrl control.
    RtcPalVideoControl_PreviewMaxFrameRate,                   // This is a non-videoproc/camctrl control.

    RtcPalVideoControl_RecordFrameRate,                       // This is a non-videoproc/camctrl control.
    RtcPalVideoControl_RecordAvgFrameRate,                    // This is a non-videoproc/camctrl control.
    RtcPalVideoControl_RecordMaxFrameRate,                    // This is a non-videoproc/camctrl control.

    RtcPalVideoControl_End
} RtcPalVideoControl_e;

// The capability flags set in the uiCaps field below.
#define RTCPALVIDCTRL_SUPPORTED             0x80000000      // Supported control.
#define RTCPALVIDCTRL_MANUAL                0x00000001      // By default, all supported controls are manually settable.
#define RTCPALVIDCTRL_AUTOMODE              0x00000002      // Some controls also support auto mode.
#define RTCPALVIDCTRL_READONLY              0x00000004      // Some controls are read only.

typedef struct _RtcPalVideoControlState_t
{
    uint32_t    uiCaps;
    uint32_t    uiMode;
    double      dblMin;
    double      dblMax;
    double      dblInc;
    double      dblDef;
} RtcPalVideoControlState_t;

typedef enum _RtcPalVideoStreamType_e
{
    RtcPalVideoStreamType_Start = 0,
    RtcPalVideoStreamType_Preview = RtcPalVideoStreamType_Start,
    RtcPalVideoStreamType_Record,
    RtcPalVideoStreamType_End
} RtcPalVideoStreamType_e;

#define RTCPAL_VIDEO_CROP_INFO_EMPTY(x) \
    (((x)->usCropLeftOffset == 0) && ((x)->usCropRightOffset == 0) && \
    ((x)->usCropTopOffset == 0) && ((x)->usCropBottomOffset == 0))

typedef struct _RtcPalVideoCropInfo2_t
{
    uint16_t usCropLeftRatio;
    uint16_t usCropTopRatio;    
    uint16_t usCropRightRatio;
    uint16_t usCropBottomRatio;
} RtcPalVideoCropInfo2_t;

// -----------------------------------------------------------------------------
// RtcPalVideoCropInfo2_t macros
// -----------------------------------------------------------------------------
#define RTCPAL_CROPINFO2_PRECISION     15
#define RTCPAL_CROPINFO2_ONE           (1 << RTCPAL_CROPINFO2_PRECISION)
#define RTCPAL_CROPINFO2_FROM_FLOAT(flt) ((uint16_t) (0.5f + (flt) * RTCPAL_CROPINFO2_ONE))
#define RTCPAL_CROPINFO2_TO_FLOAT(r)   ((float) (r) / (float) RTCPAL_CROPINFO2_ONE)
#define RTCPAL_CROPINFO2_FROM_OFFSET(x, s) ((((x) << RTCPAL_CROPINFO2_PRECISION) + ((s) / 2)) / (s))
#define RTCPAL_CROPINFO2_TO_OFFSET(r, s) \
    (((r) * (s) + (1 << (RTCPAL_CROPINFO2_PRECISION - 1))) >> RTCPAL_CROPINFO2_PRECISION)


// -----------------------------------------------------------------------------
// Video Source Frame Resolution
//
// Structure used to describe the last frame information for video source.
// including FCC and the resolution fields.
// -----------------------------------------------------------------------------
typedef struct _RtcPalVideoSourceFrameResolution_t
{
    RtcPalVideoFrameResolution_t record;
    RtcPalVideoFrameResolution_t preview;
}RtcPalVideoSourceFrameResolution_t;



// -----------------------------------------------------------------------------
// Video Decode Capability
//
// Structure used to describe decoder capability.
// -----------------------------------------------------------------------------
typedef struct _RtcPalVideoDecodeCapability_t
{
    // Unique ID
    uint32_t ulDecoderID;
    
    // Decoder type
    RtcPalVideoCodecType_e eDecoderType;
    
    // Input format
    uint32_t fccVideoFormat;
    
    // Maximum resolution
    uint16_t usMaxFrameWidth;
    uint16_t usMaxFrameHeight;
    
    // Maximum throughput
    uint32_t ulMaxMacroBlocksPerSec;

    union
    {
        struct
        {
            // Can decode AVC/SVC 'Constrained High' profile
            RTCPAL_BOOL bSupportConstrainedHigh;
            
            // Require CGSRW quality scalability mode for SVC compatibility
            RTCPAL_BOOL bSVCRequiresCGSRWMode;
        } H264;
        
        struct
        {
            // RTVideo support (SP frames)
            RTCPAL_BOOL bSupportSPFrames;
        } VC1;
    };
    
} RtcPalVideoDecodeCapability_t;



// -----------------------------------------------------------------------------
// Video Decoder Constraints
//
// Specifies a set of limits on what codecs from enumeration can be used
// by RTCPAL Decoder to choose the right decoder for subsequent frames.
// -----------------------------------------------------------------------------
typedef struct _RtcPalVideoDecoderConstraints_t
{
    int __tbd; // TODO: define members
} RtcPalVideoDecoderConstraints_t;

// -----------------------------------------------------------------------------
// Video Encoded Unit
//
// Structure used for delivery of compressed video units through
// the encoded stream, or for consumption of compressed video units
// by a decoder.
// -----------------------------------------------------------------------------
typedef struct _RtcPalVideoEncodedUnit_t
{
    // Encoded format
    uint32_t fccVideoFormat;
    
    // Resolution
    uint16_t usFrameWidth;
    uint16_t usFrameHeight;

    // Frame data
    uint8_t* pbFrameData;
    uint32_t ulDataLength;
    
    union
    {
        struct
        {
            // NAL unit data info
            uint32_t ulDataOffset;
            uint8_t  bLastNaluFlag;

            // extra info
            uint16_t usTDQId;
            uint16_t usTargetTDQId;
            uint8_t  bLossFlag;
            uint8_t  bEndOfAU;

            // crop info
            uint8_t  bCropPresence;
            uint32_t ulRecvCsrc;
            RtcPalVideoCropInfo_t CropInfo;

            uint8_t bPresentationTimeStampPresence;
            uint16_t usPresentationTimeStamp;

            // MF specific information
            void* pvMFBuffer;
            void* pvMFSample;

            PFNDESTROYENCODEDUNIT pfnTranscodeDestroyUnit;
            int32_t lTrancodeRefCount;

        } H264;

        struct
        {
            // Frame header (optional, header can also be in the data)
            uint8_t  pbFrameHeader[65];
            uint32_t ulHeaderLength;
    
            // Codec flag (RTVideo only)
            uint32_t ulCodecFlag;
        } VC1;
    };
    
    
    // allows us to follow frame delivery
    void *pvMetricsInfo;

    // Time stamp
    uint64_t ullTimeStamp;

    // function pointer and context to destroy frame
    PFNDESTROYENCODEDUNIT pfnDestroyUnit;

    // private context for frame originator.
    void * pvOriginatorPrivateData; 

    uint8_t  Test_bDropDecodedFrame;

            
} RtcPalVideoEncodedUnit_t;

typedef enum _RtcPalFrameProcessState 
{
        NO_FRAME,
        FRAME_DROPPED,
        FRAME_PROCESSED        
} RtcPalFrameProcessState;


// -----------------------------------------------------------------------------
// Video Source Statistics
//
// Structure used for description of source real-time statistics.
// -----------------------------------------------------------------------------
typedef struct _RtcPalVideoSourceStats_t
{
    RtcPalVarianceStats_t interFrameGap;
    uint32_t ulVideoProcessorsCreated;
    uint32_t ulVideoProcessorsDestroyed;
    uint32_t ulBuffersAllocated;
    uint32_t ulBuffersDestroyed;
    uint32_t ulTotalBuffersSize;
    uint32_t ulSkippedFrames;
} RtcPalVideoSourceStats_t;


// -----------------------------------------------------------------------------
// Encoder Video Processing Statistics
//
// Structure used for description of encoder real-time statistics.
// -----------------------------------------------------------------------------
#define RTCPAL_MAX_ENCODER_STREAMS   16   // Max number of streams per MLE encoder

typedef struct _RtcPalVideoEncoderProcessingStats_t
{
    uint32_t  ulNumberOfStreams;
    RtcPalVideoProcessingStats_t stats[RTCPAL_MAX_ENCODER_STREAMS];
} RtcPalVideoEncoderProcessingStats_t;


// ----------------------------------------------------------------------------
// Virtual Video Source Descriptor
//
// Type used to register a virtual video source
//-----------------------------------------------------------------------------
typedef struct _RtcPalVideoVirtualSourceDesc_t
{
    // Virtual source video source info. 
    // ulPalSourceID field is not guaranteed to stay the same after registration
    // eSourceType must be RtcPalVideoSourceType_Virtual 
    // or RtcPalVideoSourceType_VirtualBackFacing
    RtcPalVideoSourceInfo_t sourceInfo;
    RTCPAL_BOOL fAllowMultipleInstances;

    // Functions for source functionality
    HRESULT (*pfnCreateInstance)(
            const RtcPalVideoSourceInfo_t* pVirtualSourceInfo,
            HANDLE* phSourceOut, 
            HRESULT (*pfnDeliverFrame)(void* pvContext, RtcPalVideoRawFrame_t* pFrame), 
            void* pvDeliverFrameContext);

    HRESULT (*pfnDeleteInstance)(HANDLE hSource);
    HRESULT (*pfnGetVideoFormatSupportedList)(HANDLE hSource, RtcPalVideoFormat_t* pFormats, uint32_t* pulCount);
    HRESULT (*pfnGetVideoFormat)(HANDLE hSource, RtcPalVideoFormat_t* pFormatOut);
    HRESULT (*pfnSetVideoFormat)(HANDLE hSource, const RtcPalVideoFormat_t* pFormat);
    HRESULT (*pfnStartDevice)(HANDLE hSource);
    HRESULT (*pfnStopDevice)(HANDLE hSource);
    HRESULT (*pfnSetFramerateDynamic)(HANDLE hSource, float fNewRate);
    
    
} RtcPalVideoVirtualSourceDesc_t;

// forward declarations
struct IRtcPalDecoderInterface;
struct IRtcPalDecoderCallback;
struct IRtcPalVideoRenderer;

struct IRtcPalVideoEncoderCallback;
struct IRtcPalVideoEncoderInterface;
struct IRtcPalVideoFrameWrapper;


// =============================================================================
// Dependent header files
// =============================================================================

#include "rtcpal_apis_video_platform.h"
#include "rtcpal_apis_video_source.h"
#include "rtcpal_apis_video_preview.h"
#include "rtcpal_apis_video_analyze.h"
#include "rtcpal_apis_video_encode.h"
#include "mmvr_Common.h"

#ifdef __cplusplus
};

#include "VirtualDeviceCommon.h"

// ----------------------------------------------------------------------------
// Video Frame Wrapper interface
//-----------------------------------------------------------------------------
struct IRtcPalVideoFrameWrapper
{
    // Lifetime management
    virtual ULONG AddRef() = 0;
    virtual ULONG Release() = 0;
    
    // Grab the frame
    virtual HRESULT GetFrame(RtcPalVideoRawFrame_t** ppFrame) = 0;
    
    typedef enum _InputType_e {
        InputType_RawFrame = MLE_InputType_RawFrame,
        InputType_IMFSample = MLE_InputType_IMFSample,
        InputType_IMediaSample = MLE_InputType_IMediaSample,
    } InputType_e;
    virtual HRESULT GetService(InputType_e iid, void** ppvObject) = 0;
    
    // Format inspection
    virtual HRESULT GetFormat(RtcPalVideoFormat_t* pFormat) = 0;
    
    // Timestamp management
    virtual uint64_t GetTimeStamp() = 0;
    
    // Dimension accessors
    virtual uint32_t GetFourCC() const = 0;
    virtual uint16_t GetWidth() const = 0;
    virtual uint16_t GetHeight() const = 0;
    virtual uint16_t GetOriginalWidth() const = 0;
    virtual uint16_t GetOriginalHeight() const = 0;
    virtual RtcPalVideoRotation_e GetRotation() const = 0;
    virtual ScreenInfo_t * GetScreenInfo() const = 0;
        
};

#include "rtcpal_decoder_Interface.h"
#include "rtcpal_apis_video_renderer.h"
#include "rtcpal_encoder_interface.h"

#endif

#endif // __RTCPAL_APIS_VIDEO_H__
