#pragma once
/* --

Copyright (c) Microsoft Corporation. All rights reserved.

Module Name:

    rtcpal_apis_video_platform.h

Abstract:

    PAL Video Platform interface

Specification:

    http://livecommteam/sites/main/media/W15/Media%20Stack%20W15%20Plan%20Tracking/W15%20Video%20PAL%20Interface%20Design%20Doc.docx

--*/

#ifndef __RTCPAL_APIS_VIDEO_PLATFORM_H__
#define __RTCPAL_APIS_VIDEO_PLATFORM_H__


// =============================================================================
// Video Platform
// =============================================================================


// ----------------------------------------------------------------------------
// RtcPalVideoPlatformOpen
// 
// Create a Video Platform object. There can be only one Video Platform object
// per application, and this object should persist as long as Video Sources
// are in use.
// ----------------------------------------------------------------------------
RTC_CLASS_LINKAGE
HRESULT
RtcPalVideoPlatformOpen(
    __in            PVIDEOPLATFORMCALLBACK fnEventCallback,
    __in            void* pvCallbackContext,
    __in            void* pMemoryManager, //$TODO: Interface to be defined, use NULL for now
    __in            void* pThreadManager, //$TODO: Interface to be defined, use NULL for now
    __out           HPALVIDEOPLATFORM *phVideoPlatform
);


// ----------------------------------------------------------------------------
// RtcPalVideoPlatformClose
// 
// Close a Video Platform object. Before the Enumerator is closed, 
// all Sources created by the Enumerator should be released.
// ----------------------------------------------------------------------------
RTC_CLASS_LINKAGE
HRESULT
RtcPalVideoPlatformClose(
    __in            HPALVIDEOPLATFORM hVideoPlatform
);

// ----------------------------------------------------------------------------
// RtcPalVideoPlatformGetMediaExtension
// 
// Get a video source extension.
// ----------------------------------------------------------------------------
RTC_CLASS_LINKAGE
HRESULT
RtcPalVideoPlatformGetMediaExtension(
    __in            HPALVIDEOPLATFORM hVideoPlatform,
    __in            int MediaExtensionType,
    __out           void ** pMediaExtension
);

// ----------------------------------------------------------------------------
// RtcPalVideoPlatformRegisterVirtualSource
// 
// Register a virtual source.pSourceDesc->sourceInfo.ulPalSourceID will be
// populated upon exit with an assigned unique source ID.
// ----------------------------------------------------------------------------
RTC_CLASS_LINKAGE
HRESULT
RtcPalVideoPlatformRegisterVirtualSource(
    __in            HPALVIDEOPLATFORM hVideoPlatform,
    __inout         RtcPalVideoVirtualSourceDesc_t* pSourceDesc
);


// ----------------------------------------------------------------------------
// RtcPalVideoPlatformUnregisterVirtualSource
// 
// Unregister a virtual source. ulPalSourceID must refer to a previously
// registered virtual source.
// ----------------------------------------------------------------------------
RTC_CLASS_LINKAGE
HRESULT
RtcPalVideoPlatformUnregisterVirtualSource(
    __in            HPALVIDEOPLATFORM hVideoPlatform,
    __in            uint32_t ulPalSourceID
);


// ----------------------------------------------------------------------------
// RtcPalVideoPlatformQuerySourceList
// 
// Query the set of available sources. This list includes all available
// sources, including ones that may have been included in previous calls
// to RtcPalVideoPlatformQuerySourceList.
// ----------------------------------------------------------------------------
RTC_CLASS_LINKAGE
HRESULT
RtcPalVideoPlatformQuerySourceList(
    __in            HPALVIDEOPLATFORM hVideoPlatform,
    __out           RtcPalVideoSourceInfo_t* pSourceInfos,
    __inout         uint32_t* pulCount
);


// ----------------------------------------------------------------------------
// RtcPalVideoPlatformOpenSource
// 
// Open a Video Source associated with the provided Video Source Info.
// ----------------------------------------------------------------------------
RTC_CLASS_LINKAGE
HRESULT
RtcPalVideoPlatformOpenSource(
    __in            HPALVIDEOPLATFORM hVideoPlatform,
    __in            const RtcPalVideoSourceInfo_t* pSourceInfo,
    __in            PVIDEOSOURCECALLBACK fnCallback,
    __in            void* pvCallbackContext,
    __out           HPALVIDEOSOURCE* phVideoSource
);


// ----------------------------------------------------------------------------
// RtcPalVideoPlatformReleaseSource
// 
// Delete the Video Source.
// ----------------------------------------------------------------------------
RTC_CLASS_LINKAGE
HRESULT
RtcPalVideoPlatformCloseSource(
    __in            HPALVIDEOPLATFORM hVideoPlatform,
    __in            HPALVIDEOSOURCE hVideoSource
);


// ----------------------------------------------------------------------------
// RtcPalVideoPlatformQueryAnalyzerCapabilities
// 
// Query the set of analyzer capabilities for available source-independent
// analyzers
// ----------------------------------------------------------------------------
RTC_CLASS_LINKAGE
HRESULT
RtcPalVideoPlatformQueryAnalyzerCapabilities(
    __in            HPALVIDEOPLATFORM hVideoPlatform,
    __out           VA_CapabilityEx* pCapabilities,
    __inout         uint32_t* pulCount
);


// ----------------------------------------------------------------------------
// RtcPalVideoPlatformQueryLowComplexityRequest
// 
// Query the request of operating in low complexity mode
// ----------------------------------------------------------------------------
RTC_CLASS_LINKAGE
HRESULT
RtcPalVideoPlatformQueryLowComplexityRequest(
    __in            HPALVIDEOPLATFORM hVideoPlatform,
    __inout         uint8_t* pbLowComplexity
);


// ----------------------------------------------------------------------------
// RtcPalVideoPlatformQueryDecodeCapabilities
// 
// Query the set of decode capabilities for available decoders.
// ----------------------------------------------------------------------------
RTC_CLASS_LINKAGE
HRESULT
RtcPalVideoPlatformQueryDecodeCapabilities(
    __in            HPALVIDEOPLATFORM hVideoPlatform,
    __out           MLD_CapabilityEx* pCapabilities,
    __inout         uint32_t* pulCount
);

// ----------------------------------------------------------------------------
// RtcPalVideoPlatformQueryInboxDecodeCapabilities
// 
// Query the set of decode capabilities for available decoders.
//
// BUGBUG:
// This API name is temporary. It is used during transition to the new MLD world.
// We shouldn't have "Inbox" in API name. But RtcPalVideoPlatformQueryDecodeCapabilities
// is still needed for the existing code. After VSCA/test switches to the new code path,
// we should rename this API.
// ----------------------------------------------------------------------------
EXTERN_C
HRESULT
RtcPalVideoPlatformQueryInboxDecodeCapabilities(
    __in HPALVIDEOPLATFORM hVideoPlatform,
    __in MLD_CapabilityEx *pCapabilities,
    __in uint32_t * puCount
    );

EXTERN_C
HRESULT
RtcPalVideoPlatformCreateInboxDecoder(
    __in HPALVIDEOPLATFORM hVideoPlatform,
    __in uint32_t DecoderId,
    __in struct IRtcPalDecoderCallback * pDecoderCallback,
    __in struct IRtcPalVideoRenderer * pRenderer,
    __out struct IRtcPalDecoderInterface ** ppDecoder
    );

#endif // __RTCPAL_APIS_VIDEO_PLATFORM_H__


