#pragma once
/* --

Copyright (c) Microsoft Corporation. All rights reserved.

Module Name:

    rtcpal_apis_video_renderer.h

Abstract:

    PAL Video to PAL Video renderer interface
    Every function defined here must be either C or pure virtual.

Specification:

    http://livecommteam/sites/main/media/W15/Media%20Stack%20W15%20Plan%20Tracking/W15%20Video%20PAL%20Interface%20Design%20Doc.docx

--*/

#ifndef __RTCPAL_APIS_VIDEO_RENDERER_H__
#define __RTCPAL_APIS_VIDEO_RENDERER_H__


// =============================================================================
// Renderer API header dependencies.
// =============================================================================
#include "rtcpal_apis_video_renderer_dependencies.h"


// =============================================================================
// Enums
// =============================================================================


// -----------------------------------------------------------------------------
// Hardware Video Sample Type
//
// Platform-independent wrapper for platform-specific video sample type used in
// RtcPalVideoHWSample_t
// -----------------------------------------------------------------------------
typedef enum _RtcPalVideoHWSampleType_e
{
    // Special value to flag that hardware samples are not supported
    RtcPalVideoHWSample_Unsupported = 0,

    // Windows types
    RtcPalVideoHWSample_IMFSample,
    RtcPalVideoHWSample_DX9Surface,
    RtcPalVideoHWSample_DX11Texture,

    RtcPalVideoHWSample_SL,

    // Pass through sample, decoded on renderer.
    // Used in combination with PassthroughMMVR.
    RtcPalVideoHWSample_Passthrough,
    // ... place additional types here ...    
} RtcPalVideoHWSampleType_e;

// -----------------------------------------------------------------------------
// Hardware Video Sample
//
// Structure used for delivery of platform-specific video samples, generally
// located in video memory
// -----------------------------------------------------------------------------
typedef struct _RtcPalVideoHWSample_t
{
    // Type of the hardware sample
    RtcPalVideoHWSampleType_e eType;

    // Sample object
    void* pvHWSample;
	
	// Sample Meta-data
	void* pvSampleInfo;

    // Time stamp
    uint64_t ullTimeStamp;

    // Original Resolution (may be aspect ratio in H.264)
    uint16_t usOrgFrameWidth;
    uint16_t usOrgFrameHeight;

    // App defined crop info
    RTCPAL_BOOL bValidCropInfo;
    RtcPalVideoCropInfo_t CropInfo;

    // Edge crop info. If the H/W frame that contains this sample is larger
    // than the source image size, this is used to crop out the empty space
    // around the image.
    RtcPalVideoCropInfo_t EdgeCropInfo;
} RtcPalVideoHWSample_t;

#ifndef E_NOTSUPPORTED
    // cut & pasted from $SDK_INC_PATH/infotech.h
    #define E_NOTSUPPORTED      _HRESULT_TYPEDEF_(0x80001008L)
#endif

// -----------------------------------------------------------------------------
// Video Resource Type
//
// Type of resource, used in resource exchange
// -----------------------------------------------------------------------------
typedef enum _RtcPalVideoResourceType_e
{
    // Windows types
    RtcPalVideoResource_D3D9Manager,
    RtcPalVideoResource_DXGIManager,
    RtcPalVideoResource_IMFMediaFormat,

    RtcPalVideoResource_NoResource,
    RtcPalVideoResource_UnableToRecreate, // same meaning as "no resource", but
                                          // except that it means the caller would
                                          // have preferred to have a resource
    // ... place additional types here ...

    RtcPalVideoResource_SLDecodingRender,  // this is to tell SL render
                                           // the format that we're
                                           // passing to them to head
                                           // start the "Attach" process

    RtcPalVideoResource_PassthroughInitializer, // Indicates that GetResource call 
                                                // is used to initialize Passthrough MMVR.
    RtcPalVideoResource_DecoderBuffer,          // Decoder buffer for Passthrough MMVR.

} RtcPalVideoResourceType_e;


// =============================================================================
// Data Structures
// =============================================================================


struct IRtcPalVideoRenderer;

// Renderer callback interface implemented by decoder
struct IRtcPalVideoRendererCallback
{
    // Currently, we just blindly copy all the events from existing code
    // The list of event needs to be trimmed down
    virtual void Invoke(IRtcPalVideoRenderer * pSender, RtcPalVideoEvent_e event) = 0;

	// Do we need virtual destructor here?
	// virtual ~IRtcPalVideoRendererCallback() {}
};

struct IRtcPalVideoResourceExchange;

// render interface exposed to decoder
struct IRtcPalVideoRenderer
{
    // Initialize video renderer
    virtual HRESULT Attach(
        IRtcPalVideoRendererCallback * pCallback
        ) = 0;

    // Release video renderer
    virtual HRESULT Detach() = 0;

    // Get the hardware resource required by hardware decoder.
    //
    // pReserved is reserved for future unknown resource which might require callback
    // from renderer for resource change. For now, it is always NULL.
    virtual HRESULT GetResource(
        __in RtcPalVideoResourceType_e eType,
        __in_opt IRtcPalVideoResourceExchange * pReserved,
        __out_ecount(1) void ** ppvResource
        ) = 0;

    // Enable/disable mirroring
    virtual HRESULT EnableMirroring(RTCPAL_BOOL fEnable) = 0;

    // Retrieve supported formats
    virtual HRESULT GetSupportedFormats(uint32_t* pFormats, uint32_t* pulCount) = 0;

    // Render software frame
    virtual HRESULT RenderFrame(const RtcPalVideoRawFrame_t* pFrame) = 0;

    // Get suported hardware sample type
    virtual HRESULT GetSupportedHWSampleType(RtcPalVideoHWSampleType_e* peType) = 0;

    // Render hardware sample
    virtual HRESULT RenderSample(RtcPalVideoHWSample_t* pSample) = 0;

    // Get the renderer's preferred video size
    virtual HRESULT GetPreference(uint16_t* pusWidth, uint16_t* pusHeight) = 0;

    // Get the renderer's processing statistics
    virtual HRESULT GetStatistics(RtcPalVideoProcessingStats_t * pStats) = 0;

    // Reset the renderer's processing statistics
    virtual HRESULT ResetStatistics() = 0;

    // Get metrics related to the latest frame that was rendered
    virtual void * GetLastFrameMetrics() = 0;

    // Get the resolution of last frame rendered.
    virtual HRESULT GetLastFrameInfo(__out RtcPalVideoFrameResolution_t * pLastFrameInfo) = 0;

    // Used by preview only
    // This API allows preview to render a blank frame before preview stops.
    // It is not used at decoder path because decoder doesn't stop. Decoder
    // detaches from the renderer.
    virtual HRESULT RenderBlankFrame() = 0;

    // A private interface between preview and MMVR custom in VDI scenario.
    // TODO: we need an alternative method to support private channel
    virtual HRESULT GetPreferredSize(uint16_t* pusWidth, uint16_t* pusHeight) = 0;

    // return FALSE - MMVR can't support hardware sample
    // return TRUE - MMVR can support hardware sample
    // If MMVR hasn't determined its capability, it should return TRUE here.
    // MMVR shall not block this API and wait for the capabiilty to be determined.
    virtual RTCPAL_BOOL CanSupportHardwareSample() const = 0;

	// Do we need virtual destructor here?
	// virtual ~IRtcPalVideoRenderer() {}
};

#endif // __RTCPAL_APIS_VIDEO_RENDERER_H__


