#pragma once
/* --

Copyright (c) Microsoft Corporation. All rights reserved.

Module Name:

rtcpal_apis_video_renderer_dependencies.h

Abstract:

A bare minimum dependency definition header for video renderer API usage.

--*/

#ifndef __RTCPAL_APIS_VIDEO_RENDERER_DEPENDENCIES_H__
#define __RTCPAL_APIS_VIDEO_RENDERER_DEPENDENCIES_H__


#include "MediaVideoFormatCommon.h"


// =============================================================================
// Macro definitions
// =============================================================================


#define RtcPalVideoRotation_e   MediaVideoRotation_e
#define RtcPalVideoFormat_t     MediaVideoFormat_t


// =============================================================================
// Enums
// =============================================================================


// -----------------------------------------------------------------------------
// RtcPalVideoEvent_e enumeration
//
// Enumeration of PAL Video Platform events used in the event callback
// functions as a parameter
// -----------------------------------------------------------------------------
typedef enum _RtcPalVideoEvent_e
{
    // Errors
    RtcPalVideoEvent_GenericError = -1,
    RtcPalVideoEvent_ResolutionChangeError = -200,
    RtcPalVideoEvent_FailedToStartError = -201,

    // Plug & Play events
    RtcPalVideoEvent_SourceListChanged = 0,

    // Start/Stop events
    RtcPalVideoEvent_Started,
    RtcPalVideoEvent_Stopped,

    // Frame Delivery events
    RtcPalVideoEvent_FrameReady,
    RtcPalVideoEvent_FramesDropped,

    // Preference change (video sink only)
    RtcPalVideoEvent_PreferenceChange,

    // debugging-related - used to flag timestamps.
    // during the time when these are fired, call
    // the videoextension's GetLastFrameMetrics() function to
    // get caller context that's useful to correlate events
    RtcPalVideoEvent_FrameStartedDecoding,
    RtcPalVideoEvent_FrameStartedRendering,
    RtcPalVideoEvent_FrameDoneConverting,
    RtcPalVideoEvent_FrameDoneDecoding,
    RtcPalVideoEvent_FrameDoneRendering,

    // these events carry a decoder ID and hresult in a DecoderSwitchData struct
    RtcPalVideoEvent_DecoderSwitchOk,               // got a decoder - the callback data says which ID
    RtcPalVideoEvent_DecoderSwitchInitFailed,       // a decoder's initialization failed, PAL will try the next decoder on the list
    RtcPalVideoEvent_DecoderCriticalRuntimeFailure, // a decoder had a critical failure AFTER initialization succeeded
    RtcPalVideoEvent_DecoderSwitchNoMoreDecoders,   // all decoders in the current priority list failed, 
    RtcPalVideoEvent_DecoderOverflow,
    RtcPalVideoEvent_DecoderWaitingForIFrame,

    // Renderer failure during rendering
    RtcPalVideoEvent_RendererCriticalRuntimeFailure,

    // Encoder related events
    RtcPalVideoEvent_EncoderReady,
    RtcPalVideoEvent_EncoderDynamicChangeBegin,
    RtcPalVideoEvent_EncoderDynamicChangeEnd,
    RtcPalVideoEvent_EncoderRestartPipeline,
    RtcPalVideoEvent_EncoderCriticalRuntimeFailure,
    RtcPalVideoEvent_ResolutionChangeFromSource
} RtcPalVideoEvent_e;


// =============================================================================
// Callback Functions
// =============================================================================


// -----------------------------------------------------------------------------
// PFNDESTROYRAWFRAME function prototype
//
// Raw Frame destruction prototype
// -----------------------------------------------------------------------------
typedef HRESULT(*PFNDESTROYRAWFRAME)
(
__in struct _RtcPalVideoRawFrame_t *pDecodedFrame
);


// =============================================================================
// Data Structures
// =============================================================================


// -----------------------------------------------------------------------------
// Video crop information
//
// Structure used to cropping coordinates for video stream.
// -----------------------------------------------------------------------------
typedef struct _RtcPalVideoCropInfo_t
{
    uint16_t usCropLeftOffset;      // offset from left edge
    uint16_t usCropRightOffset;     // offset from right edge
    uint16_t usCropTopOffset;       // offset from top edge
    uint16_t usCropBottomOffset;    // offset from bottom edge
} RtcPalVideoCropInfo_t;

// -----------------------------------------------------------------------------
// Uncompressed (Raw) Video Frame
//
// Structure used for delivery of uncompressed video frames through
// the preview.
// -----------------------------------------------------------------------------
typedef struct _RtcPalVideoRawFrame_t
{
    // Uncompressed format
    uint32_t fccVideoFormat;

    // Coded Resolution
    uint16_t usFrameWidth;
    uint16_t usFrameHeight;

    // Original Resolution (may be aspect ratio in H.264)
    uint16_t usOrgFrameWidth;
    uint16_t usOrgFrameHeight;

    // Rotation
    RtcPalVideoRotation_e eRotation;

    // Frame data
    uint8_t* pbFrameData;
    uint32_t ulDataLength;

    // Supplemental information
    int32_t  lStride;
    uint32_t ulBitsPerPixel;

    // Additional frame data parameters -- may be used depending on format
    uint8_t* pbFrameData2;
    int32_t  lStride2;
    uint8_t* pbFrameData3;
    int32_t  lStride3;

    // allows us to follow frame delivery
    void *pvMetricsInfo;

    // Time stamp
    uint64_t ullTimeStamp;

    // Stream mask.
    uint32_t ulStreamMask;

    // function pointer and context to destroy frame
    PFNDESTROYRAWFRAME pfnDestroyFrame;
    void *pvOriginatorFrameContext;

    RTCPAL_BOOL bValidCropInfo;
    RtcPalVideoCropInfo_t CropInfo;
    ScreenInfo_t * pScreenInfo;
} RtcPalVideoRawFrame_t;

// -----------------------------------------------------------------------------
// Frame Resolution
//
// Structure used to describe a subset of RtcPalVideoRawFrame_t
// including FCC and the resolution fields.
// -----------------------------------------------------------------------------
typedef struct _RtcPalVideoFrameResolution_t
{
    // Uncompressed format
    uint32_t fccVideoFormat;

    // Coded Resolution
    uint16_t usFrameWidth;
    uint16_t usFrameHeight;

    // Rotation
    RtcPalVideoRotation_e eRotation;

    // Original Resolution (may be aspect ratio in H.264)
    uint16_t usOrgFrameWidth;
    uint16_t usOrgFrameHeight;

    uint32_t ulBitsPerPixel;

}RtcPalVideoFrameResolution_t;

// -----------------------------------------------------------------------------
// Video Rendering Processing Metrics
//
// Structure used for description of decoder/render metric
// -----------------------------------------------------------------------------
typedef struct _RtcPalVarianceStats_t
{
    uint32_t ulSum;
    uint32_t ulCount;
    uint64_t ullSumOfSquare;
    uint32_t ulMin;
    uint32_t ulMax;
    uint32_t ulDroppedFrames;
} RtcPalVarianceStats_t;

// -----------------------------------------------------------------------------
// Video Processing Statistics
//
// Structure used for description of encoder or decoder real-time statistics.
// -----------------------------------------------------------------------------
typedef struct _RtcPalVideoProcessingStats_t
{
    RtcPalVarianceStats_t latency;
    RtcPalVarianceStats_t interFrameGap;
} RtcPalVideoProcessingStats_t;


#endif
