#pragma once
/* --

Copyright (c) Microsoft Corporation. All rights reserved.

Module Name:

    rtcpal_apis_video_source.h

Abstract:

    PAL Video Source interface

Specification:

    http://livecommteam/sites/main/media/W15/Media%20Stack%20W15%20Plan%20Tracking/W15%20Video%20PAL%20Interface%20Design%20Doc.docx

--*/

#ifndef __RTCPAL_APIS_VIDEO_SOURCE_H__
#define __RTCPAL_APIS_VIDEO_SOURCE_H__

// Custom HRESULT definitions
#define         E_MJPEGTOLERATE          0xA000FF00
#define         E_MJPEGFALLBACK          0xA000FF01


// Callback from RSM to VSCA
typedef HRESULT (*PFNRSMCALLBACKTOVSCA)(HANDLE, HANDLE);

// =============================================================================
// Video Source
// =============================================================================

// Video capture source type (Capture Engine, Media Capture, DShow, etc..)
typedef enum _RtcPalVideoCaptureSourceType_e
{
    RtcPalVideoCaptureSourceType_Start = 0,
    RtcPalVideoCaptureSourceType_MediaCapture = RtcPalVideoCaptureSourceType_Start,
    RtcPalVideoCaptureSourceType_CaptureEngine,
    RtcPalVideoCaptureSourceType_DirectShow,
    RtcPalVideoCaptureSourceType_WindowsPhone,
    RtcPalVideoCaptureSourceType_Android,
    RtcPalVideoCaptureSourceType_iOS,
    RtcPalVideoCaptureSourceType_OSX,
    RtcPalVideoCaptureSourceType_End
} RtcPalVideoCaptureSourceType_e;

typedef enum _RtcPalVideoEncoderType_e
{
    RtcPalVideoEncoderType_Start = 0,
    RtcPalVideoEncoderType_InBoxH264 = RtcPalVideoEncoderType_Start,
    RtcPalVideoEncoderType_InBoxVC1,
    RtcPalVideoEncoderType_SoftwareH264,
    RtcPalVideoEncoderType_SoftwareVC1,
    RtcPalVideoEncoderType_H264Camera,
    RtcPalVideoEncoderType_End
} RtcPalVideoEncoderType_e;

// ----------------------------------------------------------------------------
// RtcPalVideoProcessorType
//
// MjpegDecoder/CC/Resizer
//-----------------------------------------------------------------------------
typedef enum _RtcPalVideoProcessorType
{
    RtcPalVideoProcessorType_NotAssigned = 00,
    RtcPalVideoProcessorType_MjpegDecoder = 10,
    RtcPalVideoProcessorType_ColorConverter = 20,
    RtcPalVideoProcessorType_Resizer = 30
} RtcPalVideoProcessorType;


// ----------------------------------------------------------------------------
// RtcPalVideoProcessorSubtype
//
// Hardware/Software
//-----------------------------------------------------------------------------
typedef enum _RtcPalVideoProcessorSubtype
{
    RtcPalVideoProcessorSubtype_NotAssigned = 0,
    RtcPalVideoProcessorSubtype_Hardware = 1,
    RtcPalVideoProcessorSubtype_Software = 2,
    RtcPalVideoProcessorSubtype_InboxSoftware = 3
} RtcPalVideoProcessorSubtype;

// ----------------------------------------------------------------------------
// RtcPalVideoProcessorFailureType
//
// Video processor failure error code
//-----------------------------------------------------------------------------

typedef enum _RtcPalVideoProcessorFailureType
{
    RtcPalVideoProcessorFailureType_NotAssigned = 0,
    RtcPalVideoProcessorFailureType_MjpegHwInit = 1,
    RtcPalVideoProcessorFailureType_MjpegSwInit = 2,
    RtcPalVideoProcessorFailureType_MjpegHwDecode = 3,
    RtcPalVideoProcessorFailureType_MjpegSwDecode = 4,
    RtcPalVideoProcessorFailureType_ResizeHwInit = 5,
    RtcPalVideoProcessorFailureType_ResizeSwInit = 6,
    RtcPalVideoProcessorFailureType_ResizeHwResize = 7,
    RtcPalVideoProcessorFailureType_ResizeSwResize = 8
} RtcPalVideoProcessorFailureType;

// ----------------------------------------------------------------------------
// RtcPalVideoProcessorTestApiType
//
// Enum for choosing the correct test API from the video processor 
// test api entry point
//-----------------------------------------------------------------------------
typedef enum _RtcPalVideoProcessorTestApiType
{
    RtcPalVideoProcessorTestApiType_QueryMjpegMetrics,
    RtcPalVideoProcessorTestApiType_ForceMjpegFallback,
    RtcPalVideoProcessorTestApiType_SetVPCapability
} RtcPalVideoProcessorTestApiType;

// ----------------------------------------------------------------------------
// Mjpeg Decoder metrics container
//
// Container holding all the metrics from the Mjpeg Decoder
//-----------------------------------------------------------------------------
typedef struct _RtcPalVideoProcessorMjpegMetrics_t
{
    // Decoder Type
    RtcPalVideoProcessorSubtype eDecoderType;

    // Internal Format
    uint32_t fccInternalFormat;

    // Init/Decode times
    FLOAT fInitTime;
    FLOAT fDecodeTime_CurrFrame;
    FLOAT fDecodeTime_AvgWindow;
    FLOAT fDecodeTime_AvgTotal;

    // Error codes
    RtcPalVideoProcessorFailureType eFailureType;
    HRESULT hrFailure;

    // HMFT GUID
    GUID gHmftGuid;

} RtcPalVideoProcessorMjpegMetrics_t;

// ----------------------------------------------------------------------------
// Video Processor Capabilities
//
// Type used to describe a source's video processor capabilities
//-----------------------------------------------------------------------------
typedef struct _RtcPalVideoProcessorCapability_t
{
    int32_t id;
    RtcPalVideoProcessorType type;
    RtcPalVideoProcessorSubtype subtype;
    int32_t MBPR;
} RtcPalVideoProcessorCapability_t;

// ----------------------------------------------------------------------------
// RtcPalVideoSourceGetSupportedFormatList
// 
// Query a list of supported capture Video Formats from the Video Source 
// object. If the array is of insufficient length, pulCount will return 
// the number of supported Video Formats but the array will not be populated.
// ----------------------------------------------------------------------------
RTC_CLASS_LINKAGE
HRESULT
RtcPalVideoSourceGetSupportedFormatList(
    __in            HPALVIDEOSOURCE hVideoSource,
    __in            RTCPAL_BOOL fHardwareEnabled,
    __out           RtcPalVideoFormat_t* pVideoFormats,
    __inout         uint32_t* pulCount
);


// ----------------------------------------------------------------------------
// RtcPalVideoSourceGetCurrentFormat
// 
// Query the current capture Video Format from the Video Source object.
// ----------------------------------------------------------------------------
RTC_CLASS_LINKAGE
HRESULT
RtcPalVideoSourceGetCurrentFormat(
    __in            HPALVIDEOSOURCE hVideoSource,
    __in            RTCPAL_BOOL fHardwareEnabled,
    __out           RtcPalVideoFormat_t* pVideoFormat
);


// ----------------------------------------------------------------------------
// RtcPalVideoSourceSetCurrentFormat
// 
// Set the current capture Video Format of the Video Source object.
// ----------------------------------------------------------------------------
RTC_CLASS_LINKAGE
HRESULT
RtcPalVideoSourceSetCurrentFormat(
    __in            HPALVIDEOSOURCE hVideoSource,
    __in            const RtcPalVideoFormat_t* pHardwareVideoFormat,
    __in            const RtcPalVideoFormat_t* pSoftwareVideoFormat
);


// ----------------------------------------------------------------------------
// RtcPalVideoSourceSetVideoCropInfo
// 
// Set the most up-to-date smart crop information on the Video Source object.
// ----------------------------------------------------------------------------
RTC_CLASS_LINKAGE
HRESULT
RtcPalVideoSourceSetVideoCropInfo(
    /* __in    */   HPALVIDEOSOURCE hVideoSource,
    /* __in    */   RTCPAL_BOOL bValid,
    /* __in    */   float fLeft,
    /* __in    */   float fTop,
    /* __in    */   float fRight,
    /* __in    */   float fBottom
);

// ----------------------------------------------------------------------------
// RtcPalVideoSourceCreatePreview
// 
// Create a Video Preview for the Video Source object.
// ----------------------------------------------------------------------------
RTC_CLASS_LINKAGE
HRESULT
RtcPalVideoSourceCreatePreview(
    __in            HPALVIDEOSOURCE hVideoSource,
    __in            PVIDEOPREVIEWCALLBACK fnCallback,
    __in            void* pvCallbackContext,
    __in            const void* pvRenderContext,
    __out           HPALVIDEOPREVIEW* phVideoPreview
);

// ----------------------------------------------------------------------------
// RtcPalVideoSourceDeletePreview
// 
// Delete a Video Preview belonging to the Video Source object.
// ----------------------------------------------------------------------------
RTC_CLASS_LINKAGE
HRESULT
RtcPalVideoSourceDeletePreview(
    __in            HPALVIDEOSOURCE hVideoSource,
    __in            HPALVIDEOPREVIEW hVideoPreview
);


// ----------------------------------------------------------------------------
// RtcPalVideoSourceQueryEncodeCapabilities
// 
// Query the set of encode capabilities for available encoders applicable to 
// this source PAL Video Source. This list is matched to current format 
// and includes any compliant encode capability of H264 cameras.
// ----------------------------------------------------------------------------
RTC_CLASS_LINKAGE
HRESULT
RtcPalVideoSourceQueryEncodeCapabilities(
    __in            HPALVIDEOSOURCE hVideoSource,
    __out           MLE_CapabilityEx* pCapabilities,
    __inout         uint32_t* pulCount
);

// ----------------------------------------------------------------------------
// RtcPalVideoSourceQueryVideoProcessorCapabilities
// 
// Query set of videoprocessor capabilities including Mjpeg, color conversion
// and resizing
// ----------------------------------------------------------------------------
RTC_CLASS_LINKAGE
HRESULT
RtcPalVideoSourceQueryVideoProcessorCapabilities(
    __in            HPALVIDEOSOURCE hVideoSource,
    __in            PFNRSMCALLBACKTOVSCA pfnCallback,
    __inout         HANDLE hCallbackContext,
    __out           RtcPalVideoProcessorCapability_t* astVideoProcessorCapability,
    __inout         uint32_t* pulCount
);

// ----------------------------------------------------------------------------
// RtcPalVideoSourceCallVideoProcessorTestApi
// 
// Query set of Mjpeg Decoder metrics including decoding time, init time and 
// error codes for failures
// ----------------------------------------------------------------------------
RTC_CLASS_LINKAGE
HRESULT
RtcPalVideoSourceCallVideoProcessorTestApi(
    __in            HPALVIDEOSOURCE hVideoSource,
    __in            RtcPalVideoProcessorTestApiType eApiType, 
    __inout         BYTE* pbBlob, 
    __in            DWORD dwBlobSize
);

// ----------------------------------------------------------------------------
// RtcPalVideoSourceIsScreenScraper
// 
// Query whether this source is a screen scraper
// ----------------------------------------------------------------------------
EXTERN_C
HRESULT
RtcPalVideoSourceIsScreenScraper(
    __in            HPALVIDEOSOURCE hVideoSource,
    __out           RTCPAL_BOOL* pbScreenContent
);

// ----------------------------------------------------------------------------
// RtcPalVideoSourceCreateEncoder
// 
// Create a Video Encoder for the Video Source object.
// ----------------------------------------------------------------------------
HRESULT
RtcPalVideoSourceCreateEncoder(
    __in            HPALVIDEOSOURCE hVideoSource,
    __in            PVIDEOENCODERCALLBACK fnCallback,
    __in            void* pvCallbackContext,
    __in            uint32_t ulEncoderID,
    __out           HPALVIDEOENCODER* phVideoEncoder
);


// ----------------------------------------------------------------------------
// RtcPalVideoSourceDeleteEncoder
// 
// Delete a Video Encoder belonging to the Video Source object.
// ----------------------------------------------------------------------------
RTC_CLASS_LINKAGE
HRESULT
RtcPalVideoSourceDeleteEncoder(
    __in            HPALVIDEOSOURCE hVideoSource,
    __in            HPALVIDEOENCODER hVideoEncoder
);


// ----------------------------------------------------------------------------
// RtcPalVideoSourceQueryAnalyzerCapabilities
// 
// Query the set of analyzer capabilities for available analyzers
// ----------------------------------------------------------------------------
RTC_CLASS_LINKAGE
HRESULT
RtcPalVideoSourceQueryAnalyzerCapabilities(
    __in            HPALVIDEOSOURCE hVideoSource,
    __out           VA_CapabilityEx* pCapabilities,
    __inout         uint32_t* pulCount
);


// ----------------------------------------------------------------------------
// RtcPalVideoSourceCreateAnalyzer
// 
// Create a Video Analyzer for the Video Source object.
// ----------------------------------------------------------------------------
HRESULT
RtcPalVideoSourceCreateAnalyzer(
    __in            HPALVIDEOSOURCE hVideoSource,
    __out           HPALVIDEOANALYZER* phVideoAnalyzer
);


// ----------------------------------------------------------------------------
// RtcPalVideoSourceDeleteAnalyzer
// 
// Delete a Video Analyzer belonging to the Video Source object.
// ----------------------------------------------------------------------------
RTC_CLASS_LINKAGE
HRESULT
RtcPalVideoSourceDeleteAnalyzer(
    __in            HPALVIDEOSOURCE hVideoSource,
    __in            HPALVIDEOANALYZER hVideoAnalyzer
);

// ----------------------------------------------------------------------------
// RtcPalVideoSourceSetFramerateDynamic
// 
// Set new frame rate.
// ----------------------------------------------------------------------------
HRESULT
RtcPalVideoSourceSetFramerateDynamic(
    __in       HPALVIDEOSOURCE hVideoSource,
    __in       float fNewRate
);


// ----------------------------------------------------------------------------
// RtcPalVideoSourceStart
// 
// Start Capturing
// ----------------------------------------------------------------------------
RTC_CLASS_LINKAGE
HRESULT
RtcPalVideoSourceStart(
    __in            HPALVIDEOSOURCE hVideoSource
);


// ----------------------------------------------------------------------------
// RtcPalVideoSourceStop
// 
// Stop Capturing
// ----------------------------------------------------------------------------
RTC_CLASS_LINKAGE
HRESULT
RtcPalVideoSourceStop(
    __in            HPALVIDEOSOURCE hVideoSource
);


// ----------------------------------------------------------------------------
// RtcPalVideoSourceRestart
// 
// Apply pending format change and restart capturing
// ----------------------------------------------------------------------------
HRESULT
RtcPalVideoSourceRestart(
    __in            HPALVIDEOSOURCE hVideoSource
);


// ----------------------------------------------------------------------------
// RtcPalVideoSourceDeviceControlState
// 
// Get the source device control state.
// ----------------------------------------------------------------------------
RTC_CLASS_LINKAGE
HRESULT
RtcPalVideoSourceControlGetState(
    __in HPALVIDEOSOURCE hVideoSource,
    __in RtcPalVideoControl_e eControl,
    __inout RtcPalVideoControlState_t* pState
    );



// ----------------------------------------------------------------------------
// RtcPalVideoSourceGetLastFrameInfo
// 
// Get the resolution of the last frame.
// ----------------------------------------------------------------------------
RTC_CLASS_LINKAGE
HRESULT
RtcPalVideoSourceGetLastFrameInfo(
    __in HPALVIDEOSOURCE hVideoSource,
    __out RtcPalVideoSourceFrameResolution_t * pFrameInfo
    );

// ----------------------------------------------------------------------------
// RtcPalVideoSourceDeviceControlState
// 
// Get the current value set for the control.
// ----------------------------------------------------------------------------
RTC_CLASS_LINKAGE
HRESULT
RtcPalVideoSourceControlGetValue(
    __in HPALVIDEOSOURCE hVideoSource,
    __in RtcPalVideoControl_e eControl,
    __out double* pValue
    );

// ----------------------------------------------------------------------------
// RtcPalVideoSourceDeviceControlState
// 
// Set the current value to the new value.  NOTE:  Attempting to set the value
// outside of the range specified in the RtcPalVideoControlState_t structure
// obtained above, will result in an error.
// ----------------------------------------------------------------------------
RTC_CLASS_LINKAGE
HRESULT
RtcPalVideoSourceControlSetValue(
    __in HPALVIDEOSOURCE hVideoSource,
    __in RtcPalVideoControl_e eControl,
    __in double Value
    );

// ----------------------------------------------------------------------------
// RtcPalVideoSourceDeviceControlState
// 
// Get the auto mode set on this control.  If auto mode is not supported, this
// will return an error..
// ----------------------------------------------------------------------------
RTC_CLASS_LINKAGE
HRESULT
RtcPalVideoSourceControlGetAutoMode(
    __in HPALVIDEOSOURCE hVideoSource,
    __in RtcPalVideoControl_e eControl,
    __out int32_t* pValue
    );

// ----------------------------------------------------------------------------
// RtcPalVideoSourceDeviceControlState
// 
// Set the auto mode on this control.  If auto mode is not supported, this
// will return an error..
// ----------------------------------------------------------------------------
RTC_CLASS_LINKAGE
HRESULT
RtcPalVideoSourceControlSetAutoMode(
    __in HPALVIDEOSOURCE hVideoSource,
    __in RtcPalVideoControl_e eControl,
    __in int32_t Value
    );

// -----------------------------------------------------------------------------
// RtcPalVideoSourceGetStatistics
// 
// Query the Video Source for real-time processing statistics.
// -----------------------------------------------------------------------------
RTC_CLASS_LINKAGE
HRESULT
RtcPalVideoSourceGetStatistics(
    __in            HPALVIDEOSOURCE hVideoSource,
    __out           RtcPalVideoSourceStats_t* pStats
);


// -----------------------------------------------------------------------------
// RtcPalVideoSourceResetStatistics
// 
// Reset the statistics of the Video Source.
// -----------------------------------------------------------------------------
RTC_CLASS_LINKAGE
HRESULT
RtcPalVideoSourceResetStatistics(
    __in            HPALVIDEOSOURCE hVideoSource
);


// -----------------------------------------------------------------------------
// RtcPalVideoSourceIsHardwareAccelerated
// 
// Check if the source is using hardware acceleration
// -----------------------------------------------------------------------------
RTC_CLASS_LINKAGE
HRESULT
RtcPalVideoSourceIsHardwareAccelerated(
    __in            HPALVIDEOSOURCE hVideoSource,
    __out           RTCPAL_BOOL* pbResult
);


// -----------------------------------------------------------------------------
// RtcPalVideoSourceSetInternalFormat
// 
// Set the (non-binding) preferred internal format for MJPG output and RSM tree
// -----------------------------------------------------------------------------
RTC_CLASS_LINKAGE
HRESULT
RtcPalVideoSourceSetInternalFormat(
    __in            HPALVIDEOSOURCE hVideoSource,
    __in            DWORD dwFormatFCC
);


// -----------------------------------------------------------------------------
// RtcPalVideoSourceSetRotationAngle
// 
// Set the initial rotation angle of the video produced by the source
// -----------------------------------------------------------------------------
RTC_CLASS_LINKAGE
HRESULT
RtcPalVideoSourceSetRotationAngle(
    __in            HPALVIDEOSOURCE hVideoSource,
    __in            DWORD dwClockwiseRotationAngle
);


// Create an encoder provided by the underlying platform. This is either an
// inbox or H264 camera encoder.
EXTERN_C
HRESULT
RtcPalVideoSourceCreatePlatformEncoder(
    __in HPALVIDEOSOURCE hVideoSource,
    __in uint32_t uiEncoderId,
    __in_ecount(1) const MLE_Config * pConfig,
    __in struct IRtcPalVideoEncoderCallback * pCallback,
    __out_ecount(1) struct IRtcPalVideoEncoderInterface ** ppEncoder
    );

#endif // __RTCPAL_APIS_VIDEO_SOURCE_H__
