/*++

Module Name:

    rtcpal_atomic.h

Abstract:

    Implement various atomic/interlocked operations with WIN32 like signature.
    Currently, this implementation is based on GCC4 built-in atomic functions.

Author:

Revision History:

--*/

#if __GNUC__ < 4
#error "Only GCC 4 and above is supported"
#endif


#ifdef __cplusplus
extern "C" {
#endif

#if !defined(RTCPAL_ARCH_X86) && !defined(RTCPAL_ARCH_X64)
static
LONG
FORCEINLINE
RtcPalInterlockedExchange(volatile LONG * target, LONG newv)
{
    long oldv;

    do {
        oldv = *target;
    } while(!__sync_bool_compare_and_swap(target, oldv, newv));

    return oldv;
}

static
PVOID
FORCEINLINE
RtcPalInterlockedExchangePointer(volatile PVOID * target, PVOID newv)
{
    PVOID oldv;

    do {
        oldv = *target;
    } while(!__sync_bool_compare_and_swap(target, oldv, newv));

    return oldv;
}
#else
#define RtcPalInterlockedExchange           __sync_lock_test_and_set
#define RtcPalInterlockedExchangePointer    __sync_lock_test_and_set
#endif

//llvm-GCC crashes when it sees 64bit __sync_bool_compare_and_swap
#if defined(_UNIX_IPHONE)
#define RtcPalCompareAndSwap64(ptr, oldv, newv) OSAtomicCompareAndSwap64Barrier(oldv, newv, ptr)
#elif HAVE_LIBKERN_OSATOMIC_H
#define RtcPalCompareAndSwap64(ptr, oldv, newv) OSAtomicCompareAndSwap64(oldv, newv, ptr)
#else
#define RtcPalCompareAndSwap64 __sync_bool_compare_and_swap
#endif

#define RtcPalInterlockedExchangeAdd        __sync_fetch_and_add
#define RtcPalInterlockedCompareExchange(Target,NewV,OldV)  __sync_val_compare_and_swap(Target,OldV,NewV)
#define RtcPalInterlockedCompareExchangePointer(Target,NewV,OldV)  (void *)__sync_val_compare_and_swap((volatile uintptr_t*)Target,(uintptr_t)OldV,(uintptr_t)NewV)

#define RtcPalInterlockedCompareExchange64      RtcPalInterlockedCompareExchange

#ifdef __APPLE__
    #if LONG_MAX == INT32_MAX
        // MACBU shim define LONG as long instead of int32_t
        #define RtcPalInterlockedDecrement(Target)  \
                (LONG)OSAtomicDecrement32((volatile int32_t *)Target)
        #define RtcPalInterlockedIncrement(Target)  \
                (LONG)OSAtomicIncrement32((volatile int32_t *)Target)
    #else
        RTCPAL_C_ASSERT(LONG_MAX == INT64_MAX);
        #define RtcPalInterlockedDecrement(Target)  \
                (LONG)OSAtomicDecrement64(volatile int32_t *)Target)
        #define RtcPalInterlockedIncrement(Target)  \
                (LONG)OSAtomicIncrement64(volatile int32_t *)Target)
    #endif
#elif defined(ANDROID)
    #define RtcPalInterlockedIncrement(Target)  (__atomic_inc((volatile int32_t *)Target) + 1)
    #define RtcPalInterlockedDecrement(Target)  (__atomic_dec((volatile int32_t *)Target) - 1)
#else
    #define RtcPalInterlockedIncrement(Target)  __sync_add_and_fetch(Target,1)
    #define RtcPalInterlockedDecrement(Target)  __sync_add_and_fetch(Target,-1)
#endif


#undef InterlockedExchange
#define InterlockedExchange(target, newv)                 RtcPalInterlockedExchange((volatile LONG *)target, (LONG)newv)
#undef InterlockedExchangePointer
#define InterlockedExchangePointer          RtcPalInterlockedExchangePointer

#undef InterlockedCompareExchangePointer
#define InterlockedCompareExchangePointer   RtcPalInterlockedCompareExchangePointer

#undef InterlockedExchangeAdd
#define InterlockedExchangeAdd              RtcPalInterlockedExchangeAdd  

#undef InterlockedCompareExchange
#define InterlockedCompareExchange          RtcPalInterlockedCompareExchange

#undef InterlockedCompareExchange64
#define InterlockedCompareExchange64        RtcPalInterlockedCompareExchange64

#undef InterlockedDecrement
#define InterlockedDecrement                RtcPalInterlockedDecrement

#undef InterlockedIncrement
#define InterlockedIncrement                RtcPalInterlockedIncrement

#ifdef __cplusplus
};
#endif

