
#ifndef __RTCPAL_BASE_H__
#define __RTCPAL_BASE_H__

#if defined(__cplusplus)

    #if _MSC_VER >= 1700 // expand for other compilers

        // Visual C++ 2012 (also known as Dev11) and above supports a vast
        // subset of C++11. For a complete list of features, see:
        // http://msdn.microsoft.com/en-us/library/vstudio/hh567368.aspx
        // The same features can often be found in non-Microsoft compilers.
        // These new features can be used conditionally, with alternate
        // implementations provided for backward compatibility where
        // compiling with less advanced toolchains is required. Once C++11
        // support becomes universal, said conditionals can be removed.

        // More granular definitions can be added as required
        #define RTCPAL_SUPPORT_CPP11_LANG   1
        #define RTCPAL_SUPPORT_CPP11_STD    1

    #endif

#endif

#if RTCPAL_SUPPORT_CPP11_LANG && RTCPAL_SUPPORT_CPP11_STD

    // Generic RAII wrapper.
    // It is preferable to use specialized smart pointers,
    // but when all else fails, this is the way to go.
    // This is only really useful with lambdas. For instance:
    // {
    //     auto p = SomeAllocator();
    //     RTCPAL_PREPARE_CLEANUP([p](){ SomeDeallocator(p); });
    //     // ...
    // } // deallocation happens here

    template <class ActionT>
    class RtcPalRAIIWrapper
    {
    public:
        RtcPalRAIIWrapper(ActionT action) : m_action(action) { }
        ~RtcPalRAIIWrapper() { m_action(); }
    private:
        ActionT m_action;
    };

    #define RTCPAL_GLUE_(lit1, lit2) lit1##lit2
    #define RTCPAL_GLUE(lit1, lit2) RTCPAL_GLUE_(lit1, lit2)
    #define RTCPAL_PREPARE_CLEANUP(lmbd) \
            RtcPalRAIIWrapper<std::function<void()>> RTCPAL_GLUE(rtcPalRAIIWrapper_, __LINE__) (lmbd)

#endif

#ifdef __cplusplus
extern "C" {
#endif

typedef uintptr_t   RTCPAL_HANDLE;

uint32_t
RTCAPI
RtcPalStartup(
    void
    );

uint32_t
RTCAPI
RtcPalGetTempFolderW(
    __out_ecount(nSize) WCHAR     *lpDst,
    __in                uint32_t   nSize
);



uint32_t
RTCAPI
RtcPalGetTempFolderA(
    __out_ecount(nSize) char      *lpDst,
    __in                uint32_t   nSize
);

/*++

Description
    Return the folder path in which application can persistent its state.
    The folder path will be null terminated. It  will also include a trailing
    slash (POSIX style path) or backslash (DOS/WINDOWS style path).

Return
    Upon success, the return value is the length, in characters, of the string
    copied into lpBuffer, not including the terminating NULL character. If the
    return value is greater than nBufferLength, the return value is the lenght,
    in characters, of the buffer required to hold the path and the null terminator.

    Upon failure, the return value is zero. Use GetLastError to fetch the error
    code.

--*/
EXTERN_C
uint32_t
RTCAPI
RtcPalGetLocalStateFolderW(
    _In_ DWORD nBufferLength,
    _Out_writes_to_opt_(nBufferLength, return + 1) LPWSTR lpBuffer
    );

RTCPAL_ERROR
RTCAPI
RtcPalGetLogFolderW(
    _Inout_ DWORD *nBufferLength,
    _Out_ LPWSTR lpBuffer
    );

RTCPAL_ERROR
RTCAPI
RtcPalGetLogFileNamePrefixW(
    _Inout_ DWORD *nBufferLength,
    _Out_ LPWSTR lpBuffer
    );

void
RTCAPI
RtcPalCleanup(
    void
    );

typedef enum RtcPalPlatformProfile {
    // Optimize PAL behavior for client
    RtcPalPlatformProfile_Client = 0,

    // Optimize PAL behavior for server
    RtcPalPlatformProfile_Server
} RtcPalPlatformProfile;

void
RTCAPI
RtcPalSetPlatformProfile(
    RtcPalPlatformProfile profile
    );

extern
uint32_t
RTCAPI
RtcPalGetLastError();

extern
void
RTCAPI
RtcPalSetLastError(uint32_t Error);

extern
int
RTCAPI
RtcPalGetLastLinuxErrno();

extern
void
RTCAPI
RtcPalSetLastLinuxErrno();

void
RTCAPI
RtcPalDbgVPrint(const char * fmt, va_list ap);

void
RTCPAL_CDECL
RtcPalDbgPrint(const char * fmt, ...);

void
RTCAPI
RtcPalAssert(const char * Assertion, const char * fn, uint32_t LineNumber);


//
// Write the function return addresses of the current call stack to the array of pointers
// referenced by array. At most, size pointers are written. 
// The number of pointers actually written to array is returned.
//
uint32_t
RTCAPI
RtcPalBacktrace(
    __out_ecount(size) PVOID * array,
    __in uint32_t size,
    __in uint32_t skip
    );

void
RTCAPI
RtcPalPrintStackTrace(
    __in uint32_t Count,
    __in_ecount(Count) const PVOID * Stack
    );

//
// Debug Symbol API
//
uint32_t
RTCAPI
RtcPalSymStartup(
    void
    );

void
RTCAPI
RtcPalSymCleanup(
    void
    );

struct RtcPalDbgSym {
    const char * ImageName;
    void * ImageBase;
    const char * SymbolName;
    uint32_t Displacement;
};

RTCPAL_ERROR
RTCAPI
RtcPalGetSymbol(
    __out_ecount(1) struct RtcPalDbgSym * SymInfo,
    __in const void * Address
    );

void
RTCAPI
RtcPalFreeSymbol(
    __in_ecount(1) struct RtcPalDbgSym * SymInfo
    );

__out_bcount_opt(size)
void *
RTCAPI
RtcPalAllocAlignedMemoryWithTag(
    size_t size,
    size_t alignment,
    uint32_t tag
    );

__out_bcount_opt(size)
void *
RTCAPI
RtcPalAllocMemoryWithTag(
    size_t size,
    uint32_t tag
    );

void
RTCAPI
RtcPalFreeMemoryWithTag(
    void * p,
    uint32_t tag
    );

#ifdef __cplusplus
    // C++ has a native bool type, and failure to use it in
    // conditional expressions results in warnings even at
    // moderate compiler warning levels.
    typedef bool LANG_BOOL;
    #define LANG_TRUE true
    #define LANG_FALSE false
#else
    // There's no bool in C90, although support was added in C99
    typedef int LANG_BOOL;
    #define LANG_TRUE 1
    #define LANG_FALSE 0
#endif

#ifdef NDEBUG
    #define RTCPAL_ASSERT(exp)  ((void) 0)
    #define RTCPAL_VERIFY(exp)  ((void)((LANG_BOOL)!!(exp)))
#else
    #define RTCPAL_ASSERT_NONVOID__(exp)      \
        ((LANG_BOOL)!!((!(exp)) ? \
        (RtcPalAssert( #exp, __FILE__, __LINE__),LANG_FALSE) : \
        LANG_TRUE))
    #define RTCPAL_ASSERT(exp)  ((void)(RTCPAL_ASSERT_NONVOID__(exp)))
    #define RTCPAL_VERIFY(exp)  ((void)(RTCPAL_ASSERT_NONVOID__(exp)))
#endif

RTCPAL_BOOL
RTCAPI
RtcPalGetOSName(
    __out_ecount(cchMax) WCHAR * pszName,
    __in uint32_t cchMax
    );

RTCPAL_BOOL
RTCAPI
RtcPalIsWinModern();

RTCPAL_BOOL
RTCAPI
RtcPalGetUserName(
    __out_ecount_part_opt(*pcbBuffer, *pcbBuffer) LPWSTR lpBuffer,
    __inout uint32_t *pcbBuffer
    );

RTCPAL_BOOL
RTCAPI
RtcPalGetWinSATProcessorScore(
    __out_ecount(1) float * pfScore
    );

RTCPAL_BOOL
RTCAPI
RtcPalGetWinSATVideoEncodeScore(
    __out_ecount(1) float * pfScore
    );

RTCPAL_BOOL
RTCAPI
RtcPalGetWinSATGraphicsScore(
    __out_ecount(1) float * pfScore
    );

RTCPAL_BOOL
RTCAPI
RtcPalGetPowerSchemeFriendlyName(
    __out_ecount(dwFriendlyNameLength) wchar_t * pwszFriendlyName,
    uint32_t dwFriendlyNameLength
    );

void
RTCAPI
RtcPalSleep(
    __in uint32_t dwMilliseconds
    );

#ifdef __cplusplus
typedef struct RtcPalEvent * RtcPalEventHandle;
typedef struct RtcPalThread * RtcPalThreadHandle;
typedef struct RtcPalMutex * RtcPalMutexHandle;
typedef struct RtcPalSemaphore * RtcPalSemaphoreHandle;

typedef union RtcPalWaitableHandle {
    uintptr_t _uint;
    void * _void;
    RtcPalMutexHandle _mutex;
    RtcPalSemaphoreHandle _sem;
    RtcPalEventHandle _event;
    RtcPalThreadHandle _thread;

    RtcPalWaitableHandle() {}
    RtcPalWaitableHandle(RtcPalMutexHandle mu): _mutex(mu) {}
    RtcPalWaitableHandle(RtcPalSemaphoreHandle sem): _sem(sem) {}
    RtcPalWaitableHandle(RtcPalEventHandle ev): _event(ev) {}
    RtcPalWaitableHandle(RtcPalThreadHandle th): _thread(th) {}
    RtcPalWaitableHandle(void * v): _void(v) {}
    RtcPalWaitableHandle(uintptr_t v): _uint(v) {}
    RtcPalWaitableHandle(int v): _uint(uintptr_t(v)) {}
    operator const void *() const { return _void; }
    operator uintptr_t() const { return _uint; }
    operator int() const { return int(_uint); }

    /*
    operator void *() const { return _void; }
    */
    operator RtcPalEventHandle() const { return _event; }
    operator RtcPalThreadHandle() const { return _thread; }
} RtcPalWaitableHandle;
#else
typedef void * RtcPalWaitableHandle;
typedef RtcPalWaitableHandle RtcPalEventHandle;
typedef RtcPalWaitableHandle RtcPalThreadHandle;
typedef RtcPalWaitableHandle RtcPalMutexHandle;
typedef RtcPalWaitableHandle RtcPalSemaphoreHandle;
#endif

RTCPAL_C_ASSERT(sizeof(RtcPalWaitableHandle) == sizeof(void *));

RTCPAL_BOOL
RTCAPI
RtcPalCloseWaitableHandle(
    __in RtcPalWaitableHandle handle
    );

uint32_t
RTCAPI
RtcPalWaitForSingleObject(
    __in RtcPalWaitableHandle hHandle,
    __in uint32_t dwMilliseconds
    );

//
// Replace WIN32 WaitForMultipleObjects. Support only WaitAny. WaitAll isn't
// supported because WINCE doesn't natively support "waitall".
//
// Media stack uses "waitall" only for waiting for all threads to exit. For
// this specific usage, we can emulate in a more efficient way since this
// usage doesn't really need to be "atomic" 
//
uint32_t
RTCAPI
RtcPalWaitForAnyObject(
    __in uint32_t nCount,
    __in_ecount(nCount) const RtcPalWaitableHandle * lpHandles,
    __in uint32_t dwMilliseconds
    );

// COM friendly wait for Windows Platform. On all other platforms,
// this API is just an alias of RtcPalWaitForAnyObject
// This API is for mediamgr to consume since it can be invoked from STA
// thread (owned by Lync client).
//
// When calling from MTA threads on Windows, this API should behave the
// same as RtcPalWaitForAnyObject except that this API burns a few cycles
// for checking if the thread is MTA or not.
#if defined(_WIN32_WINNT)
uint32_t
RTCAPI
RtcPalCoWaitForAnyObject(
    __in uint32_t nCount,
    __in_ecount(nCount) const RtcPalWaitableHandle * lpHandles,
    __in uint32_t dwMilliseconds
    );
#else
    #undef RtcPalCoWaitForAnyObject
    #define RtcPalCoWaitForAnyObject    RtcPalWaitForAnyObject
#endif

//
// Wait for indefinitely until all threads to exit.
//  Replace CSystemCall::WaitForAllThreads in media stack
//
uint32_t
RTCAPI
RtcPalWaitForAllThreads(
    __in uint32_t nCount,
    __in_ecount(nCount) const RtcPalThreadHandle * lpHandles
    );

RtcPalEventHandle
RTCAPI
RtcPalCreateEvent(
    __in RTCPAL_BOOL bManualReset,
    __in RTCPAL_BOOL bInitialState
    );

RTCPAL_BOOL
RTCAPI
RtcPalSetEvent(
    RtcPalEventHandle event
    );

RTCPAL_BOOL
RTCAPI
RtcPalResetEvent(
    RtcPalEventHandle event
    );

__out_opt
RtcPalMutexHandle
RTCAPI
RtcPalCreateMutex(
    __in RTCPAL_BOOL bInitialOwner
    );

RTCPAL_BOOL
RTCAPI
RtcPalReleaseMutex(
    __in void * hMutex
    );

__out_opt
RtcPalSemaphoreHandle
RTCAPI
RtcPalCreateSemaphore(
    __in int32_t InitialCount,
    __in int32_t MaximumCount
    );

//
// TODO: replace LPLONG with int32_t *
// We leave it this way because "long" == LONG in MACBU header.
// GCC will warns when converting "pointer to long" to "pointer to int32_t"
//
RTCPAL_BOOL
RTCAPI
RtcPalReleaseSemaphore(
    __in RtcPalSemaphoreHandle hSemaphore,
    __in int32_t ReleaseCount,
    __out_opt LPLONG lpPreviousCount
    );

typedef enum RTCPAL_SCHEDULER_POLICY {
    // normal non-realtime policy
    RtcPalSchedulerPolicyNormal = 0,

    // the rest are for real time scheduler
    RtcPalSchedulerPolicyRealtimeAudio = 1,
    RtcPalSchedulerPolicyRealtimeProAudio = 2,
    RtcPalSchedulerPolicyRealtimePlayback = 3,
    RtcPalSchedulerPolicyRealtimeCapture = 4,
    RtcPalSchedulerPolicyRealtimeDistribution = 5,
    RtcPalSchedulerPolicyRealtimeLowLatency = 6,

    RtcPalSchedulerPolicyCount
} RTCPAL_SCHEDULER_POLICY;


// Bitmask for RTCPAL_THREAD_ATTR::ValidationFlags
#define RTCPAL_THREAD_VALIDATION_BLOCKING_CALL      0x00000001U     // Check blocking API call
#define RTCPAL_THREAD_VALIDATION_LOCK_DURATION      0x00000002U     // break if a thread hold a lock for too long

typedef struct RTCPAL_THREAD_ATTR {
    uint32_t StackSize;
    enum RTCPAL_SCHEDULER_POLICY SchedulerPolicy;
    int Priority;
    RTCPAL_BOOL DisablePriorityBoost;
    uint32_t ValidationFlags;
} RTCPAL_THREAD_ATTR, * PRTCPAL_THREAD_ATTR;

// Initialize thread attributes to system default value
void
RtcPalInitThreadAttr(
    __out PRTCPAL_THREAD_ATTR Attributes
    );

typedef uint32_t (RTCAPI * RTCPAL_THREAD_START_ROUTINE)(void * ThreadParameter);
RtcPalThreadHandle
RtcPalCreateThread(
    __in      RTCPAL_THREAD_START_ROUTINE StartAddress,
    __in_opt __deref __drv_aliasesMem void * Parameter,
    __in_opt const RTCPAL_THREAD_ATTR * Attributes
    );

RTCPAL_BOOL
RTCAPI
RtcPalSetSchedulerPolicy(
    enum RTCPAL_SCHEDULER_POLICY SchedulerPolicy
    );

RTCPAL_BOOL
RTCAPI
RtcPalGetExitCodeThread(
    __in RtcPalThreadHandle Thread,
    __out uint32_t * ExitCode
    );

RtcPalThreadHandle
RTCAPI
RtcPalGetCurrentThread(
    VOID
    );

int
RTCAPI
RtcPalGetThreadPriority(
    __in RtcPalThreadHandle Thread
    );

RTCPAL_BOOL
RTCAPI
RtcPalSetThreadPriority(
    __in RtcPalThreadHandle Thread,
    __in int nPriority
    );

RTCPAL_BOOL
RTCAPI
RtcPalSetPriorityClass(
    uint32_t PriorityClass
    );

uint32_t
RTCAPI
RtcPalGetPriorityClass(
    void
    );

// CPU information collected by CChannelInfo::PopulateAudioQMData
typedef struct RTCPAL_QMDATA_CPU {
    #define RTCPAL_QMDATA_CPU_BRAND_MAXLEN  32
    WCHAR Brand[RTCPAL_QMDATA_CPU_BRAND_MAXLEN];
    uint32_t Family;
    uint32_t Model;
    uint32_t InstructionSetBitmap;
    uint32_t NumberCores;
} RTCPAL_QMDATA_CPU, * PRTCPAL_QMDATA_CPU;

void
RTCAPI
RtcPalGetQMDataCPU(
    __out_ecount(1) PRTCPAL_QMDATA_CPU pQMData
    );

void
RTCAPI
RtcPalGetProcessorName(
    __out_ecount(cch) WCHAR * szName,
    __in uint32_t cch
    );

RTCPAL_BOOL
RTCAPI
RtcPalIsProcessorVirtualized(
    void
    );

// TODO: move this into rtcpal_x86.h since this is very x86 centric information.
typedef struct RTCPAL_PROCESSOR_TOPOLOGY {
    uint32_t LPPerPackage;      // Logical Processor per package
    uint32_t CoresPerPackage;   // Cores per package
    uint32_t PackageCount;      // Number of packages (sockets)
    uint32_t NUMACount;         // Number of NUMA nodes
} RTCPAL_PROCESSOR_TOPOLOGY, * PRTCPAL_PROCESSOR_TOPOLOGY;

void
RTCAPI
RtcPalGetProcessorTopology(
    __out_ecount(1) PRTCPAL_PROCESSOR_TOPOLOGY topo
    );

// Return the NUMA group which a CPU core belongs to
uint32_t
RTCAPI
RtcPalGetNUMANode(
    __in uint32_t CoreIndex
    );

RTCPAL_BOOL
RTCAPI
RtcPalIsLaptop(
    void
    );

RTCPAL_BOOL
RTCAPI
RtcPalQueryPerformanceCounter(
    __out int64_t * PerformanceCount
    );

double
RTCAPI
RtcPalPerfCounterToSeconds(
    __in int64_t v
    );

uint64_t
RTCAPI
RtcPalPerfCounterToNanoseconds(
    __in int64_t v
    );

double
RTCAPI
RtcPalGetTimeDouble(
    void
    );

//
// Enable/Disable RDTSC
// TODO: get rid of this function - it is very PC centric
//
void
RTCAPI
RtcPalEnableRDTSCTime(
    RTCPAL_BOOL bTestRDTSC
    );

//
// Return time in 100ns unit
//
uint64_t
RTCAPI
RtcPalGetTimeLongIn100ns(
    void
    );

// A less accurate version of RtcPalGetTimeLongIn100nsFast. PAL implementation
// should favor speed over accuracy
//
// Historical Information:
// LCCTIME introduced this one to work around expensive QPC issue under VM
// environment.
uint64_t
RTCAPI
RtcPalGetTimeLongIn100nsFast(
    void
    );

// Obtain the elapsed time as a ULONGLONG representing the 100s of
// nano-seconds between the initialization time and the time passed as
// a parameter (previously obtained from RtcPalGetElapsedTimeFromStart)
//
// TODO: get rid of this function. It shouldn't be in PAL.
uint64_t
RTCAPI
RtcPalGetElapsedTimeFromStart(
    uint64_t ullTime
    );

// return the private memory usage in MB for the current process
uint32_t
RTCAPI
RtcPalGetPrivateMemoryUsageInMB(
    void
    );

#ifndef RTCPAL_MAX_NATURAL_ALIGNMENT
    #define RTCPAL_MAX_NATURAL_ALIGNMENT sizeof(uint64_t)
#endif

#ifndef RTCPAL_BYTESWAP_UINT16
    static
    uint16_t
    FORCEINLINE
    rtcpal_byteswap_uint16(uint16_t x)
    {
        return (x << 8) | (x >> 8);
    }
    #define RTCPAL_BYTESWAP_UINT16(x)   rtcpal_byteswap_uint16(x)
#endif

#ifndef RTCPAL_BYTESWAP_UINT32
    static
    uint32_t
    FORCEINLINE
    rtcpal_byteswap_uint32(uint32_t x)
    {
        return ((x << 24) | ((x & 0xff00) << 8) |
            ((x & 0xff0000) >> 8) | (x >> 24));
    }

    #define RTCPAL_BYTESWAP_UINT32(x)   rtcpal_byteswap_uint32(x)
#endif

#ifndef RTCPAL_BYTESWAP_UINT64
    static
    uint64_t
    FORCEINLINE
    rtcpal_byteswap_uint64(uint64_t x)
    {
        return (((uint64_t) RTCPAL_BYTESWAP_UINT32((uint32_t) x)) << 32) |
            RTCPAL_BYTESWAP_UINT32((uint32_t) (x >> 32));
    }

    #define RTCPAL_BYTESWAP_UINT64(x)   rtcpal_byteswap_uint64(x)
#endif

#if defined(RTCPAL_LITTLE_ENDIAN)
    #define RTCPAL_MAKEFOURCC(c0,c1,c2,c3)  \
        (((uint32_t)c0) | (((uint32_t)c1) << 8) | \
            (((uint32_t)c2) << 16) | (((uint32_t)c3) << 24))

    #ifndef htonll
        #define htonll(x)           RTCPAL_BYTESWAP_UINT64(x)
    #endif
    #ifndef ntohll
        #define ntohll(x)           htonll(x)
    #endif

#elif defined(RTCPAL_BIG_ENDIAN)
    #define RTCPAL_MAKEFOURCC(c0,c1,c2,c3)  \
        (((uint32_t)c3) | (((uint32_t)c2) << 8) | \
            (((uint32_t)c1) << 16) | (((uint32_t)c0) << 24))

    #ifndef htonll
        #define htonll(x)       (x)
    #endif
    #ifndef ntohll
        #define ntohll(x)       (x)
    #endif
#else
    #error "Unknown endian".
#endif

#ifndef _MSC_VER
    // The following two RtcPalXXX depends on definitions in this header
    // file (rtcpal_base.h). We can't define them in unix/winapi/winbase.h,
    // which is included before rtcpal_base.h

    // Can't use the following macro due to atlbase.h 
    // #define WaitForSingleObject(h,t) RtcPalWaitForSingleObject(h,t)
    //
    // Use inline function instead
    static
    uint32_t
    FORCEINLINE
    RTCAPI
    RtcPalWin32WaitForSingleObject(
        __in HANDLE hHandle,
        __in uint32_t dwMilliseconds
        )
    {
        return RtcPalWaitForSingleObject(hHandle,dwMilliseconds);
    }
    #undef WaitForSingleObject
    #define WaitForSingleObject RtcPalWin32WaitForSingleObject

    __out_opt
    HANDLE
    RTCAPI
    RtcPalWin32CreateThread(
        __in_opt  LPSECURITY_ATTRIBUTES lpThreadAttributes,
        __in      SIZE_T dwStackSize,
        __in      LPTHREAD_START_ROUTINE lpStartAddress,
        __in_opt  LPVOID lpParameter,
        __in      DWORD dwCreationFlags,
        __out_opt LPDWORD lpThreadId
        );

	void
	RTCAPI
	RtcPalExitThread(
	    __in uint32_t ExitCode
	    );
	
    #undef CreateThread
    #define CreateThread RtcPalWin32CreateThread
#endif

#ifdef __cplusplus
};
#endif

#if defined(__cplusplus) && defined(__APPLE__)
//Helper overloaded functions to handle strict type conversion check of certain compilers (such as clang)
inline
RTCPAL_BOOL
RTCAPI
RtcPalSetEvent(
               RtcPalWaitableHandle event
               )
{
    return RtcPalSetEvent((RtcPalEventHandle)event);
}

inline
RTCPAL_BOOL
RTCAPI
RtcPalResetEvent(
                 RtcPalWaitableHandle event
                 )
{
    return RtcPalResetEvent((RtcPalEventHandle)event);
}

inline
RTCPAL_BOOL
RTCAPI
RtcPalReleaseMutex(
                   __in HANDLE hMutex
                   )
{
    return RtcPalReleaseMutex((RtcPalMutexHandle)hMutex);
}

inline
int
RTCAPI
RtcPalGetThreadPriority(
                        __in HANDLE Thread
                        )
{
    return RtcPalGetThreadPriority((RtcPalThreadHandle)Thread);
}

inline
RTCPAL_BOOL
RTCAPI
RtcPalSetThreadPriority(
                        __in HANDLE Thread,
                        __in int nPriority
                        )
{
    return RtcPalSetThreadPriority((RtcPalThreadHandle)Thread, nPriority);
}

inline
RTCPAL_BOOL
RTCAPI
RtcPalReleaseSemaphore(
                       __in HANDLE hSemaphore,
                       __in int32_t ReleaseCount,
                       __out_opt LPLONG lpPreviousCount
                       )
{
    return RtcPalReleaseSemaphore((RtcPalSemaphoreHandle)hSemaphore, ReleaseCount, lpPreviousCount);
}
#endif

#include <rtcpal_linkedlist.h>

#define FILE_ATTRIBUTE_DIRECTORY                0x00000010


typedef enum _GET_FILEEX_INFO_LEVELS {
  GetFileExInfoStandard
} GET_FILEEX_INFO_LEVELS;

typedef struct _WIN32_FILE_ATTRIBUTE_DATA {
    DWORD      dwFileAttributes;
    FILETIME   ftCreationTime;
    FILETIME   ftLastAccessTime;
    FILETIME   ftLastWriteTime;
    DWORD      nFileSizeHigh;
    DWORD      nFileSizeLow;
} WIN32_FILE_ATTRIBUTE_DATA, *LPWIN32_FILE_ATTRIBUTE_DATA;


/*++
Function:
  FILEGetLastErrorFromErrno

Convert errno into the appropriate win32 error and return it.
--*/
DWORD
FILEGetLastErrorFromErrno();

void
FILEGetProperNotFoundError(
    LPSTR lpPath,
    LPDWORD lpErrorCode
    );

/*++
Function:
  FileDosToUnixPathA

Abstract:
  Change a DOS path to a Unix path. Replace '\' by '/'.

Parameter:
  IN/OUT lpPath: path to be modified
--*/
void 
FILEDosToUnixPathA(LPSTR lpPath);

/*++
Function:
  FileDosToUnixPathW

Abstract:
  Change a DOS path to a Unix path. Replace '\' by '/'.

Parameter:
  IN/OUT lpPath: path to be modified
  --*/
void
FILEDosToUnixPathW(LPWSTR lpPath);

#ifdef UNICODE
#define FILEDosToUnixPath FILEDosToUnixPathW
#else
#define FILEDosToUnixPath FILEDosToUnixPathA
#endif // UNICODE


DWORD
RTCAPI
GetFileAttributesA(
    __in LPCSTR lpFileName);

DWORD
RTCAPI
GetFileAttributesW(
    __in LPCWSTR lpFileName);

#ifdef UNICODE
#define GetFileAttributes GetFileAttributesW
#else
#define GetFileAttributes GetFileAttributesA
#endif // UNICODE

BOOL
RTCAPI
GetFileAttributesExW(
    __in LPCWSTR lpFileName,
    __in GET_FILEEX_INFO_LEVELS fInfoLevelId,
    __out_opt LPVOID lpFileInformation);

#ifdef UNICODE
#define GetFileAttributesEx GetFileAttributesExW
#endif // UNICODE

LONG
RTCAPI
CompareFileTime(
    __in CONST FILETIME *lpFileTime1,
    __in CONST FILETIME *lpFileTime2);


BOOL
RTCAPI
GetFileTime(
        __in HANDLE hFile,
        __out_opt LPFILETIME lpCreationTime,
        __out_opt LPFILETIME lpLastAccessTime,
        __out_opt LPFILETIME lpLastWriteTime);

BOOL
SetFileTime(
    HANDLE hFile,
    const FILETIME * lpCreationTime,
    const FILETIME * lpLastAccessTime,
    const FILETIME * lpLastWriteTime);


#endif // __RTCPAL_BASE_H__
