#ifndef __RTCPAL_COM_LITE_H__
#define __RTCPAL_COM_LITE_H__

/*++

Copyright (C) Microsoft Corporation. All rights reserved.

Module Name:

rtcpal_com_lite.h

Abstract:

Defines:
    - a set of concise macros that implement IUnknown
    - a generic intrusive pointer that works well with COM-like interfaces

--*/

template <class T, T defVal = T()>
struct AutoInteger
{
    AutoInteger() : value(defVal) { }
    T value;
};

#define COMLITE_IMPLEMENT_REFCOUNTING() \
    private: \
        AutoInteger<volatile LONG, 1> m_refCount; \
    public: \
        virtual ULONG STDMETHODCALLTYPE AddRef() \
        { \
            return InterlockedIncrement(&m_refCount.value); \
        } \
        virtual ULONG STDMETHODCALLTYPE Release() \
        { \
            long refCount = InterlockedDecrement(&m_refCount.value); \
            if (refCount == 0) \
                delete this; \
            return refCount; \
        }

#define COMLITE_BEGIN_QI_MAP() \
    public: \
        HRESULT STDMETHODCALLTYPE QueryInterface(REFIID riid, void **ppvObject) \
        { \
            if (ppvObject == 0) \
                return E_POINTER;

#define COMLITE_QI_MAP_ENTRY_EX(ifaceName, ifaceProxy, ifaceID) \
            else if (riid == ifaceID) \
            { \
                *ppvObject = static_cast<ifaceName*>(static_cast<ifaceProxy*>(this)); \
                AddRef(); \
                return S_OK; \
            }

#if _MSC_VER
    #define COMLITE_IDOF(ifaceName) __uuidof(ifaceName)
#else
    #define COMLITE_IDOF(ifaceName) IID_##ifaceName
#endif

#define COMLITE_QI_MAP_ENTRY(ifaceName) \
    COMLITE_QI_MAP_ENTRY_EX(ifaceName, ifaceName, COMLITE_IDOF(ifaceName))

#define COMLITE_QI_MAP_ENTRY_IUNKNOWN(ifaceName) \
    COMLITE_QI_MAP_ENTRY_EX(IUnknown, ifaceName, COMLITE_IDOF(IUnknown)) \
    COMLITE_QI_MAP_ENTRY(ifaceName)

#define COMLITE_END_QI_MAP() \
            *ppvObject = 0; \
            return E_NOINTERFACE; \
        }

#define COMLITE_BEGIN_COCLASS_MAP() \
    COMLITE_IMPLEMENT_REFCOUNTING() \
    COMLITE_BEGIN_QI_MAP()

#define COMLITE_END_COCLASS_MAP() \
    COMLITE_END_QI_MAP()

struct ComLiteIntrusiveDefinition
{
    template <class T> static void AddRef(T& obj) { obj.AddRef(); }
    template <class T> static void Release(T& obj) { obj.Release(); }
};

struct ExistingObjectTag { };

template <class T, class IntrusiveDefinitionT = ComLiteIntrusiveDefinition >
class IntrusivePtr
{
public:
    IntrusivePtr() : m_p() { }
    explicit IntrusivePtr(T* p) : m_p() { Attach(p); }
    IntrusivePtr(T* p, ExistingObjectTag) : m_p() { Reset(p); }
    IntrusivePtr(IntrusivePtr<T, IntrusiveDefinitionT>&& p) : m_p(p.Detach()) { }
    IntrusivePtr(const IntrusivePtr<T, IntrusiveDefinitionT>& p) : m_p() { Reset(p.Get()); }
    template <class OtherT> explicit IntrusivePtr(OtherT* p) : m_p() { Attach(p); }
    template <class OtherT> IntrusivePtr(OtherT* p, ExistingObjectTag) : m_p() { Reset(p); }
    template <class OtherT> IntrusivePtr(IntrusivePtr<OtherT, IntrusiveDefinitionT>&& p) : m_p(p.Detach()) { }
    template <class OtherT> IntrusivePtr(const IntrusivePtr<OtherT, IntrusiveDefinitionT>& p) : m_p() { Reset(p.Get()); }
    ~IntrusivePtr() { Reset(); }

    IntrusivePtr<T, IntrusiveDefinitionT>& operator=(const IntrusivePtr<T, IntrusiveDefinitionT>& p)
    {
        if (m_p != p.m_p)
            Reset(p.Get());
        return *this;
    }
    IntrusivePtr<T, IntrusiveDefinitionT>& operator=(IntrusivePtr<T, IntrusiveDefinitionT>&& p)
    {
        if (m_p != p.m_p)
            Attach(p.Detach());
        return *this;
    }
    template <class OtherT> IntrusivePtr<T, IntrusiveDefinitionT>& operator=(const IntrusivePtr<OtherT, IntrusiveDefinitionT>& p)
    {
        if (m_p != p.Get())
            Reset(p.Get());
        return *this;
    }
    template <class OtherT> IntrusivePtr<T, IntrusiveDefinitionT>& operator=(IntrusivePtr<OtherT, IntrusiveDefinitionT>&& p)
    {
        if (m_p != p.Get())
            Attach(p.Detach());
        return *this;
    }

    template <class OtherT> bool operator==(const IntrusivePtr<OtherT, IntrusiveDefinitionT>& p) { return m_p == p.Get(); }
    template <class OtherT> bool operator!=(const IntrusivePtr<OtherT, IntrusiveDefinitionT>& p) { return !operator==(p); }
    template <class OtherT> bool operator<(const IntrusivePtr<OtherT, IntrusiveDefinitionT>& p) { return m_p < p.Get(); }
    template <class OtherT> bool operator<=(const IntrusivePtr<OtherT, IntrusiveDefinitionT>& p) { return operator<(p) || operator==(p); }
    template <class OtherT> bool operator>(const IntrusivePtr<OtherT, IntrusiveDefinitionT>& p) { return !operator<=(p); }
    template <class OtherT> bool operator>=(const IntrusivePtr<OtherT, IntrusiveDefinitionT>& p) { return !operator<(p); }

    T& operator*() const { return *m_p; }
    T* operator->() const { return m_p; }
    bool operator!() const { return !m_p; }
    operator bool() const { return !!m_p; }

    template <class OtherT> void Attach(OtherT* p)
    {
        if (m_p)
            IntrusiveDefinitionT::Release(*m_p);
        m_p = p;
    }
    template <class OtherT> void Reset(OtherT* p)
    {
        if (p == m_p)
            return;
        if (p)
            IntrusiveDefinitionT::AddRef(*p);
        Attach(p);
    }
    void Reset() { Attach<T>((T*)0); }
    T* Detach()
    {
        T* pTmp = m_p;
        m_p = (T*)0;
        return pTmp;
    }
    T** ResetAndGetAddressOf() { Reset(); return &m_p; }
    T* Get() const { return m_p; }

private:
    T* m_p;
};

#endif // __RTCPAL_COM_LITE_H__
