/*++

Module Name:

    rtcpal_dataconv.h

Abstract:

    Implement routines for converting data types. To save CPU cycles, media
    stack will dump data in native format on the runtime platform.
    This paradigm differs with the normal one usually used in networking
    protocol, in which the sender is required to convert data into platform
    neutral formats, for example, network order (big endian).

    It usage model is as the following:
    
    Media Stack Core:
        1) Call RtcPalGetDataConvSignature to obtain RTCPAL_DATACONV_SIGNATURE
        2) Persistent RTCPAL_DATACONV_SIGNATURE in platform neutral formats.
           It is up to media stack core to decide how it will store this struc
           in a platform neutral way.
            a) it may sprintf the number into a string and save the string in
               XML.
            b) or, it can htonl each fields in RTCPAL_DATACONV_SIGNATURE and
               then store the whole new RTCPAL_DATACONV_SIGNATURE as a binary
               BLOB.
        3) Media stack core then can dump its other data in native formats

    A consumer will be written to consume/parse the data offline. The consumer
    usually will be a native WIN32 applications. Although it is possible,
    there is no need to port the consumer to other platforms. The consumer
    will typicall do the following

        1) Reconstruct the RTCPAL_DATACONV_SIGNATURE struc which media stack
           core save at runtime.

           The RTCPAL_DATACONV_SIGNATURE must be in the native format of
           the platform on which the consumer will be running.

        2) Use RtcPalConvertArchSignature to convert the blob into
           client machines' format
        3) Use RtcPalToXXXX APIs for converting saved data into
           client machines' formats.

Author:

Revision History:

--*/
#ifndef __RTCPAL_OFFLINE_DATA_CONVERSION_H__
#define __RTCPAL_OFFLINE_DATA_CONVERSION_H__

#ifdef __cplusplus
extern "C" {
#endif

#define RTCPAL_DATACONV_ENDIAN_LITTLE       1
#define RTCPAL_DATACONV_ENDIAN_BIG          2
#define RTCPAL_DATACONV_ENDIAN_PDP          3

#define RTCPAL_DATACONV_FP_IEEE754_1985     1

#include <pshpack4.h>
typedef struct RTCPAL_DATACONV_SIGNATURE {
    uint16_t ByteOrder;
    uint16_t FpFormat;
    uint16_t FloatSize;
    uint16_t DoubleSize;
} RTCPAL_DATACONV_SIGNATURE, * PRTCPAL_DATACONV_SIGNATURE;
#include <poppack.h>

void
RTCAPI
RtcPalGetDataConvSignature(
    __out_ecount(1) PRTCPAL_DATACONV_SIGNATURE pSignature
    );

#ifdef _WIN32_WINNT
// Media stack core is not supposed to call offline data conversion APIs.
// Make the offline data conversion APIs available only on Windows platform
// because trace file consumers are written only for Windows.
typedef struct RTCPAL_DATACONV_CONTEXT * RTCPAL_DATACONV_HANDLE;

RTCPAL_DATACONV_HANDLE
RTCAPI
RtcPalCreateDataConverter(
    __in_ecount(1) const struct RTCPAL_DATACONV_SIGNATURE * pSignature
    );

void
RTCAPI
RtcPalDestroyDataConverter(
    RTCPAL_DATACONV_HANDLE hConv
    );

// RtcPalToXXXX routines convert from the native format of the machines
// on which the data was taken into the format of the current host.
//
// All the APIs assume that the input pSignature have been validated
// by RtcPalValidateArchSignature. When the input pSignature is
// invalid, RtcPalToXXXX routines can return non-deterministics
// result.
uint16_t
RtcPalToHostUInt16(
    __in_ecount(1) const struct RTCPAL_DATACONV_CONTEXT * pSignature,
    uint16_t v
    );

int16_t
RtcPalToHostInt16(
    __in_ecount(1) const struct RTCPAL_DATACONV_CONTEXT * pSignature,
    int16_t v
    );

uint32_t
RtcPalToHostUInt32(
    __in_ecount(1) const struct RTCPAL_DATACONV_CONTEXT * pSignature,
    uint32_t v
    );

int32_t
RtcPalToHostInt32(
    __in_ecount(1) const struct RTCPAL_DATACONV_CONTEXT * pSignature,
    int32_t v
    );

uint64_t
RtcPalToHostUInt64(
    __in_ecount(1) const struct RTCPAL_DATACONV_CONTEXT * pSignature,
    uint64_t v
    );

int64_t
RtcPalToHostInt64(
    __in_ecount(1) const struct RTCPAL_DATACONV_CONTEXT * pSignature,
    int64_t v
    );

float
RtcPalToHostFloat(
    __in_ecount(1) const struct RTCPAL_DATACONV_CONTEXT * pSignature,
    __in uint8_t size,
    __in_bcount(size) const uint8_t * val
    );

double
RtcPalToHostDouble(
    __in_ecount(1) const struct RTCPAL_DATACONV_CONTEXT * pSignature,
    __in uint8_t size,
    __in_bcount(size) const uint8_t * val
    );
#endif // _WIN32_WINNT

#ifdef __cplusplus
};
#endif

#endif // __RTCPAL_OFFLINE_DATA_CONVERSION_H__
