#ifndef _DECODER_INTERFACE_H_
#define _DECODER_INTERFACE_H_

// Structures

typedef struct _RtcPalDec_InfoStruct_t {
    uint32_t NumberofConcealedMBs; // Number of concealed MBs
    uint32_t Qp; // Qp used
    uint32_t NumberofLostFrames; // number of lostFrames
    uint32_t uiLatency; // may be reported by HW decoders
    RTCPAL_BOOL bRendererLate; // reported by renderer
} RtcPalDec_InfoStruct_t;

typedef struct _RtcPalDecVideoEncodedUnit_t
{
    // Resolution
    uint16_t usFrameWidth;
    uint16_t usFrameHeight;

    // Frame data
    uint8_t* pbFrameData;
    uint32_t ulDataLength;

    union
    {
        struct
        {
            uint8_t  bEndOfAU;
        } H264;

        struct
        {
            // Frame header (optional, header can also be in the data)
            uint8_t  pbFrameHeader[65];
            uint32_t ulHeaderLength;

            // Codec flag (RTVideo only)
            uint32_t ulCodecFlag;
        } VC1;
    };
    // Time stamp
    uint64_t ullTimeStamp;
    RTCPAL_BOOL  bValidCropInfo;
    RtcPalVideoCropInfo_t CropInfo;
} RtcPalDecVideoEncodedUnit_t;

// Destruction Sequence
// In engine thread context,
//  1. MLD should set a flag to prevent decode worker thread from calling
//     DecodeData.
//  2. MLD calls NotifyDestroy from engine thread.
//  3. MLD queue a work item for invoking DestroyDecoder
//     this workitem must be queued to the same queue which MLD uses for
//     calling DecodeData
//
// In decoding worker thread context, MLD calls DestroyDecoder.
struct IRtcPalDecoderInterface
{
    // Function Name: DestroyDecoder
    // Function Description: This function is used to destroy the decoder instance
    // Arguments:
    // Return : HRESULT
    // MLDInterface: Called when calling MLD_Destroy
    //
    // When DestroyDecoder is called, MLD must ensure there are no other IRtcPalDecoderInterface
    // invokation.
    virtual HRESULT DestroyDecoder () = 0;

    // Function Name: DecodeData
    // Function Description: This function is used to pass the encoded bits to decoder
    // Arguments:
    // Inputs:
    //        EncodedUnit: encode unit containing encoded data and timestamp
    // Return : HRESULT
    // MLDInterface: Called from decoder thread that is created when calling MLDInstantiate
    //
    // This function can block the calling thread for indefinite time. In the
    // extreme case, it can block the calling thread forever, for example, when
    // the decoder pipeline stalls. NotifyDestroy is used for aborting the stuck
    // pipeline.

    virtual HRESULT DecodeData (
        __in const RtcPalDecVideoEncodedUnit_t* pstEncodedUnit
        ) = 0; 

    // Notify the decoder that it is being destroyed. Decoder is expected to
    // abort the current DecodeData.
    virtual HRESULT NotifyDestroy() = 0;

    // Function Name: GetMetrics
    // Function Description: This function is used to get the required information or metrics from decoder
    // Arguments:
    // Outputs:
    //        pDecInfo: pointer to decoder info structure
    // Return : HRESULT
    // MLDInterface: Called from decoder thread that is created when calling MLDInstantiate
    virtual HRESULT GetMetrics (
    __out   RtcPalDec_InfoStruct_t    *pDecInfo
    ) = 0;


    // Function Name: SetDecoderOptions
    // Function Description: This function is used to set the required options on decoder
    // Arguments:
    // Outputs:
    //        decOptions: pointer to string of key value pairs like "width 640 height 480"
    // Return : HRESULT
    // MLDInterface: Called from decoder thread that is created when calling MLDInstantiate
    virtual HRESULT SetDecoderOptions(
    __in const char* decOptions
    ) = 0;
};

struct IRtcPalDecoderCallback
{
    virtual void Invoke(IRtcPalDecoderInterface * pSender, RtcPalVideoEvent_e event, void *pData) = 0;
};

#endif
