/*++

Abstract:
    Interface definitionas for the public interface into RTCPAL filesource implementations
    This defines the interface for interactions between MediaStack and RtcPalFilesource

--*/

#ifndef __rtcpal_filesource__
#define __rtcpal_filesource__

// at this layer, all interfaces are C-styled
#ifdef __cplusplus
extern "C"
{
#endif

///////////////////////////////////////////////////////////////////////
// Sturctures and enums to facilitate platform independent calls to 
// do status updates, report capability information, and specify
// media types to be used

typedef enum _RtcPalOfflineSourceReader_STATUS 
{
  RtcPalOfflineSourceReader_ERROR                         = 0,
  RtcPalOfflineSourceReader_OPENED                        = 1,
  RtcPalOfflineSourceReader_EOF                           = 2,
  RtcPalOfflineSourceReader_END_OF_FILE                   = 2,
  RtcPalOfflineSourceReader_MISSING_CODEC                 = 3,
  RtcPalOfflineSourceReader_STARTED                       = 4,
  RtcPalOfflineSourceReader_STOPPED                       = 5,
  RtcPalOfflineSourceReader_CLOSED                        = 6,
} RtcPalOfflineSourceReader_STATUS;

typedef enum _RtcPalOfflineSourceReader_TYPE 
{
    RtcPalOfflineSourceReader_AUDIO = 0,
    RtcPalOfflineSourceReader_VIDEO = 1,
} RtcPalOfflineSourceReader_TYPE;

typedef struct _RtcPalOfflineSourceReaderCapability
{
    ULONG  SamplingRate;     // Expressed in Hz
    USHORT BitsPerSample;
    USHORT FrameSize;        // Expressed in bytes
    USHORT FrameDuration;    // Expressed in ms
    USHORT NumberOfChannels; // 1 = Mono, 2 = Stereo
} RtcPalOfflineSourceReaderCapability;


// callback structure using the following two methods to facilitate callbacks between the source and reader
// pvContext is the context where the function calls should be made to
// pfnOnReadSampleCallback handles sample callbacks
// pfnOnStatusCallback handles all status updates as defined above in RtcPalOfflineSourceReader_STATUS

// ulSampleTime is in 100ns units
typedef int32_t (WINAPI *PRVDONREADSAMPLECALLBACK)(ULONGLONG ulSampleTime, DWORD dwBufferLength, BYTE * pBuffer, void* pvContext);
typedef int32_t (WINAPI *PRVDONSTATUSCALLBACK)(RtcPalOfflineSourceReader_STATUS Status, void* pvContext);

typedef struct _RtcPalOfflineSourceReaderCallback_t
{
    VOID                        * pvContext;
    PRVDONREADSAMPLECALLBACK      pfnOnReadSampleCallback;
    PRVDONSTATUSCALLBACK          pfnOnStatusCallback;
} RtcPalOfflineSourceReaderCallback_t;



/*
    Will open a SourceReader on the given filepath and mediatype and depending on whether a callback is specified will run Synchronously/Asynchronously

    pvFileSourceReader: handle to SourceReader, must not be NULL
    wszFilePath: path to media file to be opened
    MediaType: specify audio or video, currently only support audio
    pCap: pointer to the outside capability requirements, will use this info to set up output type of samples
    pCBInfo: if NULL the reader will open in Synchronous mode, otherwise it will use the RtcPalOfflineSourceReaderCallback_t
             to set up callbacks for Asynchronous operation
    pContext: context from which this call is being made
*/
RTC_CLASS_LINKAGE
HRESULT
RtcPalOfflineSourceReaderOpen(
    __in            const WCHAR *wszFilePath,
    __in            RtcPalOfflineSourceReader_TYPE MediaType,
    __in            RtcPalOfflineSourceReaderCapability * pCap,
    __in            RtcPalOfflineSourceReaderCallback_t * pCBInfo,
    __in            void * pContext,
    __out           void ** pvFileSourceReader
        );

/*
    Closes the reader specified by pvFileSourceReader, and clears up any memory or supporting API's needed by the reader
    pvFileSourceReader: must not be NULL
*/
RTC_CLASS_LINKAGE
HRESULT
RtcPalOfflineSourceReaderClose(
    __in            VOID * pvFileSourceReader
    );

/*
    Returns the duration in ms of the file

    pvFileSourceReader: must not be NULL
    pulDuration: address where duration will be placed, in ms
*/
RTC_CLASS_LINKAGE
HRESULT
RtcPalOfflineSourceReaderGetDuration(
    __in            VOID * pvFileSourceReader,    
    __out           ULONG *pulDuration
    );

/*
   Performs a read operation, behavior depends on whether the reader was opened in Synchronous/Asynchronous mode

   Synchronous:

   pvFileSourceReader: must not be NULL
   ulReadTime: must be NULL
   ppBuffer: will use this double pointer to return the sample
   * dwLength: returns the length of the pointer at the address pointed

   After you have copied the data a corresponding call to RtcPalOfflineSourceReaderReadRelease with the same double
   pointer returned by RtcPalOfflineSourceReaderRead must be made to free the memory before the next call to RtcPalOfflineSourceReaderRead

   Asynchronous:

   pvFileSourceReader: must not be NULL
   ulReadTime: time at which the caller is wanting to read data from, in ms units
   ppBuffer: must be NULL
   * dwLength: must be NULL

   If ulReadTime is less than what has already been given by the callbacks then read does nothing. However if it is beyond
   what has been thrown up by callbacks then a new read from below is initiated and a callback is sent up.
   This behavior is such to be adaptable to the different sample sizes returned from files
*/
RTC_CLASS_LINKAGE
HRESULT
RtcPalOfflineSourceReaderRead(
    __in            VOID * pvFileSourceReader,
    __in            ULONG ulReadTime,
    __inout         BYTE ** ppBuffer, 
    __inout         DWORD * dwLength
    );

/*
    Synchronous Mode:
    After a call to RtcPalOfflineSourceReaderRead, the same double pointer must be returned through ppBuffer to be released by this method.
    This facilitates local memory allocation and deallocation conventions

    pvFileSourceReader: must not be NULL
    ppBuffer: must not be NULL, and *ppBuffer must not be NULL
*/
RTC_CLASS_LINKAGE
HRESULT
RtcPalOfflineSourceReaderReadRelease(
    __in            VOID * pvFileSourceReader,
    __inout         BYTE * ppBuffer
    );

/*
    Sets the reader position as close as possible to the value of ulNewTime, precision can vary based on implementation

    pvFileSourceReader: must not be NULL
    ulNewTime: position to set the reader to in ms
    pvContext: not used at this time
*/
RTC_CLASS_LINKAGE
HRESULT
RtcPalOfflineSourceReaderSetPosition(
    __in            VOID * pvFileSourceReader,
    __in            ULONG ulNewTime, 
    __in            ULONG_PTR pvContext
    );

/*
    Returns the current output capability of the reader

    pvFileSourceReader: must not be NULL
    pCap: pointer to allocated RtcPalOfflineSourceReaderCapability structure to populate
*/
RTC_CLASS_LINKAGE
HRESULT
RtcPalOfflineSourceReaderGetCapability(
    __in            VOID * pvFileSourceReader,
    __out           RtcPalOfflineSourceReaderCapability * pCap
    );

// at this layer, all interfaces are C-styled
#ifdef __cplusplus
}
#endif

#endif  /*  __rtcpal_filesource__ */
