#ifndef __RTCPAL_LOGGING_H__
#define __RTCPAL_LOGGING_H__

// RTC PAL asynchronous logging functions
typedef void * HRTCLOG;

#define RTCPAL_LOG_MAX_PRINT_SIZE       4096  // In number of characters
#define RTCPAL_LOG_FILE_SIZE_LIMIT      INT_MAX // 2 GB (2^31 - 1)
#define RTCPAL_LOG_DEFAULT_BUFFER_SIZE  64*1024 // 64k
#define RTCPAL_LOG_MAX_WRITE_SESSIONS   64

#if defined(WINPHONE)   // WinPhone has very strict memory useage limit
    #define RTCPAL_LOG_DEFAULT_WRITE_SESSIONS   2
#elif defined(_UNIX_IPHONE) || defined(_UNIX_IPAD) || defined(ANDROID) || defined(__linux)
    #define RTCPAL_LOG_DEFAULT_WRITE_SESSIONS   8
#else
    #define RTCPAL_LOG_DEFAULT_WRITE_SESSIONS   16
#endif

#ifdef __cplusplus
extern "C" {
#endif


/* ------------------------------------------------------------------------
RtcPalLogCreate - Create log write object and allocate resources
    phRtcLog - pointer to the log handle
    ptzFileName     - Log file name
    uLogSessions    - Maximum concurrnt log sessions. Each session has a 
        buffer with user specified buffer length. For each write request,
        data is saved into the buffer first. After the buffer is full, the
        buffer will be submitted to OS asyncrhonously. And the next available
        session will be continue used to buffer the data.

        If number of log session is 1, we are in a special mode. In this mode
        data will not be submitted to OS untill RtcPalLogFlush is called. The
        flag bSessionCircOverwrite specifies the buffer behavior after buffer
        if full. 
    uSessionBufSize - Buffer size for each log session.
    bSessionCircOverwrite - Only valid when number of session is 1. If it TRUE,
        after the buffer is full, new data will overwrite the earliest data in
        the buffer in a circular manner. If it is FALSE, all new data will be
        dropped after the buffer is full.
-------------------------------------------------------------------------- */
HRESULT 
RTCAPI
RtcPalLogCreate(
    HRTCLOG * phRtcLog,             // Log handle
    const char * pszFileName,       // File name
    uint32_t  uLogSessions,         // Maximum concurrent write sessions
    uint32_t  uSessionBufSize,      // Minimum async write block
    int       bSessionCircOverwrite // Overwrite the circular buffer for single session scenario?
    );


/* ------------------------------------------------------------------------
RtcPalLogClose - Close log write handle and release all resources. 

Remarks: If there is incomplete write sessions, RtcPalLogClose will wait
  until all log sessions are completeed.
-------------------------------------------------------------------------- */
void
RTCAPI
RtcPalLogClose(
    HRTCLOG   hRtcLog              // Log handle
    );


/* ------------------------------------------------------------------------
RtcPalLogSeek - move the file write pointer to a specific position
    iOffset - Number of bytes from origin
    iOrigin - Initial position, which can be 
              SEEK_CUR - current write position
              SEEK_SET - Beginning of file

  Maximum seeking offset is 2^15-1. (2GB)
-------------------------------------------------------------------------- */
HRESULT
RTCAPI
RtcPalLogSeek(
    HRTCLOG   hRtcLog,              // Log handle
    int32_t   iOffset,
    int32_t   iOrigin
    );


/* ------------------------------------------------------------------------
RtcPalLogWrite - Binary log writer.
-------------------------------------------------------------------------- */
HRESULT
RTCAPI
RtcPalLogWrite(
    HRTCLOG   hRtcLog,              // Log handle
    __in_bcount(uNumBytes) const void * pbData,
    uint32_t  uNumBytes
    );


/* ------------------------------------------------------------------------
RtcPalLogPrint - Text log writer.

Remarks: The max string print size is 4096 charactor
-------------------------------------------------------------------------- */
HRESULT
RTCAPI
RtcPalLogPrint(
    HRTCLOG   hRtcLog,              // Log handle
    const char * pszString,
    ...
    );

/* ------------------------------------------------------------------------
RtcPalLogPrint - Text log writer sing a pointer to a list of arguments

Remarks: The max string print size is 4096 charactor
-------------------------------------------------------------------------- */
HRESULT
RTCAPI
RtcPalLogVPrint(
    HRTCLOG   hRtcLog,              // Log handle
    const char * pszString,
    va_list va_args
    );

/* ------------------------------------------------------------------------
RtcPalLogFlush - Flush all data in the buffer. 

Remarks: The function will wait until all write sessions complete. 

Calling RtcPalLogFlush after each write call will effectively turn file 
output to synchronous
-------------------------------------------------------------------------- */
HRESULT
RTCAPI
RtcPalLogFlush(
    HRTCLOG   hRtcLog              // Log handle
    );

/* ------------------------------------------------------------------------
RtcPalSetLogPath - Set the WPP log path 

Remarks: This function MUST be called before WPP trace is started.
pPathName: WCHAR pointer to a NULL ended WCHAR string.
-------------------------------------------------------------------------- */
HRESULT
RTCAPI
RtcPalSetLogPath(
    __in LPCWSTR pPathName
    );

#ifdef __cplusplus
}
#endif

#endif //__RTCPAL_LOGGING_H__

