/*++

Copyright (c)  2000-2001 Microsoft Corporation

Module Name:

    rtcpal_reference.h

Abstract:

    This module contains funtions that allow keeping references across
    Initialize, UnInitialize, Reference and Dereference operations on an
    object.  They are designed to not require any locks to be associated
    with the object and also permit asynchronous completion of the
    UnInitialize operation. On LLP64 system like MSC, the maximum allowed
    references is 2^30. On non-LLP64 system (GNUC for example), the maximum
    allowed reference is 2^62 on 64-bit platforms and 2^30 on 32-bit platforms.

Interfaces:
    RtcPalRoInitialize
        Initialize the reference object

    RtcPalRoReference and RtcPalRoReferenceEx
        These two APIs return TRUE if references can be honored. Otherwise,
        they return FALSE.
        Use these APIs in the code path for queuing new requests to synchronize
        with close/shutdown code path.

    RtcPalRoReferenceValidObject
        This works similarly to IUnknown::AddRef. It always add the references.
        Use this API for situations not mentioned in RtcPalRoReferenceEx above.

    RtcPalRoDereference and RtcPalRoDereferenceEx
        These two APIs works similary to IUnknown::Release

Author:

    Mohit Talwar (mohitt) 31-Oct-2000

    Jiandong Ruan (jruan) 24-May-2012
        Lift this from TCPIP stack and make the implementation platform neutral

Environment:

    Both user and kernel mode.

--*/

#ifndef __RTCPAL_REFERENCE_OBJECT_H__
#define __RTCPAL_REFERENCE_OBJECT_H__

#ifdef __cplusplus
extern "C" {
#endif

//
// Deleting = TRUE, ReferenceCount = 0;
//
#define RTCPAL_RO_VALUE_UNINITIALIZED 1

//
// Deleting = FALSE, ReferenceCount = 1;
//
#define RTCPAL_RO_VALUE_INITIALIZED 2

// Bit order is platform/compiler dependent.
//  typedef struct RTCPAL_REFERENCE_OBJECT {
//      uint32_t Deleting: 1;           // Least Significant bit
//      uint32_t ReferenceCount: 30;
//      uint32_t Overflow: 1;           // Most Significant bit
//  } RTCPAL_REFERENCE_OBJECT;
typedef struct {
    volatile LONG Value;
} RTCPAL_REFERENCE_OBJECT, * PRTCPAL_REFERENCE_OBJECT;

// 0x01u is 'int' (4-bytes). 0x01ul is 'long' (8-bytes)
#define RTCPAL_RO_MASK_DELETING     0x01ul
#define RTCPAL_RO_MASK_OVERFLOW     ((LONG)(0x01ul << (sizeof(LONG) * 8 - 1)))

RTCPAL_C_ASSERT(RTCPAL_RO_MASK_OVERFLOW < 0);
RTCPAL_C_ASSERT((LONG)((ULONG)RTCPAL_RO_MASK_OVERFLOW - 1) > 0);

static
VOID
RTCPAL_FORCEINLINE
RtcPalRoInitialize(
    _Out_ PRTCPAL_REFERENCE_OBJECT ReferenceObject
    )
/*++

Routine Description:

    This function is called to initialize a RTCPAL_REFERENCE_OBJECT.

Arguments:

    ReferenceObject - Returns an initialized reference object (stored at
        the caller supplied address).

Return Value:

    None.

--*/
{
    //
    // ReferenceObject->Deleting = FALSE;
    // ReferenceObject->ReferenceCount = 1;
    //
    ReferenceObject->Value = RTCPAL_RO_VALUE_INITIALIZED;
}


static
VOID
RTCPAL_FORCEINLINE
RtcPalRoInitializeAsInvalid(
    _Out_ PRTCPAL_REFERENCE_OBJECT ReferenceObject
    )
/*++

Routine Description:

    This function is called to initialize a RTCPAL_REFERENCE_OBJECT to an invalid
    state.  Subsequent calls to RoReference will always return FALSE until
    RoInitialize is called.

Arguments:

    ReferenceObject - Returns an initialized reference object (stored at
        the caller supplied address).

Return Value:

    None.

--*/
{
    //
    // ReferenceObject->Deleting = TRUE;
    // ReferenceObject->ReferenceCount = 0;
    //
    ReferenceObject->Value = RTCPAL_RO_VALUE_UNINITIALIZED;
}


static
RTCPAL_BOOL
RTCPAL_FORCEINLINE
RtcPalRoIsDeleting(
    _In_ const RTCPAL_REFERENCE_OBJECT * ReferenceObject
    )
/*++

Routine Description:

    This function is called to detect if a RTCPAL_REFERENCE_OBJECT is deleted.

Arguments:

    ReferenceObject - Supplies a reference object.

Return Value:

    TRUE if the reference object was deleted, FALSE if not.

--*/
{
    return (ReferenceObject->Value & RTCPAL_RO_MASK_DELETING)? TRUE: FALSE;
}


static
RTCPAL_BOOL
RTCPAL_FORCEINLINE
RtcPalRoIsDeleted(
    _In_ const RTCPAL_REFERENCE_OBJECT * ReferenceObject
    )
/*++

Routine Description:

    This function is called to detect if a RTCPAL_REFERENCE_OBJECT is deleted.

Arguments:

    ReferenceObject - Supplies a reference object.

Return Value:

    TRUE if the reference object has been deleted, FALSE if not.

--*/
{
    return (RTCPAL_RO_VALUE_UNINITIALIZED == ReferenceObject->Value)? TRUE: FALSE;
}

#define RtcPalRoIsUnInitialized RtcPalRoIsDeleted


static
RTCPAL_BOOL
RTCPAL_FORCEINLINE
RtcPalRoUnInitialize(
    _Inout_ PRTCPAL_REFERENCE_OBJECT ReferenceObject
    )
/*++

Routine Description:

    This function is called to start uninitializing a RTCPAL_REFERENCE_OBJECT.
    It might indicate completion asynchronously.

Arguments:

    ReferenceObject - Supplies the reference object to uninitialize.
        Returns its update.

Return Value:

    TRUE if uninitialization is complete.  FALSE if completion will be
    signalled asynchronously (from an RoDereference call).

--*/
{
    RTCPAL_REFERENCE_OBJECT New;

    //
    // ReferenceObject->Deleting = TRUE;    // Mark Deleting
    // ReferenceObject->ReferenceCount--;   // Decrement ReferenceCount
    // The former is achieved by adding 1 to the reference object value
    // and the latter by subtracting 2 from it.  We decrement by 1 to
    // achieve the combined effect!
    //
    New.Value = InterlockedDecrement(&ReferenceObject->Value);

    //
    // Deleting should never get marked more than once.  A successive
    // attempt to do so would flip this bit resulting in an assert.
    //
    RTCPAL_ASSERT(New.Value & RTCPAL_RO_MASK_DELETING);

    //
    // If ReferenceCount falls to 0 and Deleting has been set, then
    // indicate to the caller that the object can now be deleted.
    //
    if (New.Value == RTCPAL_RO_MASK_DELETING) {
        return TRUE;
    }
    return FALSE;
}

static
VOID
RTCPAL_FORCEINLINE
RtcPalRoReInitialize(
    _Inout_ PRTCPAL_REFERENCE_OBJECT ReferenceObject
    )
/*++

Routine Description:

    This function is called to reinitialize an uninitialized RTCPAL_REFERENCE_OBJECT.

Arguments:

    ReferenceObject - Supplies the reference object to reinitialize.

Return Value:

    None.

--*/
{
    RTCPAL_REFERENCE_OBJECT New;

    //
    // ReferenceObject->Deleting = FALSE;   // Unmark Deleting
    // ReferenceObject->ReferenceCount++;   // Increment ReferenceCount
    // The former is achieved by subtracting 1 from the reference object
    // value and the latter by adding 1 from it.  We increment by 1 to
    // achieve the combined effect!
    //
    New.Value = InterlockedIncrement(&ReferenceObject->Value);

    //
    // Deleting should have been marked.  An attempt to reinitialize an
    // initialized object would flip this bit resulting in an assert.
    //
    RTCPAL_ASSERT(0 == (New.Value & RTCPAL_RO_MASK_DELETING));
}


static
LONG
RTCPAL_FORCEINLINE
RtcPalRoReferenceValidObject(
    _Inout_ PRTCPAL_REFERENCE_OBJECT ReferenceObject,
    _In_ uint32_t Count
    )
/*++

Routine Description:

    This function is called to reference a RTCPAL_REFERENCE_OBJECT that is guaranteed
    to be valid, for example when the caller already holds a reference.
    This ensures that we can safely add more references.

    This works similarly to IUnknown::AddRef

Arguments:

    ReferenceObject - Supplies the reference object to reference.
        Returns its update.

    Count - Supplies the number of references to take.

Return Value:

    New reference count

--*/
{
    RTCPAL_REFERENCE_OBJECT Old;

    RTCPAL_ASSERT(Count < 0x40000000); // 2^30.

    //
    // ReferenceObject.ReferenceCount += Count;
    // This is achieved by adding 2*Count to the ReferenceObject value.
    //
    Old.Value =
        InterlockedExchangeAdd(&ReferenceObject->Value,  (2 * Count));

    //
    // InterlockedExchangeAdd returns the original value!
    // The reference object must not have been uninitialized.
    //
    RTCPAL_ASSERT(!RtcPalRoIsUnInitialized(&Old));

    //
    // Finally, this is the affect our operation had.
    // Do the number of outstanding references exceed our limit?
    // That would cause the most significant bit to get set!
    //
    Old.Value += (2 * Count);
    RTCPAL_ASSERT(0 == (Old.Value & RTCPAL_RO_MASK_OVERFLOW));

    return (Old.Value >> 1);
}


static
RTCPAL_BOOL
RTCPAL_FORCEINLINE
RtcPalRoReferenceEx(
    _Inout_ PRTCPAL_REFERENCE_OBJECT ReferenceObject,
    _In_ uint32_t Count
    )
/*++

Routine Description:

    This function is called to reference a RTCPAL_REFERENCE_OBJECT.

    This function behave slightly differently from IUnknown::AddRef.
    If the object is being deleted, reference will NOT be added.

Arguments:

    ReferenceObject - Supplies the reference object to reference.
        Returns its update.

    Count - Supplies the number of references to take.

Return Value:

    TRUE if the reference object was successfully referenced.  FALSE if
    the reference object is being uninitialized and cannot be referenced.

--*/
{
    RTCPAL_REFERENCE_OBJECT Snapshot, Old, New;

    RTCPAL_ASSERT(Count < 0x40000000); // 2^30.

    do {
        //
        // Take a snapshot of the current value.
        //
        Snapshot.Value = ReferenceObject->Value;

        //
        // Fail if Deleting.
        //
        if (Snapshot.Value & RTCPAL_RO_MASK_DELETING) {
            return FALSE;
        }

        //
        // New.Deleting = Snapshot.Deleting;
        // New.ReferenceCount = Snapshot.ReferenceCount + Count;
        // This is achieved by adding 2*Count to the snapshot value.
        //
        New.Value = Snapshot.Value + (2 * Count);

        //
        // Do the number of outstanding references exceed our limit?
        // That would cause the most significant bit to get set!
        //
        RTCPAL_ASSERT(0 == (New.Value & RTCPAL_RO_MASK_OVERFLOW));

        //
        // Update ReferenceCount, but only if the current value is the
        // same as that obtained in the snapshot.  Hence this operation
        // will fail if the reference object has been marked Deleting.
        //
        Old.Value =
            InterlockedCompareExchange(
                &ReferenceObject->Value, New.Value, Snapshot.Value);

        //
        // Repeat until the ReferenceCount is successfully updated or
        // the Deleting bit is observed to be set.
        //
    } while (Old.Value != Snapshot.Value);

    return TRUE;
}

#define RtcPalRoReference(ReferenceObject) RtcPalRoReferenceEx((ReferenceObject), 1)

static
LONG
RTCPAL_FORCEINLINE
RtcPalRoDereferenceEx(
    _Inout_ PRTCPAL_REFERENCE_OBJECT ReferenceObject,
    _In_ uint32_t Count
    )
/*++

Routine Description:

    This function is called to dereference a RTCPAL_REFERENCE_OBJECT.
    This works similarly to IUnknown::Release

Arguments:

    ReferenceObject - Supplies the reference object to dereference.
        Returns its update.

    Count - Supplies the number of references to remove.

Return Value:

    New reference count

--*/
{
    RTCPAL_REFERENCE_OBJECT New;

    RTCPAL_ASSERT(Count < 0x40000000);

    //
    // ReferenceObject->ReferenceCount -= Count;
    // This is achieved by subtracting (2 * Count).
    //
    New.Value =
        InterlockedExchangeAdd(
            &ReferenceObject->Value, -(2 * (int32_t) Count));

    //
    // InterlockedExchangeAdd returns the original value!
    //
    New.Value -= 2 * (int32_t) Count;

    //
    // Did we take away more references than we actually had?
    // That would have caused the most significant bit to get set!
    //
    RTCPAL_ASSERT(0 == (New.Value & RTCPAL_RO_MASK_OVERFLOW));

    //
    // Value should never fall to 0
    //
    RTCPAL_ASSERT(New.Value != 0);

    return (New.Value >> 1);
}

#define RtcPalRoDereference(ReferenceObject) RtcPalRoDereferenceEx((ReferenceObject), 1)

#ifdef __cplusplus
};
#endif

#endif // __RTCPAL_REFERENCE_OBJECT_H__

