/*++

Copyright (C) Microsoft Corporation. All rights reserved.

Module Name:

    rtcpal_systemmetrics.h

Abstract:

    A few API which collect system metrics/performance counter for
    troubleshooting A/V issues.

    It is upto each platform implementation to determine what
    metrics will be collected.

Author:

Revision History:

--*/

#ifndef __RTCPAL_SYSTEM_METRICS_H__
#define __RTCPAL_SYSTEM_METRICS_H__

#ifdef __cplusplus
extern "C" {
#endif

enum RtcPalMetricType {
    RTCPAL_METRIC_TYPE_FLOAT,
    RTCPAL_METRIC_TYPE_UINT32,
    RTCPAL_METRIC_TYPE_UINT64,
    RTCPAL_METRIC_TYPE_STRING
};

struct RtcPalSystemMetricsCollector;

struct RtcPalSystemMetricDescription {
    const CHAR * MetricCategory;
    const CHAR * MetricName;

    // Metric data type
    enum RtcPalMetricType MetricType;

    // The offset of metric value in RtcPalSystemMetrics structure.
    uint32_t MetricOffset;
};

struct RtcPalSystemMetrics;

typedef void (RTCAPI *PRTCPAL_SYSTEM_METRICS_CALLBACK)(
    void * Context,
    const struct RtcPalSystemMetrics *
    );

#define RTCPAL_SYSTEM_METRICS_MAXIMUM_INTERVAL      60000
#define RTCPAL_SYSTEM_METRICS_MINIMUM_INTERVAL      1000

enum
{
    RtcPalSystemMetricsFlag_EnableWiFiStats = (1<<0)
};

// Init collecting system metrics.
// PAL will call pCallback every IntervalInMs ms after it is started
//
// Return
//      NULL upon failure. Use GetLastError() to retrieve the error code.
struct RtcPalSystemMetricsCollector *
RtcPalInitCollectSystemMetrics(
    __in uint32_t IntervalInMs,
    __in PRTCPAL_SYSTEM_METRICS_CALLBACK pCallback,
    __in PVOID pvContext,
    __in uint32_t flags
#ifdef __cplusplus
    = 0 // the flags default to zero (no extra stats included)
#endif
    );

// Uninit collecting system metrics.
// This API will block until metric collector has been stopped.
//
// NEVER call this API from PRTCPAL_SYSTEM_METRICS_CALLBACK callback routine.
// Doing so will cause deadlock.
void
RtcPalUninitCollectSystemMetrics(
    __in struct RtcPalSystemMetricsCollector * pCollector
    );

// Start collecting system metrics.
// Start and stop are not thread safe!
void
RtcPalStartCollectSystemMetrics(
    __in struct RtcPalSystemMetricsCollector * pCollector
    );

// Stop collecting system metrics.
// This API will block until metric collector has been stopped.
//
// NEVER call this API from PRTCPAL_SYSTEM_METRICS_CALLBACK callback routine.
// Doing so will cause deadlock.
// Start and stop are not thread safe!
void
RtcPalStopCollectSystemMetrics(
    __in struct RtcPalSystemMetricsCollector * pCollector
    );

struct RtcPalSystemMetricsTemplate {
    uint32_t SizeOfMetricsBlob;
    uint8_t NumberOfMetrics;
    const struct RtcPalSystemMetricDescription * MetricsDescription;
};

// Description:
//  This API returns information on how to interpret the metrics. The information
//  is determined at build time for each platform.
void
RtcPalGetSystemMetricsTemplate(
    __out_ecount(1) struct RtcPalSystemMetricsTemplate * pTemplate
    );

// Description:
// This API returns the system configuration in a name=value pair for metrics
// data.  The name=value pairs are delimited with a \n character.
// For multi-field values, their delimiting template must be defined in this
// header (with an example).
//
// For wireless NICs, the value will contain description;provider:version, for
// example:
// 
// wifi=Intel(R) WiFi Link 5300 AGN;Microsoft:14.0.2.2\n
//
// PARAMETERS:
//  pwzConfigInfo - Caller allocated buffer of wchar_t.  Caller must allocate sufficient
//      amount of space for the entire string to be written.  May be NULL.
//
//  cchConfigInfo - Size in characters of allocated space pointed to by pwzConfigInfo.  If
//                  cchConfigInfo < the amount required, the string may be truncated.
//
//  pcchRequred   - Upon return this field will hold the size in characters required 
//                  for the string to be written.
HRESULT
RtcPalGetSystemConfigurationInfo(
    __inout wchar_t* pwzConfigInfo,
    __in uint32_t cchConfigInfo,
    __out uint32_t* pcchRequred
    );

#ifdef __cplusplus
};
#endif

#endif // __RTCPAL_SYSTEM_METRICS_H__

