#pragma once

/** \file rtcpal_thread_safe.h

    This file defines a thread safe wrapper.

    This class is used to access resource in multi-thread more easily. It's very
    easy to use it and code is very straight forward.

    For example:
    \code
    class WorkEngine
    {
    public:
        // this function may be invoked from producer thread.
        void AddWorkItem(WorkItem item);
        
        // this function may be invoked from consumer thread.
        void ProcessWorkItem();
        
    private:
        // this element may be accessed by producer thread (via AddWorkItem()) or
        // may be accessed by consumer thread, so mark it as thread safe.
        RtcPalThreadSafe<std::list<WorkItem>> m_workItems;
    };
    
    void WorkEngine::AddWorkItem(WorkItem item)
    {
        m_workItems.push_back(item);    // compiler-error, you can't invoke push_back() directly!!
                                        
        // you must via an accessor to access it.
        RtcPalThreadSafe<std::list<WorkItem>>::Accessor accessor = m_workItems.GetAccessor();
        accessor->push_back(item);
    }
    
    void WorkEngine::ProcessWorkItem(WorkItem item)
    {
        WorkItem item;
        
        item = m_workItems.front();     // compiler-error, you can't invoke front() directly!
        m_workItems.pop_front();        // compiler-error, you can't invoke pop_front() directly!
                                        
        // you must via an accessor to access it. If your compiler support c++11, you
        // can use auto syntax, it's very convenient!! Who says "auto" in c++11 is nice to 
        // have? It is must to have!
        {
            auto accessor = m_workItems.GetAccessor();
            item = accessor->front();
            accessor->pop_front();
        }   // lock will be released here, the lock's scope is the same as the accessor's scope.
        
        //.... process work item here ...
    }
    \endcode
    
    This wrapper doesn't only support class, it also support POD data. You can use it like this:
    
    \code
    RtcPalThreadSafe<int> threadSafeInt;
    
    RtcPalThreadSafe<int>::Accessor accessor = threadSafeInt.GetAccessor();
    *accessor = 19890604;
    \endcode
*/

#include <rtcpal_platform_specific.h>

template<typename T>
class RtcPalThreadSafe
{
public:
    class Accessor
    {
    friend class RtcPalThreadSafe;
    
    public:
        T* operator->() { return &m_threadSafeWrapper->m_obj; }
        T& operator*() { return m_threadSafeWrapper->m_obj; }
        
        Accessor(const Accessor& accessor)
        {
            (void)operator=(accessor);
        }
        
        Accessor& operator=(const Accessor& accessor)
        {
            if (this != &accessor)
            {
                // transfer the lock's ownership.
                m_threadSafeWrapper = accessor.m_threadSafeWrapper;
                accessor.m_threadSafeWrapper = NULL;
            }
            return *this;
        }
        
        ~Accessor()
        {
            if (m_threadSafeWrapper)
            {
                // if it takes the ownership at the second, release the lock.
                m_threadSafeWrapper->ReleaseLock();
            }
        }
        
    private:
        Accessor(RtcPalThreadSafe* wrapper) : m_threadSafeWrapper(wrapper) 
        {
        }
        
        mutable RtcPalThreadSafe* m_threadSafeWrapper;
    };
    
    class Reader
    {
    friend class RtcPalThreadSafe;
    
    public:
        const T* operator->() const { return &m_threadSafeWrapper->m_obj; }
        const T& operator*() const { return m_threadSafeWrapper->m_obj; }
        
    private:
        Reader(const RtcPalThreadSafe* wrapper) : m_threadSafeWrapper(wrapper) 
        {
        }
        
        const RtcPalThreadSafe* m_threadSafeWrapper;
    };
    
    friend class RtcPalThreadSafe::Accessor;
    friend class RtcPalThreadSafe::Reader;
    
    RtcPalThreadSafe()
    {
        (void)RtcPalInitializeCriticalSection(&m_lock);
    }
    
    ~RtcPalThreadSafe()
    {
        (void)RtcPalDeleteCriticalSection(&m_lock);
    }
    
    Accessor GetAccessor()
    {
        AcquireLock();
        return this;
    }
    
    Reader GetReader()
    {
        return this;
    }
    
private:
    void AcquireLock()
    {
        (void)RtcPalEnterCriticalSection(&m_lock);
    }
    
    void ReleaseLock()
    {
        (void)RtcPalLeaveCriticalSection(&m_lock);
    }
    
    RTCPAL_CRITICAL_SECTION m_lock;
    T m_obj;
};
