// FILE: RTCPal_Tracing.h
//
// PURPOSE: TRACE function definitions for RTCPAL
//          and related DLL's.

#pragma once

/**********************************************************************
 * Overview
 **********************************************************************/

/*
 * A trace statement using this logger uses 4 pieces of information:
 *
 * 1. The class of log, e.g. error or warning.
 * 2. The group of log, e.g. RTP, RTCP, Redundancy, etc.
 * 3. A selection withing the given group, e.g. in RTP: reception
 * 4. The format specification and probably some parameters
 *
 * These are examples of how trace staments might look like:
 *
 * TraceRetail((CLASS_ERROR, GROUP_RTP, SELECTION_RTP_RECV,
 *              "%s: Object [%p] is in the invalid state, hr=0x%X",
 *              "MyRtpReceptionFunctionName",
 *              pReceptionObject,
 *              hr));
 *
 * The ouput would look similar to this:
 *
 * 13:12.329 5A4:FC0 ERROR  :: MyRtpReceptionFunctionName: Object [0x06A486E0]
 *  is in the invalid state, hr=0xC0590031
 *
 * The above log statement will produce ouput depenindig of the following:
 *
 * + Either (or a combination) of File | Console | Debugger is
 * enabled, if UseAdvancedTracing is set to 0, that will be the only
 * condition tested to decide to log or not
 *
 * + If UseAdvancedTracing is set to 1, then the class, group and
 * selection will be used, in this case, if class ERROR is enabled the
 * log will be output, or, if the selection "receive" in the group RTP
 * is set.
 *
 * The TraceRetail macro would have included the particular logger to
 * use, which would be the same for the whole module.
 *
 * The same example as above, but not using at all advanced tracing:
 *
 * LOG(("%s: Object [%p] is the invalid state, hr=0x%X",
 *      "MyRtpReceptionFunctionName",
 *       pReceptionObject,
 *       hr));
 *
 * The ouput would look similar to this:
 *
 * 13:12.329 5A4:FC0 :: MyRtpReceptionFunctionName: Object [0x06A486E0]
 *  is in the invalid state, hr=0xC0590031
 *
 * In the case, the macro LOG also includes the particular logger to
 * use, and gives values of 0 to all the class, group and selection.
 *
 * As you can see, the logger is optimized towards the advanced
 * tracing and its usage is encouraged as the non advanced tracing
 * enables everything in 1 shot with no posibility of any kind of
 * granularity.
 *
 */

/* For sample code see:
 *
 * This is stand alone test tool toplay with the tracer, it creates
 * entry TestTraceV2.0 in the registry:
 * media/stream/test/trace
 *
 * -or-
 *
 * The RTP logging support, you can copy this code and modify it to
 * your module needs:
 * media/stream/rtp/rtpstack/inc/rtpdbg.h
 * media/stream/rtp/rtpstack/rtpdbg.cpp
 *
 */

/**********************************************************************
 * Macros and enums
 **********************************************************************/
#ifndef TL_NONE
#define TL_NONE     0
#endif

#ifndef TL_FATAL
#define TL_FATAL    1
#endif

#ifndef TL_ERROR
#define TL_ERROR    2
#endif

#ifndef TL_WARN
#define TL_WARN     3
#endif

#ifndef TL_INFO
#define TL_INFO     4
#endif

#ifndef TL_VERBOSE
#define TL_VERBOSE  5
#endif

#ifndef TL_NOISE
#define TL_NOISE    6
#endif

#if defined(DEBUG) && defined(ANDROID)
#include <android/log.h>
#endif

/*
 * WARNING
 *
 * Modifying the trace classes MUST keep in sync the following:
 *
 * + The enum definition below, for the clasess
 * + The corrresponding variables in lcctrace.h/RtcPalLogRegistry_t
 * + The corresponding entries in lcctrace.cpp/g_dwRtcPalRegistryReadCtrl
 * + The classes name in lcctrace.cpp/g_psRtcPalLogClassName
 */
/* Class */
enum
{
    CLASS_BEGIN,

    /* *** Log classes *** */
    CLASS_ERROR,
    CLASS_WARNING,
    CLASS_REFCOUNT,
    CLASS_INFO,
    CLASS_TRACE,
    CLASS_EXPERT,
    CLASS_EXPERIMENTAL,
    /* *** *********** *** */

    CLASS_END,

    CLASS_NONE = CLASS_BEGIN,
    CLASS_ADVANCEDONLY = CLASS_BEGIN,
    CLASS_FIRST = CLASS_BEGIN + 1,
    CLASS_LAST = CLASS_END - 1
};

#define INVALID_TRACEID         0xFFFFFFFF

/* This mask is used to make any other class also log to single file */
#define CLASS_MASK_SINGLEFILE   0x00010000

/* Flags to use in higher 16 bits of dwNumGroups in RtcPalTraceRegister() */
/* This flag indicates this will be a single file logger */
#define TRACEFLAG_ISSINGLEFILE (1 << (16+0))


#ifndef ENABLE_FILE_TRACE

    //global switch to turn on/off file based tracing
    #if defined(DBG) || defined(DEBUG)
        #define ENABLE_FILE_TRACE 2
    #else
        #define ENABLE_FILE_TRACE 0
    #endif

#endif

// Map from WPP levels specified in evntrace.h
DECLSPEC_SELECTANY DWORD g_aWppLevelMap[] =
{
    CLASS_BEGIN,        // 0 - maps from TRACE_LEVEL_NONE
    CLASS_ERROR,        // 1 - maps from TRACE_LEVEL_CRITICAL    and TRACE_LEVEL_FATAL
    CLASS_ERROR,        // 2 - maps from TRACE_LEVEL_ERROR       and TL_ERROR
    CLASS_WARNING,      // 3 - maps from TRACE_LEVEL_WARNING     and TL_WARN
    CLASS_INFO,         // 4 - maps from TRACE_LEVEL_INFORMATION and TL_INFO
    CLASS_ADVANCEDONLY  // 5 - maps from TRACE_LEVEL_VERBOSE     and TL_VERBOSE
};

#define WPPMAP(LEVEL) g_aWppLevelMap[LEVEL]

#if defined(__OBJC__)

    static const void * const self = NULL;

    #define _IF_EXISTS_THIS_GET_TRACE_ID    GET_TRACE_ID_NOT_SUPPORTED_IN_OBJ_C
    #define _IF_EXISTS_THIS_GET_THIS_BLOCK  ,self
    #define _IF_EXISTS_THIS_GET_THIS_FMT    "[0x%p]"
    #define _IF_NOT_EXISTS_THIS_GET_0       0

#elif defined(__cplusplus) && !defined(__GNUC__)

    #define _IF_EXISTS_THIS_GET_TRACE_ID    __if_exists(this){GetTracingId()}
    #define _IF_EXISTS_THIS_GET_THIS_BLOCK  __if_exists(this){,this}
    #define _IF_EXISTS_THIS_GET_THIS_FMT    __if_exists(this){"[0x%p]"}
    #define _IF_NOT_EXISTS_THIS_GET_0       __if_not_exists(this){0}

#else

    #define _IF_EXISTS_THIS_GET_TRACE_ID
    #define _IF_EXISTS_THIS_GET_THIS_BLOCK
    #define _IF_EXISTS_THIS_GET_THIS_FMT
    #define _IF_NOT_EXISTS_THIS_GET_0       0

#endif

#if defined(UNDER_CE) || defined(__RTCPAL_FORCE_FILE_TRACING__)
    #define WPP_LEVEL_ENABLED(LEVEL) false

    // ATTN: Make sure that the compound condition manipulates only constants known at pre-processing or compile time.
    //       This is needed so to ensure that the trace statements not passing the condition are compiled out.
    // For debug build, output warning, error, info. For ship build, output warning, error
    #ifdef DEBUG
        #define IF_COMPILE_TRACE(LEVEL)   if(LEVEL == TL_ERROR || LEVEL == TL_WARN || LEVEL == TL_INFO || TL_VERBOSE)
    #else
        #ifdef __GNUC__
            #define IF_COMPILE_TRACE(LEVEL)   if(LEVEL == TL_ERROR || LEVEL == TL_WARN || LEVEL == TL_INFO || LEVEL == TL_VERBOSE)
        #else
            #define IF_COMPILE_TRACE(LEVEL)   if(LEVEL == TL_ERROR || LEVEL == TL_WARN)
        #endif
    #endif

    #define WPP_FLAVOR_LEVEL_GROUP_FLAGS_ENABLED(FLAVOR,LEVEL,GROUP,FLAGS) false

    #define WPP_FLAGS_ENABLED(FLAGS)  true

#else

    #define IF_COMPILE_TRACE(LEVEL)

    #define WPP_FLAVOR_LEVEL_GROUP_FLAGS_ENABLED(FLAVOR,LEVEL,GROUP,FLAGS)   \
        (( WPP_LEVEL_ENABLED(FLAGS)                   \
                 && (WPP_CONTROL(WPP_BIT_##FLAGS).Level >= LEVEL)))

    #define WPP_FLAVOR_LEVEL_GROUP_FLAGS_LOGGER(FLAVOR,LEVEL,GROUP,FLAGS)    \
        (WPP_CONTROL(WPP_BIT_ ## FLAGS).Logger),

    #define WPP_FLAGS_ENABLED(FLAGS) \
        (WPP_CONTROL(WPP_BIT_ ## FLAGS).Flags[WPP_FLAG_NO(WPP_BIT_ ## FLAGS)] & WPP_MASK(WPP_BIT_ ## FLAGS))
    #define WPP_FLAGS_LOGGER(FLAGS)  (WPP_CONTROL(WPP_BIT_ ## FLAGS).Logger),

#endif // #if defined(UNDER_CE) || defined(__RTCPAL_FORCE_FILE_TRACING__)

#define WPP_COMPID_ENABLED(FLAGS)   WPP_FLAGS_ENABLED(FLAGS)

#define WPP_FLAVOR_LEVEL_GROUP_FLAGS_CORE_PRE(FLAVOR, LEVEL, GROUP, FLAGS, MSG) \
    { IF_COMPILE_TRACE(LEVEL) {                            \
        Trace##FLAVOR(WPPMAP(LEVEL), GROUP, WPP_BIT_##FLAGS, __FUNCTION__ \
        _IF_EXISTS_THIS_GET_THIS_FMT ": " MSG _IF_EXISTS_THIS_GET_THIS_BLOCK

#define WPP_FLAVOR_LEVEL_GROUP_FLAGS_ARG_PRE(FLAVOR, LEVEL, GROUP, FLAGS, MSG, ...)       \
        WPP_FLAVOR_LEVEL_GROUP_FLAGS_CORE_PRE(FLAVOR, LEVEL, GROUP, FLAGS, MSG), __VA_ARGS__); \
    }

#define WPP_FLAVOR_LEVEL_GROUP_FLAGS_ARG_POST(FLAVOR, LEVEL, GROUP, FLAGS, ...)       ; }

#define WPP_FLAVOR_LEVEL_GROUP_FLAGS_NARG_PRE(FLAVOR, LEVEL, GROUP, FLAGS, MSG)            \
        WPP_FLAVOR_LEVEL_GROUP_FLAGS_CORE_PRE(FLAVOR, LEVEL, GROUP, FLAGS, MSG));                \
    }

#define WPP_FLAVOR_LEVEL_GROUP_FLAGS_NARG_POST(FLAVOR, LEVEL, GROUP, FLAGS, ...)      ; }

#define WPP_FLAGS_CORE_PRE(FLAGS, MSG) \
    { {                            \
        TraceRetail(CLASS_MASK_SINGLEFILE | CLASS_INFO, 0, WPP_BIT_##FLAGS, __FUNCTION__ \
        _IF_EXISTS_THIS_GET_THIS_FMT ": " MSG _IF_EXISTS_THIS_GET_THIS_BLOCK

#define WPP_FLAGS_ARG_PRE(FLAGS, MSG, ...)       \
        WPP_FLAGS_CORE_PRE(FLAGS, MSG), __VA_ARGS__); \
    }

#define WPP_FLAGS_ARG_POST(FLAGS, MSG, ...)    ; }

#define WPP_FLAGS_NARG_PRE(FLAGS, MSG)            \
        WPP_FLAGS_CORE_PRE(FLAGS, MSG));                \
    }

#define WPP_FLAGS_NARG_POST(FLAGS, MSG)    ; }

#define WPP_CORE_PRE(MSG) \
    { { \
        TraceRetail(CLASS_MASK_SINGLEFILE | CLASS_INFO, 0, 0, __FUNCTION__ \
        _IF_EXISTS_THIS_GET_THIS_FMT ": " MSG _IF_EXISTS_THIS_GET_THIS_BLOCK

#define WPP_ARG_PRE(MSG, ...) \
        WPP_CORE_PRE(MSG), __VA_ARGS__); \
    }

#define WPP_ARG_POST(MSG, ...)  ; }

#define WPP_NARG_PRE(MSG)            \
        WPP_CORE_PRE(MSG));                \
    }

#define WPP_NARG_POST(MSG)  ; }

#if defined(HTRACE)
    static inline int GetTracingId() { return 0; }
    extern unsigned int g_traceEnableBitMap;

    #ifndef TRACE
        #ifdef __cplusplus
        
            #define TRACE(FLAVOR, LEVEL, GROUP, FLAGS, MSG, ...)    \
                do {    \
                    IF_COMPILE_TRACE(LEVEL) {   \
                        if ((1 << (LEVEL - 1)) & g_traceEnableBitMap) {  \
                            Trace<__LINE__, sizeof(__FILE__)>(0 _IF_EXISTS_THIS_GET_THIS_BLOCK, ##__VA_ARGS__);    \
                        }   \
                    }   \
                } while(false)
        #else
            #define TRACE(FLAVOR, LEVEL, GROUP, FLAGS, MSG, ...)    \
                do {    \
                    IF_COMPILE_TRACE(LEVEL) {   \
                        if ((1 << (LEVEL - 1)) & g_traceEnableBitMap) {  \
                            Trace(__LINE__, sizeof(__FILE__) _IF_EXISTS_THIS_GET_THIS_BLOCK, ##__VA_ARGS__);    \
                        }   \
                    }   \
                } while(false)
        #endif
    #endif
    #ifndef TRACE_COMPID
        #if defined(__OBJC__)
            #define TRACE_COMPID(FLAGS, MSG, ...)    \
                do {    \
                    Trace<__LINE__, sizeof(__FILE__)>(0 _IF_EXISTS_THIS_GET_THIS_BLOCK, ##__VA_ARGS__);   \
                } while(false)
        #elif defined(__cplusplus)
            #define TRACE_COMPID(FLAGS, MSG, ...)    \
                do {    \
                    Trace<__LINE__, sizeof(__FILE__)>(0, static_cast<intptr_t>(GetTracingId()), ##__VA_ARGS__);   \
                } while(false)
        #else
            #define TRACE_COMPID(FLAGS, MSG, ...)    \
                do {    \
                    Trace(__LINE__, sizeof(__FILE__), 0, ##__VA_ARGS__);    \
                } while(false)
        #endif
    #endif
    #ifndef CTRACE_COMPID
        #ifdef __cplusplus
            #define CTRACE_COMPID(FLAGS, MSG, ...)    \
                do {    \
                    Trace<__LINE__, sizeof(__FILE__)>(0, ##__VA_ARGS__);    \
                } while(false)
        #else
            #define CTRACE_COMPID(FLAGS, MSG, ...)    \
                do {    \
                    Trace(__LINE__, sizeof(__FILE__), ##__VA_ARGS__);   \
                } while(false)
        #endif
    #endif
#else
    #ifdef __RTCPAL_FORCE_FILE_TRACING__

        #define TRACE(FLAVOR, LEVEL, GROUP, FLAGS, MSG, ...)    \
            IF_COMPILE_TRACE(LEVEL) {                                                                       \
                Trace##FLAVOR(                                                                                          \
                    WPPMAP(LEVEL), \
                    GROUP, \
                    WPP_BIT_##FLAGS, \
                    _IF_EXISTS_THIS_GET_THIS_FMT ": " MSG _IF_EXISTS_THIS_GET_THIS_BLOCK,                 \
            ##__VA_ARGS__);                                                                                      \
            }

        #define TRACE_COMPID(FLAGS, MSG, ...)    \
            {                                                                       \
                TraceRetail(                                                                                          \
                    CLASS_MASK_SINGLEFILE | CLASS_INFO, \
                    0, \
                    WPP_BIT_##FLAGS, \
                    _IF_EXISTS_THIS_GET_THIS_FMT ": " MSG _IF_EXISTS_THIS_GET_THIS_BLOCK,                 \
            ##__VA_ARGS__);                                                                                      \
            }

        #define CTRACE_COMPID(FLAGS, MSG, ...)    \
            {                                                                       \
                TraceRetail(                                                                                          \
                    CLASS_MASK_SINGLEFILE | CLASS_INFO, \
                    0, \
                    WPP_BIT_##FLAGS, \
                    _IF_EXISTS_THIS_GET_THIS_FMT ": " MSG _IF_EXISTS_THIS_GET_THIS_BLOCK,                 \
            ##__VA_ARGS__);                                                                                      \
            }

    #endif
#endif

/**********************************************************************
 * API
 **********************************************************************/
#ifdef __cplusplus
extern "C" {
#endif

HRESULT RtcPalInitTracing(const char *pTraceQualifier);
HRESULT RtcPalDeinitTracing(void);

#if ENABLE_FILE_TRACE > 0
/* The RtcPalTraceRegister() function creates a new trace ID and
 * registers a file pointer name with this trace ID, on failure return
 * INVALID_TRACEID
 *
 * WARNING The array of names must be read only memory, those names
 * are not copied internally but just used from the given pointer
 * passed */
DWORD RtcPalTraceRegister(
        const char        *psCallerName,
        DWORD        dwNumGroups,  /* 16 msb: flags; 16 lsb:Number of
                                    * groups to use */
        char const **ppsGroupNames /* Pointer to an array of strings */
);

/* This is the complementary function to RtcPalTraceRegister, you call
 * this function whenever you don't need the logger anymore */
DWORD RtcPalTraceDeregister(
        DWORD        dwTraceID
);
#endif // ENABLE_FILE_TRACE > 0

// shortcut for attaching and creating single-file trace
void
RtcPalTrace_ProcessAttached(
    void
    );


// shortcut for detaching and creating single-file trace
void
RtcPalTrace_ProcessDetached(
);


#if ENABLE_FILE_TRACE > 0
/* Note this receives arglist instead of the variable number of
 * arguments */
size_t RtcPalTraceVprintfEx(
        DWORD        dwTraceID,
        DWORD        dwClass,    /* 0, 1, ... */
        DWORD        dwGroup,    /* 0, 1, ... */
        DWORD        dwSelection,/* 0x00000001, 0x00000002, 0x0000004, ... */
        const char  *psFormat,
        va_list      arglist
);

/* Returns the mask of logs enabled, e.g. */
DWORD RtcPalGetLogsEnabled(
        DWORD        dwTraceID
);
#endif // ENABLE_FILE_TRACE > 0

/* Get the trace folder */
HRESULT RtcPalTraceGetRootDir(
    __out_ecount(uBufferLen) char* const psBuffer,
    __in                     DWORD uBufferLen
    );

EXTERN_C
void
RTCAPI
RtcPalFlushTracing(
    void
    );

EXTERN_C
RTCPAL_BOOL
RTCAPI
RtcPalIsTracingEnabled(
    void
    );

EXTERN_C
RTCPAL_ERROR
RTCAPI
RtcPalEnableTracing(
    RTCPAL_BOOL bEnabled
    );

#ifdef __cplusplus
}
#endif

/* Bitmask returned by RtcPalGetLogsEnabled() */
#define RTCPALTRACE_DEBUGGER       0x1
#define RTCPALTRACE_CONSOLE        0x2
#define RTCPALTRACE_FILE           0x4
#define RTCPALTRACE_SINGLEFILE     0x8

#define RTCPALTRACE_ALLLOGS        (RTCPALTRACE_DEBUGGER |    \
                                 RTCPALTRACE_CONSOLE  |    \
                                 RTCPALTRACE_FILE     |    \
                                 RTCPALTRACE_SINGLEFILE)
