/*++

Copyright (C) Microsoft Corporation. All rights reserved.

Module Name:

    rtcpal_threadpool.h

Abstract:

    PAL thread pool API

    PAL work item APIs separate resource allocation from resource usage
    using the following API model

        RtcPalAllocateXXXWorkItem   ==> resource allocation can fail
        RtcPalQueueXXXWorkItem      ==> resource usage never fail

    When reliability guarantee is requried, you should pre-allocate the
    work item before you commit to doing the work.

    1. Concurrent work item
       As the name suggest, concurrent work item can be run concurrently.
       Work item can be dispatched out of order.

       The following API are used for concurrent work item

            RtcPalAllocateWorkItem
            RtcPalQueueWorkItem
            RtcPalFreeWorkItem

    2. Serial work item
       Serial work item is dispatched and retire sequencially, that is,
       PAL won't dispatch a new work item until the previous work item
       from the same queue has finished.
       Serial work items from different queues can still run concurrently.

       The following API are used for serial work item

            RtcPalAllocateSerialWorkItemQueue
            RtcPalAllocateSerialWorkItem
            RtcPalQueueSerialWorkItem
            RtcPalFreeSerialWorkItem
            RtcPalFreeSerialWorkItemQueue

    3. Timer Work Item - To Be Added Later
       A work item that will call a callback at a specified timeout.

    Work Item Priority
        For concurrent work item, priority is set at workitem level.
        For serial work item, priority is set at the workitem queue level.

        In W15, none of the threads we want to consolidate requires
        priority settings. Therefore, PAL supports only one priority level -
        RTCPAL_WORKITEM_PRIORITY_NORMAL.
    
    IRtcPalAsyncTask
        Helper class available in C++ to more easily manage serialization and
        termination of work item queues.

Author:

Revision History:

--*/
#ifndef __RTCPAL_WORKITEM_H__
#define __RTCPAL_WORKITEM_H__

#ifdef __cplusplus
extern "C" {
#endif

typedef struct RtcPalWorkItem * PRTCPAL_WORKITEM;

typedef enum RTCPAL_WORKITEM_PRIORITY {
    RTCPAL_WORKITEM_PRIORITY_LOW = 0,
    RTCPAL_WORKITEM_PRIORITY_NORMAL,
    RTCPAL_WORKITEM_PRIORITY_HIGH
} RTCPAL_WORKITEM_PRIORITY;

typedef void (RTCAPI * PRTCPAL_WORKITEM_ROUTINE)(
    PRTCPAL_WORKITEM pWork,
    void * CallbackContext
    );

// API for concurrent work items
PRTCPAL_WORKITEM
RtcPalAllocateWorkItem(
    RTCPAL_WORKITEM_PRIORITY Priority,
    PRTCPAL_WORKITEM_ROUTINE WorkerRoutine,
    PVOID pvCallbackContext
    );

void
RtcPalQueueWorkItem(
    PRTCPAL_WORKITEM pWork
    );

void
RtcPalFreeWorkItem(
    PRTCPAL_WORKITEM pWork
    );

// APIs for serialized work items
typedef struct RtcPalSerialWorkItem * PRTCPAL_SERIAL_WORKITEM;
typedef struct RtcPalSerialWorkItemQueue * PRTCPAL_SERIAL_WORKITEM_QUEUE;

typedef void (RTCAPI * PRTCPAL_SERIAL_WORKITEM_ROUTINE)(
    PRTCPAL_SERIAL_WORKITEM pWork,
    void * CallbackContext
    );

PRTCPAL_SERIAL_WORKITEM_QUEUE
RtcPalAllocateSerialWorkItemQueue(
    RTCPAL_WORKITEM_PRIORITY Priority
    );

void
RtcPalFreeSerialWorkItemQueue(
    PRTCPAL_SERIAL_WORKITEM_QUEUE SerialQueue
    );

PRTCPAL_SERIAL_WORKITEM
RtcPalAllocateSerialWorkItem(
    __in PRTCPAL_SERIAL_WORKITEM_ROUTINE WorkerRoutine,
    __in PVOID pvCallbackContext
    );

void
RtcPalQueueSerialWorkItem(
    __in PRTCPAL_SERIAL_WORKITEM_QUEUE pSerialQueue,
    __in PRTCPAL_SERIAL_WORKITEM pWork
    );

void
RtcPalFreeSerialWorkItem(
    PRTCPAL_SERIAL_WORKITEM pWork
    );

#ifdef __cplusplus
};


// Asynchronous task interface, for easy scheduling of thredpool work items
struct IRtcPalAsyncTask
{
    // Lifetime management
    virtual ULONG Release() = 0;
    
    // Schedule and execute
    virtual HRESULT ScheduleAsyncTask() = 0;
};


// IRtcPalAsyncTask callback interface
struct IRtcPalAsyncTaskCallback
{
    // Threadpool callback requsted by IRtcPalAsyncTask::ScheduleAsyncTask
    virtual HRESULT ExecuteAsyncTask() = 0;
};


// IRtcPalAsyncTask instance factory function
extern "C" HRESULT RtcPalCreateAsyncTask(
    RTCPAL_WORKITEM_PRIORITY ePriority,
    RTCPAL_BOOL fSerializeTasks,
    IRtcPalAsyncTaskCallback* pCallback,
    IRtcPalAsyncTask** ppAsyncTaskOut
);

#endif

#endif // __RTCPAL_WORKITEM_H__

