// -----------------------------------------------------------------------
// <copyright file="CertStoreOpenssl.h" company="Microsoft">
//     Copyright (c) Microsoft. All rights reserved.
// </copyright>
// -----------------------------------------------------------------------

#pragma once

#if !defined(_WIN32)

#ifndef _CERTIFICATESTOREOPENSSL_H_
#define _CERTIFICATESTOREOPENSSL_H_

#include <memory>
#include "CertOpenssl.h"
#include "rtcpal_util.h"

/// <summary>
/// Represents a OpenSSL certificate store.
/// </summary>
class RtcPalCertificateStore
{
public:
    /// <summary>
    /// Initializes a CertificateStore object.
    /// </summary>
    RtcPalCertificateStore(
        const std::wstring& certFile,
        const std::wstring& privateKeyFile,
        const std::wstring& sslDigest) :
        m_CertFile(wstring2string(certFile)),
        m_PrivateKeyFile(wstring2string(privateKeyFile)),
        m_SslDigest(wstring2string(sslDigest))
        {

        }

    RtcPalCertificateStore(
        const std::string& certFile,
        const std::string& privateKeyFile,
        const std::string& sslDigest) :
        m_CertFile(certFile),
        m_PrivateKeyFile(privateKeyFile),
        m_SslDigest(sslDigest)
        {

        }

    /// <summary>
    /// Destroys the object and releases all associated resources.
    /// </summary>
    ~RtcPalCertificateStore()
    {
        
    }

    /// <summary>
    /// Loads certificate from the store by certificate's thumbprint.
    /// </summary>
    std::shared_ptr<RtcPalCertificate> LoadCertificate(const std::string& thumbprint);

private:
    /// Read a public certificate object from the file. The file
    /// must be a plain text file in PEM format.
    /// Caller function needs to call X509_free(X509*) to free the object.
    /// Return X509* object or NULL if any error.
    std::shared_ptr<X509> ReadCertFromFile();

    /// Read a private key object from the file. The file
    /// must be in a plain text file in PEM format.
    /// Caller function needs to call EVP_PKEY_free(EVP_PKEY*) to free the object.
    /// Return EVP_PKEY* object or NULL if any error.
    std::shared_ptr<EVP_PKEY> ReadPrivateKeyFromFile();

    /// Get the certificate digest object.
    /// If m_SslDigest is empty, or if any error occurs when using the given
    /// non-empty SslDigest, return SHA1 digest object.        
    const EVP_MD* GetCertDigest();

    /// Return the thumbprint in upper case for the given cert object.
    /// If any error, return "".
    std::string GetCertThumbprint(X509* cert);


    std::string m_CertFile;
    std::string m_PrivateKeyFile;
    std::string m_SslDigest;
};


#endif // _CERTIFICATESTOREOPENSSL_H_
#endif // !_WIN32
