#pragma once
#include "rtcpal.h"
#include "rtcpal_auto_critical_section.h"
#if defined(__linux)
#include <unistd.h>
#endif

const unsigned int g_kTraceBufferUnitSize   = 1024 * 1024;      // 1MB

class HTraceBuffer;
extern HTraceBuffer* g_pHTraceBuffer;
extern unsigned int g_uTraceMaxFileSize;

#if !defined(RTCPAL_TRACE_FLUSH_INTERVAL)
#define RTCPAL_TRACE_FLUSH_INTERVAL 0xFFFFFFFFFFFFFFFFULL
#endif

class HTraceBuffer
{
    friend class HTraceSignalHooker;
    
    class Buffer
    {
        int m_pos;
        bool m_fBusy;
        int m_usage;
        aiocb m_aiocb;
        char m_buffer[g_kTraceBufferUnitSize];
        
    public:
        Buffer() : m_pos(0), m_fBusy(false), m_usage(0) 
        {
            memset(&m_aiocb, 0, sizeof(m_aiocb));
        }
        
        int GetContentSize() {return m_pos;}
        
        /** Require the buffer. The user who wants output the trace, it must invokes
            this function to ask for reserving the space. If there is no enough 
            sapce return NULL. */
        void* Require(int size)
        {
            // if the buffer doesn't have enouth space, or the buffer is writing,
            // return directly.
            if ((g_kTraceBufferUnitSize - m_pos) < size || m_fBusy)
            {
                return NULL;
            }
            
            // reserve the space.
            char* res = &m_buffer[m_pos];
            m_pos += size;
            
            // increse the usage ref count.
            InterlockedIncrement(&m_usage);
            return res;
        }
        
        /** After finish the trace writing, user must invoke this function to 
            release the buffer reserved by Require(). */
        void Release()
        {
            // decrese the usage ref count.
            InterlockedDecrement(&m_usage);
        }
        
        /** Flush imediately. */
        bool Flush(int fd)
        {
            int i = m_pos;
            char* p = m_buffer;
            while (i > 0)
            {
                int r = write(fd, p, i);
                if (r <= 0)
                {
                    return false;
                }
                p += r;
                i -= r;
            }
            return true;
        }
        
        /** flush the buffer to file with aio */
        bool AsyncFlush(int fd);
        
        /** Wait aio command finished. 
            \return true if success */
        bool Wait()
        {
            if (m_fBusy)
            {
                m_fBusy = false;
                m_pos = 0;
                m_usage = 0;
                aiocb* cbs[] = {&m_aiocb};
                bool res = !rtcpal_aio_suspend(cbs, 1, INFINITE);
                // because of our stupid design for android aio, we must call
                // rtcpal_aio_return to avoid memory leak.
                (void)rtcpal_aio_return(&m_aiocb);
                return res;
            }
            return true;
        }
    };
    
public:
    HTraceBuffer();
    ~HTraceBuffer();
    HRESULT Init(const char* pszQualifier) throw();
    
    /** Return the trace buffer to the caller thread.

    This function must be thread safe.
    */
    void* Require(unsigned int size)
    {
        if (RTCPAL_UNLIKELY(m_hTrace < 0))
        {
            return NULL;
        }
        
        RtcPalAutoCriticalSection lock(&m_hLock);

        if (RtcPalGetTimeLongIn100nsFast() - m_ullLastFlushTime > RTCPAL_TRACE_FLUSH_INTERVAL)
        {
            _FlushAndSwitchBuffer();
        }

        void* p = m_pBuffer->Require(size);
        if (!p && _FlushAndSwitchBuffer() == S_OK)
        {
            p = m_pBuffer->Require(size);
        }
        return p;
    }
    
    void Release(void* p)
    {
        if ((size_t)p >= (size_t)&m_buffer2)
        {
            m_buffer2.Release();
        }
        else
        {
            m_buffer1.Release();
        }
    }
    
    /** Same as Flush(), but it is thread-safe.
    
        \note This function can't be invoked in signal handler. Otherwise, it 
        may cause deadlock! Because, we may be in Require, the lock has been added
        for this thread already. So we can't added lock in this function again. If 
        you need flush this trace in signal handler, please invoke Flush() directly.
    */
    HRESULT Flush()
    {
        if (RTCPAL_UNLIKELY(m_hTrace < 0))
        {
            return S_OK;
        }
        
        RtcPalAutoCriticalSection lock(&m_hLock);
        
        return _FlushAndSwitchBuffer(true/*fSync*/);
    }
    
private:
    void _OpenTraceFileForWritting(const char* filePath) throw(HRESULT);
    void _CloseTraceFile() throw();
    
    /**
        \param fForceFlush  If true, don't wait other thread to finish the trace 
                            writing and flush the trace buffer right now. Some traces 
                            may be missed. So it should be only used under rare 
                            scenarios (e.g. in crash handler). Default value is false,
                            means it will wait all thread finish the trace writing 
                            before flush the trace buffer.
    */
    HRESULT _FlushAndSwitchBuffer(bool fSync = false, bool fForceFlush = false) throw();
    
    CRITICAL_SECTION m_hLock;

    // use two buffers to support aio. at the same time, there is at most only one
    // buffer is writing.    
    Buffer m_buffer1;
    Buffer m_buffer2;
    
    /// point to the active buffer
    Buffer* m_pBuffer;
    
    int m_hTrace;
    char m_szFile0[_MAX_PATH + 1];
    char m_szFile1[_MAX_PATH + 1];
    const char* m_pczCurrentFile;
    const char* m_pczBakFile;
    int m_iFileSize;
    uint64_t m_ullLastFlushTime;
};
