#pragma once
/* --

Copyright (c) Microsoft Corporation. All rights reserved.

Module Name:

    mmvr_cagl.h (external interface)

Abstract:

    Core Animation OpenGL-based MMVR client interface, internal interface for Video PAL is not included here

Specification:

    http://livecommteam/sites/main/media/W15/Media Stack W15 Plan Tracking/W15 PAL Video Extension Design Doc.docm

--*/


#ifndef __MMVR_CAGL__
#define __MMVR_CAGL__

#include <mmvr_Common.h>

#ifdef __cplusplus
extern "C" {
#endif

struct MMVRMacGLRenderer;
    
typedef struct MMVRMacGLRenderer*  HMMVRMACGLRENDERER;

// =============================================================================
// Enums
// =============================================================================

// -----------------------------------------------------------------------------
// MediaVREvent_e enumeration
//
// Enumeration of MMVR events used in the event callback function as a parameter
// -----------------------------------------------------------------------------
typedef enum _MediaVREvent_e
{
    MediaVREvent_GenericError = -1,
    
    // Format change notification events
    MediaVREvent_VideoSizeChange,

} MediaVREvent_e;

// -----------------------------------------------------------------------------
// MediaVRAutoFitMode_e enumeration
//
// Enumeration of MMVR present modes. Supported modes are
//      MediaVRAutoFit_Stretch - Stretch both dimensions disregarding 
//                               original aspect ratio
//      MediaVRAutoFit_Fit     - Fit entire video preserving original
//                               aspect ratio, with letterbox or pillars
//      MediaVRAutoFit_Crop    - Crop video to known region of interest,
//                               preserving original aspect ratio
// -----------------------------------------------------------------------------
typedef enum _MediaVRAutoFitMode_e
{
    MediaVRAutoFit_Stretch = 0,
    MediaVRAutoFit_Fit,
    MediaVRAutoFit_Crop,
    
} MediaVRAutoFitMode_e;

// -----------------------------------------------------------------------------
// PMEDIAVRCALLBACK function prototype
//
// MMVR event callback
// -----------------------------------------------------------------------------
typedef
HRESULT
(*PMEDIAVRCALLBACK)(
    __in            HMMVRMACGLRENDERER hMMediaVRCAGL,
    __in            MediaVREvent_e eEventType,
    __in            void* pvContext
);


// ----------------------------------------------------------------------------
// MediaVRCreateCAGLRenderer
//
// Create and retrieve a handle to an CAGL Renderer
//
// Note: the output param phWnd points to a object of CALayer or sub-class on return. 
//
// ----------------------------------------------------------------------------
HRESULT
MediaVRCreateCAGLRenderer(
    __out   HWND *phWnd,
    __in    PMEDIAVRCALLBACK fnCallback,
    __in    void* pvCallbackContext,
    __in    MediaVRAutoFitMode_e eAutoFitMode,
    __out   HMMVRMACGLRENDERER* phMMediaVRCAGL
);


// ----------------------------------------------------------------------------
// MediaVRDestroyCAGLRenderer
//
// Destroy an CAGL Renderer
// ----------------------------------------------------------------------------
HRESULT
MediaVRDestroyCAGLRenderer(
    __in    HMMVRMACGLRENDERER hMMediaVRCAGL
);


// ----------------------------------------------------------------------------
// MediaVRGetCAGLRenderContext
//
// Retrieve the render context to be used in creation of the sink.
// ----------------------------------------------------------------------------
HRESULT
MediaVRGetCAGLRenderContext(
    __in    HMMVRMACGLRENDERER hMMediaVRCAGL,
    __out   const void** ppVoidContext
);

// ----------------------------------------------------------------------------
// MediaVRSetCAGLAutoFitMode
//
// Set auto-fit mode
// ----------------------------------------------------------------------------
HRESULT
MediaVRSetCAGLAutoFitMode(
    __in    HMMVRMACGLRENDERER hMMediaVRCAGL,
    __in    MediaVRAutoFitMode_e eAutoFitMode
);
// ----------------------------------------------------------------------------
// MediaVRGetCAGLRenderedVideoSize
//
// Retrieve the rendered video size.
// ----------------------------------------------------------------------------
HRESULT
MediaVRGetCAGLRenderedVideoSize(
    __in    HMMVRMACGLRENDERER hMMediaVRCAGL,
    __out   WORD* pwRenderWidth,
    __out   WORD* pwRenderHeight,
    __out   WORD* pwRenderCropWidth,
    __out   WORD* pwRenderCropHeight
);

void
MediaVRCAGLEnableTrace(
    __in BOOL bEnabled,
    __in const char *pszQualifier
);

HRESULT
MediaVRSetCAGLImageProcessingOptions(
    __in HMMVRMACGLRENDERER hMMediaVRCAGL,
    __in const MMVR_ImageProcessingOption * pOptions
);

#ifdef __cplusplus
}; // extern "C"
#endif

#endif //__MMVR_CAGL__
