#ifndef __RTCPAL_PRIVATE_H__
#define __RTCPAL_PRIVATE_H__

#ifdef __cplusplus

#include <deque>

#if defined(ANDROID)

    // patch ANDROID STL.
    //
    // This must come before including 'algorithm' for __median
    // to be effective.
    // Since it references _GLIBXX_BEGING_NAMESPACE, it must
    // come after one of STL header file.
#ifdef _GLIBCXX_BEGIN_NAMESPACE
    _GLIBCXX_BEGIN_NAMESPACE(std)
#else
    namespace __gnu_cxx _GLIBCXX_VISIBILITY(default)
    {
    _GLIBCXX_BEGIN_NAMESPACE_VERSION
#endif
      template<typename _Tp, typename _Compare>
        inline _Tp&
        __median(_Tp& __a, _Tp& __b, _Tp& __c, _Compare __comp)
        {
          // concept requirements
          __glibcxx_function_requires(_BinaryFunctionConcept<_Compare, bool,
                                                             _Tp, _Tp>)
          if (__comp(__a, __b))
            if (__comp(__b, __c))
              return __b;
            else if (__comp(__a, __c))
              return __c;
            else
              return __a;
          else if (__comp(__a, __c))
            return __a;
          else if (__comp(__b, __c))
            return __c;
          else
            return __b;
        }
#ifdef _GLIBCXX_END_NAMESPACE
    _GLIBCXX_END_NAMESPACE
#else
    _GLIBCXX_END_NAMESPACE_VERSION
    } // namespace
#endif
#endif

#include <bitset>
#include <list>
#include <limits>
#include <vector>
#include <set>
#include <sstream>
#include <iterator>
#include <algorithm>
#endif

#include <rtcpal_types.h>
typedef uint32_t RTCPAL_ERROR;

#undef __if_exists
#define __if_exists(symbol)
#undef __if_not_exists
#define __if_not_exists(symbol)
//#define __declspec(whatever)

///Missing RPC SAL Annotations
#define __RPC__inout_xcount(count)
#define __RPC__in_xcount(count)

#if HAVE_LIBKERN_OSATOMIC_H
    // for OSAtomicIncrement, OSSpinLock and etc
    #include <libkern/OSAtomic.h>
#endif

#if HAVE_SYS_ATOMICS_H
    #include <sys/atomics.h>
#endif

#if HAVE_LIBKERN_OSTYPES_H
    // for UInt, SInt date type definitions (required by AudioDsp codec)
    // We need to cleanup AudioDsp code to use PAL data type in order to
    // get rid of this dependency.
    #include <libkern/OSTypes.h>
#else
    typedef unsigned int UInt;
    typedef int SInt;
#endif

#include <pthread.h>
#include <string.h>
#include <strings.h>
#include <limits.h>
#include <wchar.h>
#include <netinet/in.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/errno.h>
#include <fcntl.h>
#include <getopt.h>

#if defined(RTCPAL_ARCH_X86) || defined(RTCPAL_ARCH_X64)
#if !defined(__linux)
#include <pmmintrin.h>
#endif
#include <mmintrin.h>
#include <emmintrin.h>
#include <xmmintrin.h>
#endif

#define RTCPAL_ASE128_KEY_LENGTH        16

// CommonCrypto is preferred over openssl
#if HAVE_COMMONCRYPTO_COMMONCRYPTOR_H
    #include <CommonCrypto/CommonCryptor.h>
    #include <CommonCrypto/CommonDigest.h>
    #include <CommonCrypto/CommonHMAC.h>

	typedef struct RTCPAL_HMACSHAKEY {
		uint32_t len;
		void* key;
	} RTCPAL_HMACSHAKEY;

    typedef RTCPAL_HMACSHAKEY RTCPAL_HMACSHA1KEY;
    typedef CCHmacContext RTCPAL_HMACSHA1;

    typedef RTCPAL_HMACSHAKEY RTCPAL_HMACSHA256KEY;
    typedef CCHmacContext RTCPAL_HMACSHA256;

    typedef RTCPAL_HMACSHAKEY RTCPAL_HMACSHA384KEY;
    typedef CCHmacContext RTCPAL_HMACSHA384;

    typedef RTCPAL_HMACSHAKEY RTCPAL_HMACSHA512KEY;
    typedef CCHmacContext RTCPAL_HMACSHA512;

    typedef CC_MD5_CTX RTCPAL_MD5_CTX;
    typedef CC_SHA256_CTX RTCPAL_SHA256_CTX;
    typedef CC_SHA512_CTX RTCPAL_SHA384_CTX;
    typedef CC_SHA512_CTX RTCPAL_SHA512_CTX;

    typedef struct RTCPAL_AESKEY {
        CCCryptorRef AES;
    } RTCPAL_AESKEY;

#elif HAVE_OPENSSL
    #include <openssl/md5.h>
    #include <openssl/aes.h>
    #include <openssl/hmac.h>
    #include <openssl/sha.h>

    typedef struct RTCPAL_HMACSHAKEY {
        uint32_t len;
        const void * key;
    } RTCPAL_HMACSHAKEY;

    typedef HMAC_CTX            RTCPAL_HMACSHA1;
    typedef RTCPAL_HMACSHAKEY   RTCPAL_HMACSHA1KEY;    
    typedef HMAC_CTX            RTCPAL_HMACSHA256;
    typedef RTCPAL_HMACSHAKEY   RTCPAL_HMACSHA256KEY;
    typedef HMAC_CTX            RTCPAL_HMACSHA384;
    typedef RTCPAL_HMACSHAKEY   RTCPAL_HMACSHA384KEY;    
    typedef HMAC_CTX            RTCPAL_HMACSHA512;
    typedef RTCPAL_HMACSHAKEY   RTCPAL_HMACSHA512KEY;

    typedef MD5_CTX             RTCPAL_MD5_CTX;
	typedef SHA512_CTX			SHA384_CTX;
	typedef SHA256_CTX			RTCPAL_SHA256_CTX;
    typedef SHA384_CTX 			RTCPAL_SHA384_CTX;
    typedef SHA512_CTX 			RTCPAL_SHA512_CTX;

    typedef struct RTCPAL_AESKEY {
        AES_KEY AES;
    } RTCPAL_AESKEY;

#endif

#ifdef __cplusplus
extern "C" {
#endif

#if BYTE_ORDER == BIG_ENDIAN
    #define RTCPAL_BIG_ENDIAN
#elif BYTE_ORDER == LITTLE_ENDIAN
    #define RTCPAL_LITTLE_ENDIAN
#else
    #error "BYTE_ORDER must be defined"
#endif

#if HAVE_OSSPINLOCKLOCK
    
// Use pthread_mutex for critical section. Please use RTCPAL_SLIM_LOCK if spin lock is needed for performance issue.
typedef pthread_mutex_t RTCPAL_CRITICAL_SECTION, * PRTCPAL_CRITICAL_SECTION;
typedef OSSpinLock RTCPAL_SLIM_LOCK, * PRTCPAL_SLIM_LOCK;
    
#else

typedef pthread_mutex_t RTCPAL_CRITICAL_SECTION, * PRTCPAL_CRITICAL_SECTION;
typedef pthread_mutex_t RTCPAL_SLIM_LOCK, * PRTCPAL_SLIM_LOCK;

#endif

#ifdef __cplusplus
};
#endif

#include <specstrings.h>

#include <rtcpal_msc_crt.h>
#include <windef.h> // required for rtcpal_atomic.h
#include <rtcpal_atomic.h>
#include <rtcpal_windows.h>


#ifdef __cplusplus
extern "C" {
#endif
#define ASYNC_STATUS_PAL_RESERVED   32

#define RTCPAL_PATH_DELIMITER   '/'

#if HAVE_OSSPINLOCKLOCK

static
FORCEINLINE
RTCPAL_BOOL
RTCAPI
RtcPalInitializeSlimLock(
    PRTCPAL_SLIM_LOCK lock
    )
{
    *lock = 0;
    return TRUE;
}

    #define RtcPalDeleteSlimLock(lock)
    #define RtcPalAcquireSlimLock(lock)     OSSpinLockLock(lock)
    #define RtcPalReleaseSlimLock(lock)     OSSpinLockUnlock(lock)
    #define RtcPalTryAcquireSlimLock(lock)  OSSpinLockTry(lock)

#else //no spin lock, use mutex instead

RTCPAL_BOOL
RTCAPI
RtcPalInitializeSlimLock(
	PRTCPAL_SLIM_LOCK lock
	);

void
RTCAPI
RtcPalDeleteSlimLock(
	PRTCPAL_SLIM_LOCK lock
	);

void
RTCAPI
RtcPalAcquireSlimLock(
	PRTCPAL_SLIM_LOCK lock
	);

void
RTCAPI
RtcPalReleaseSlimLock(
	PRTCPAL_SLIM_LOCK lock
	);

RTCPAL_BOOL
RTCAPI
RtcPalTryAcquireSlimLock(
	PRTCPAL_SLIM_LOCK lock
	);

#ifdef HAVE_INTERLOCK_64

// Only include this file in C compiler
#ifndef __cplusplus
#include <stdbool.h>
#endif

#ifndef __clang__
bool
RTCAPI
__sync_bool_compare_and_swap_8(
    volatile void*,
    uint64_t oldv,
    uint64_t newv
    );

uint64_t
RTCAPI
__sync_val_compare_and_swap_8(
    volatile void * target,
    uint64_t oldv,
    uint64_t newv
    );
#endif

#endif

#endif

#define __drv_aliasesMem

// undefine all the definition from MACBU header files
#undef THREAD_PRIORITY_TIME_CRITICAL
#undef THREAD_PRIORITY_HIGHEST
#undef THREAD_PRIORITY_ABOVE_NORMAL
#undef THREAD_PRIORITY_NORMAL
#undef THREAD_PRIORITY_BELOW_NORMAL
#undef THREAD_PRIORITY_LOWEST
#undef THREAD_PRIORITY_ABOVE_IDLE
#undef THREAD_PRIORITY_IDLE
#undef THREAD_PRIORITY_ERROR_RETURN

#define THREAD_PRIORITY_TIME_CRITICAL   7
#define THREAD_PRIORITY_HIGHEST         6
#define THREAD_PRIORITY_ABOVE_NORMAL    5
#define THREAD_PRIORITY_NORMAL          4
#define THREAD_PRIORITY_BELOW_NORMAL    3
#define THREAD_PRIORITY_LOWEST          2
#define THREAD_PRIORITY_ABOVE_IDLE      1
#define THREAD_PRIORITY_IDLE            0

#define THREAD_PRIORITY_ERROR_RETURN    (-1)

// per DNS specification, each label may contain up to 63 characters
#define RTCPAL_MAX_COMPUTERNAME_LENGTH  63

#ifdef __cplusplus
};
#endif

#endif // __RTCPAL_PRIVATE_H__
