#ifndef __ATLAUTOPTR_H__
#define __ATLAUTOPTR_H__

#include <atlconv.h>
#include <atlcomcli.h>

namespace ATL
{


template< typename T >
class CAutoPtr
{
public:
	CAutoPtr() RTCPAL_NOTHROW :
		m_p( NULL )
	{
	}
    // under Android it doesn't support overload such functions, so comment it out
	//template< typename TSrc >
	//CAutoPtr(CAutoPtr< TSrc >& p) RTCPAL_NOTHROW;
	template< typename TSrc >
	CAutoPtr(CAutoPtr< TSrc >& p) 
	{
		m_p = p.Detach();  // Transfer ownership
	}
	CAutoPtr(CAutoPtr< T >& p) RTCPAL_NOTHROW
	{
		m_p = p.Detach();  // Transfer ownership
	}
	explicit CAutoPtr(T* p) RTCPAL_NOTHROW :
		m_p( p )
	{
	}
	~CAutoPtr() RTCPAL_NOTHROW
	{
		Free();
	}

	// Templated version to allow pBase = pDerived
	// under Android it doesn't support overload such functions, so comment it out
	//template< typename TSrc >
	//CAutoPtr< T >& operator=(CAutoPtr< TSrc >& p) RTCPAL_NOTHROW;
	template< typename TSrc >
	CAutoPtr< T >& operator=(CAutoPtr< TSrc >& p) 
	{
		if(m_p==p.m_p)
		{
			// This means that two CAutoPtrs of two different types had the same m_p in them
			// which is never correct
			ATLASSERT(FALSE);
		}
		else
		{
			Free();
			Attach( p.Detach() );  // Transfer ownership
		}
		return( *this );
	}
	CAutoPtr< T >& operator=(CAutoPtr< T >& p) RTCPAL_NOTHROW
	{
		if(*this==p)
		{
			if(this!=&p)
			{
				// If this assert fires, it means you attempted to assign one CAutoPtr to another when they both contained
				// a pointer to the same underlying object. This means a bug in your code, since your object will get
				// double-deleted.
#ifdef ATL_AUTOPTR_ASSIGNMENT_ASSERT
				ATLASSERT(FALSE);
#endif

				// For safety, we are going to detach the other CAutoPtr to avoid a double-free. Your code still
				// has a bug, though.
				p.Detach();
			}
			else
			{
				// Alternatively, this branch means that you are assigning a CAutoPtr to itself, which is
				// pointless but permissible

				// nothing to do
			}
		}
		else
		{
			Free();
			Attach( p.Detach() );  // Transfer ownership
		}
		return( *this );
	}

	// basic comparison operators
	bool operator!=(CAutoPtr<T>& p) const
	{
		return !operator==(p);
	}

	bool operator==(CAutoPtr<T>& p) const
	{
		return m_p==p.m_p;
	}

	operator T*() const RTCPAL_NOTHROW
	{
		return( m_p );
	}
	T* operator->() const RTCPAL_NOTHROW
	{
		ATLASSUME( m_p != NULL );
		return( m_p );
	}

	// Attach to an existing pointer (takes ownership)
	void Attach(T* p) RTCPAL_NOTHROW
	{
		ATLASSUME( m_p == NULL );
		m_p = p;
	}
	// Detach the pointer (releases ownership)
	T* Detach() RTCPAL_NOTHROW
	{
		T* p;

		p = m_p;
		m_p = NULL;

		return( p );
	}
	// Delete the object pointed to, and set the pointer to NULL
	void Free() RTCPAL_NOTHROW
	{
		delete m_p;
		m_p = NULL;
	}

public:
	T* m_p;
};


}	// namespace ATL

#ifndef _ATL_NO_AUTOMATIC_NAMESPACE
using namespace ATL;
#endif //!_ATL_NO_AUTOMATIC_NAMESPACE

#endif // __ATLAUTOPTR_H__
