#ifndef __ATLBASE_H__
#define __ATLBASE_H__

#include <atlconv.h>
#include <atlcomcli.h>

namespace ATL
{

template< typename T >
class CAutoVectorPtr
{
public:
	CAutoVectorPtr() RTCPAL_NOTHROW :
		m_p( NULL )
	{
	}
	CAutoVectorPtr( CAutoVectorPtr< T >& p ) RTCPAL_NOTHROW
	{
		m_p = p.Detach();  // Transfer ownership
	}
	explicit CAutoVectorPtr( T* p ) RTCPAL_NOTHROW :
		m_p( p )
	{
	}
	~CAutoVectorPtr() RTCPAL_NOTHROW
	{
		Free();
	}

	operator T*() const RTCPAL_NOTHROW
	{
		return( m_p );
	}

	CAutoVectorPtr< T >& operator=( CAutoVectorPtr< T >& p ) RTCPAL_NOTHROW
	{
        if(*this==p)
        {
            if(this!=&p)
            {
                // If this assert fires, it means you attempted to assign one CAutoVectorPtr to another when they both contained 
                // a pointer to the same underlying vector. This means a bug in your code, since your vector will get 
                // double-deleted. 
                ATLASSERT(FALSE);

                // For safety, we are going to detach the other CAutoVectorPtr to avoid a double-free. Your code still
                // has a bug, though.
                p.Detach();
            }
            else
            {
                // Alternatively, this branch means that you are assigning a CAutoVectorPtr to itself, which is
                // pointless but permissible

                // nothing to do
            }
        }
        else
        {
		    Free();
		    Attach( p.Detach() );  // Transfer ownership
        }
		return( *this );
	}

    // basic comparison operators
    bool operator!=(CAutoVectorPtr<T>& p) const
    {
        return !operator==(p);
    }

    bool operator==(CAutoVectorPtr<T>& p) const
    {
        return m_p==p.m_p;
    }

	// Allocate the vector
	bool Allocate( size_t nElements ) RTCPAL_NOTHROW
	{
		ATLASSUME( m_p == NULL );
		ATLTRY( m_p = new T[nElements] );
		if( m_p == NULL )
		{
			return( false );
		}

		return( true );
	}
	// Attach to an existing pointer (takes ownership)
	void Attach( T* p ) RTCPAL_NOTHROW
	{
		ATLASSUME( m_p == NULL );
		m_p = p;
	}
	// Detach the pointer (releases ownership)
	T* Detach() RTCPAL_NOTHROW
	{
		T* p;

		p = m_p;
		m_p = NULL;

		return( p );
	}
	// Delete the vector pointed to, and set the pointer to NULL
	void Free() RTCPAL_NOTHROW
	{
		delete[] m_p;
		m_p = NULL;
	}

public:
	T* m_p;
};

}	// namespace ATL

#ifndef _ATL_NO_AUTOMATIC_NAMESPACE
using namespace ATL;
#endif //!_ATL_NO_AUTOMATIC_NAMESPACE

#include <atlautoptr.h>
#include <atlcomvar.h>
#include <atlsimpcoll.h>

#endif // __ATLBASE_H__
