/*++

Copyright (C) Microsoft Corporation. All rights reserved.

Module Name:

    rtcpal_private.h

Abstract:

    Platform specific header file for Windows classic and Windows Modern
    platforms.

    RTCPAL_MSDK_CLEAN
        Defining this macro will cause rtcpal.h to include only the header
        files from modern SDK. Some required definitions will be missing and
        can be added to rtcpal_msdk_private.h if required.

        For example, the following header files will not be included
                ws2tcpip.h
                winsock2.h
                wtypes.h
                mmsystem.h

        Some header files are present in both classic WIN32 SDK and modern SDK.
        In INCLUDES macro, MOSDK_INC_PATH must be before SDK_INC_PATH in order
        to really pick up the modern version header files.

Author:

Revision History:

--*/
#pragma once

#include <sal.h>

// All Windows platforms except XBOX360 runs on little endian machines
#define RTCPAL_LITTLE_ENDIAN

#ifndef RTCPAL_MSDK_CLEAN
    #include <ws2tcpip.h>
    #include <mstcpip.h>
    #include <winsock2.h>
#endif  // RTCPAL_MSDK_CLEAN

#include <rtcpal_windows.h>

#ifndef RTCPAL_MSDK_CLEAN
    #include <wtypes.h>
    #include <mmsystem.h>
#endif

#if !defined(ARM)
#include <mmintrin.h>
#include <emmintrin.h>
#include <xmmintrin.h>
#endif

#if defined(WINPHONE)
#include <errno.h>
#include <mbstring.h>
#include <oleauto.h>
#endif

// No need to bring in cfgmgr32.h just for CMP_WaitNoPendingInstallEvents
// #include <cfgmgr32.h>
EXTERN_C
DWORD WINAPI
CMP_WaitNoPendingInstallEvents(
    IN DWORD dwTimeout
    );

#include <fcntl.h>
#include <io.h>

// disable noisy warnings
//  4100 - unreferenced parameters
#pragma warning(disable:4100)

// disable C4995 warning temporarily for intrin.h.
#pragma warning(push)
#pragma warning(disable:4995)
#include <intrin.h>
#pragma warning(pop)

#include <objbase.h>

#ifdef __cplusplus
extern "C" {
#endif

#if _MSC_VER >= 1600
    // Visual Studio 2010 supports C99 data types.
    //
    // We can't include stdint.h here since
    // stdint.h is in stl100 directory in Windows and WP8 build environment.
    // This requires us to set USE_STL which bring in the exception-throwing
    // new operator.
    //
    // #include <stdint.h>

    // The following C99 definitions are compatible with MSC stdint.h. If
    // they didn't, we would get C2371 error (type redefinition) when
    // built under lcsrazzle. This is because some CRT headers in LCS build
    // environment include stdint.h. And MSC compiler raise errors only when
    // two defitions are not identical.

    typedef UINT8 uint8_t;
    typedef UINT16 uint16_t;
    typedef UINT32 uint32_t;
    typedef UINT64 uint64_t;
    typedef INT8 int8_t;
    typedef INT16 int16_t;
    typedef INT32 int32_t;
    typedef INT64 int64_t;

    // intsafe.h requires disabling "macro redefinition" warning.
    #pragma warning(push)
    #pragma warning(disable:4005)
    #include <intsafe.h>
    #pragma warning(pop)
#else
    // MSC before VS2010 doesn't support C99 data types. We need to define our own.
    #if 1
        // This is the original definition when PAL was implemented.
        typedef unsigned char uint8_t;
        typedef unsigned short uint16_t;
        typedef unsigned long uint32_t;
        typedef unsigned __int64 uint64_t;
        typedef signed char int8_t;
        typedef signed short int16_t;
        typedef signed long int32_t;
        typedef signed __int64 int64_t;
        #if !defined(_WCHAR_T_DEFINED)
            typedef unsigned short wchar_t;
        #endif
    #else
        // These are the definitions compatible with DEV10/DEV11 CRT.
        // They might break XBOX build.
        typedef UINT8 uint8_t;
        typedef UINT16 uint16_t;
        typedef UINT32 uint32_t;
        typedef UINT64 uint64_t;
        typedef INT8 int8_t;
        typedef INT16 int16_t;
        typedef INT32 int32_t;
        typedef INT64 int64_t;
        #if !defined(_WCHAR_T_DEFINED)
            typedef WCHAR wchar_t;
        #endif
    #endif
#endif

#include <rtcpal_types.h>

typedef uint32_t RTCPAL_ERROR;

typedef LONG    NTSTATUS;

#ifdef RTCPAL_MSDK_CLEAN
    #include "rtcpal_msdk_private.h"
#endif

#define RTCPAL_PATH_DELIMITER   '\\'
#define ASYNC_STATUS_PAL_RESERVED   32

// PAL crypto structure layout for Windows
//  ctx - an opaque context size.
//      A build time break will be raised if the context size doesn't meet
//      the minimum requirements for the crypto module.
//
//  CNG Vista vs. Win7
//      CNG key handle is a pointer to the context memory.
//      On Win7, CNG library will allocate context memory if caller doesn't
//      provide one.
//      On Vista, caller must always supply context memory.
//
//      In order to work on Vista, we choose to always allocate our own context
//      memory. In PAL's implementation, handle and pvXXXObj always have the
//      same value. Although we don't rely on this undocumented behavior, knowing
//      it may help debug some memory leak issues.
//
//  RSA32 vs CNG
//      RSA32 offers a little bit more robustness than CNG since there are no
//      failure path in RSA32. Eliminating failure path from PAL crypto isn't
//      very attractive because media stack core itself isn't hardened for
//      handling out-of-memory situation. A lot of works need to be done in
//      other code paths before we can harvest the advantage of RSA32.

#if defined(RTCPAL_ARCH_X86) || defined(RTCPAL_ARCH_X64)
    // Fast path for SRTP to avoid per-packet allocation
    // 512 bytes is worst case for HMAC SHA1
    //
    // On Win7/Vista, the required size is 286 bytes for default crypto
    // provider on x64. Size for other providers may vary. If the size is
    // bigger than what we allocate here, PAL internally will make dynamic
    // allocation. It is a litte bit slow, but is still functional.
    #define RTCPAL_HMACSHA1_CONTEXT_SIZE    64
#else
    #define RTCPAL_HMACSHA1_CONTEXT_SIZE    2
#endif

typedef union RTCPAL_HMACSHA1KEY {
    uint64_t Ctxt[RTCPAL_HMACSHA1_CONTEXT_SIZE];
} RTCPAL_HMACSHA1KEY;

typedef union RTCPAL_HMACSHA1 {
    uint64_t Ctxt[RTCPAL_HMACSHA1_CONTEXT_SIZE];
} RTCPAL_HMACSHA1;

// HMAC-SHA256 context
// if RSA32 is enabled, this size must be at least 26
// 2 is enough for CNG slow path.
// You will get a build break if this is too small.
#define RTCPAL_HMACSHA256_CONTEXT_SIZE      2
typedef union RTCPAL_HMACSHA256KEY {
    uint64_t ctx[RTCPAL_HMACSHA256_CONTEXT_SIZE];
} RTCPAL_HMACSHA256KEY;

// if RSA32 is enabled, this size must be at least 13
#define RTCPAL_SHA256_CONTEXT_SIZE          2

typedef union RTCPAL_HMACSHA256 {
    uint64_t ctx[RTCPAL_HMACSHA256_CONTEXT_SIZE];
} RTCPAL_HMACSHA256;

typedef union RTCPAL_SHA256_CTX {
    uint64_t ctx[RTCPAL_SHA256_CONTEXT_SIZE];
} RTCPAL_SHA256_CTX;

// HMAC-SHA384 context
// if RSA32 is enabled, 52 is the minimum here.
#define RTCPAL_HMACSHA384_CONTEXT_SIZE      2
typedef union RTCPAL_HMACSHA384KEY {
    uint64_t ctx[RTCPAL_HMACSHA384_CONTEXT_SIZE];
} RTCPAL_HMACSHA384KEY;

typedef union RTCPAL_HMACSHA384 {
    uint64_t ctx[RTCPAL_HMACSHA384_CONTEXT_SIZE];
} RTCPAL_HMACSHA384;

// if RSA32 is enabled, 26 is the minimum here.
#define RTCPAL_SHA384_CONTEXT_SIZE      2
typedef union RTCPAL_SHA384_CTX {
    uint64_t ctx[RTCPAL_SHA384_CONTEXT_SIZE];
} RTCPAL_SHA384_CTX;

// HMAC-SHA512 context
// if RSA32 is enabled, 52 is the minimum here.
#define RTCPAL_HMACSHA512_CONTEXT_SIZE      2
typedef union RTCPAL_HMACSHA512KEY {
    uint64_t ctx[RTCPAL_HMACSHA512_CONTEXT_SIZE];
} RTCPAL_HMACSHA512KEY;

typedef union RTCPAL_HMACSHA512 {
    uint64_t ctx[RTCPAL_HMACSHA512_CONTEXT_SIZE];
} RTCPAL_HMACSHA512;

// if RSA32 is enabled, 26 is the minimum here.
#define RTCPAL_SHA512_CONTEXT_SIZE      2
typedef union RTCPAL_SHA512_CTX {
    uint64_t ctx[RTCPAL_SHA512_CONTEXT_SIZE];
} RTCPAL_SHA512_CTX;

typedef union RTCPAL_MD5_CTX {
    uint32_t ctx[26];
} RTCPAL_MD5_CTX;

// This definition is copied from the aesfast.h.  This structure's layout should never change.
typedef struct RTCPAL_AESKEY_RSA
{
    __declspec( align( 16 ) ) BYTE RoundKey[29][4][4];  
        // Round keys, first the encryption round keys in encryption order, 
        // followed by the decryption round keys in decryption order.
        // The first decryption round key is the last encryption round key.
        // AES-256 has 14 rounds and thus 15 round keys for encryption and 15
        // for decryption. As they share one round key, we need room for 29.
    unsigned int    lastEncRoundKey;    // offset to last round key for encryption
                                        // also the first round key for decryption
    unsigned int    lastDecRoundKey;    // offset to last round key for decryption
} RTCPAL_AESKEY_RSA;


// RTCPAL_AESKEY can't be a "union" at the outer because union can't be inherited.
// TODO: change rtpstack to use RTCPAL_AESKEY as the embedded field (instead of
// inheritance).
typedef struct RTCPAL_AESKEY {
    union {
        // RTCPAL_AESKEY_RSA rsa;
        void * hKey;
        uint64_t ctxt[2];
    } aes;
} RTCPAL_AESKEY;

#define RTCPAL_ASE128_KEY_LENGTH        16

// TODO: run perf test and decide if to use SWLOCK
#if 0
typedef struct RTCPAL_CRITICAL_SECTION {
    SRWLOCK Lock;
    DWORD Owner;    // TID of the owner thread
    int Count;
} RTCPAL_CRITICAL_SECTION, * PRTCPAL_CRITICAL_SECTION;
#else
typedef CRITICAL_SECTION RTCPAL_CRITICAL_SECTION;
typedef RTCPAL_CRITICAL_SECTION * PRTCPAL_CRITICAL_SECTION;

#if defined(RTCPAL_MSDK_CLEAN) || defined(WINPHONE)
    #define RtcPalInitializeCriticalSection(c)  InitializeCriticalSectionEx(c,0,0)
    #define RtcPalInitializeCriticalSectionAndSpinCount(c,s)  InitializeCriticalSectionEx(c,s,0)
#else
    #define RtcPalInitializeCriticalSection(c)  InitializeCriticalSectionAndSpinCount(c,0)
    #define RtcPalInitializeCriticalSectionAndSpinCount(c,s)  InitializeCriticalSectionAndSpinCount(c,s)
#endif

#ifdef RTCPAL_VALIDATE_CRITICAL_THREAD_TIMING
    EXTERN_C
    void
    RTCAPI
    RtcPalEnterCriticalSection(
        __inout PRTCPAL_CRITICAL_SECTION lpCriticalSection
        );

    EXTERN_C
    RTCPAL_BOOL
    RTCAPI
    RtcPalTryEnterCriticalSection(
        __inout PRTCPAL_CRITICAL_SECTION lpCriticalSection
        );

    EXTERN_C
    void
    RTCAPI
    RtcPalLeaveCriticalSection(
        __inout PRTCPAL_CRITICAL_SECTION lpCriticalSection
        );

    // Intercept critical section used from inside PAL.
    //
    // Note: critical section usage from outside PAL is intercepted via
    //       DLL export table forwarder.
    //       For inside PAL usage, we need this macro
    #undef EnterCriticalSection
    #undef TryEnterCriticalSection
    #undef LeaveCriticalSection

    #define EnterCriticalSection(c)       RtcPalEnterCriticalSection(c)
    #define TryEnterCriticalSection(c)    RtcPalTryEnterCriticalSection(c)
    #define LeaveCriticalSection(c)       RtcPalLeaveCriticalSection(c)
#else // RTCPAL_VALIDATE_CRITICAL_THREAD_TIMING
    #define RtcPalEnterCriticalSection(c)       EnterCriticalSection(c)
    #define RtcPalTryEnterCriticalSection(c)    TryEnterCriticalSection(c)
    #define RtcPalLeaveCriticalSection(c)       LeaveCriticalSection(c)
#endif // RTCPAL_VALIDATE_CRITICAL_THREAD_TIMING

#define RtcPalDeleteCriticalSection(c)      DeleteCriticalSection(c)

#define RtcPalHeapCreate                    HeapCreate
#define RtcPalHeapDestroy                   HeapDestroy
#define RtcPalHeapAlloc                     HeapAlloc
#define RtcPalHeapFree                      HeapFree

#define RtcPalFlushFileBuffers              FlushFileBuffers

#define RtcPalSystemTimeToFileTime          SystemTimeToFileTime
#define RtcPalFileTimeToSystemTime          FileTimeToSystemTime

#endif

typedef CRITICAL_SECTION RTCPAL_SLIM_LOCK, *PRTCPAL_SLIM_LOCK;

RTCPAL_BOOL RtcPalInitializeSlimLock(PRTCPAL_SLIM_LOCK pLock);
void RtcPalDeleteSlimLock(PRTCPAL_SLIM_LOCK pLock);
void RtcPalAcquireSlimLock(PRTCPAL_SLIM_LOCK pLock);
void RtcPalReleaseSlimLock(PRTCPAL_SLIM_LOCK pLock);
RTCPAL_BOOL RtcPalTryAcquireSlimLock(PRTCPAL_SLIM_LOCK pLock);

#ifdef WINXP
#include "rtcpal_private_xp.h"
#endif

// Perf: Use intrinsics for interlocked operations

// perf optimization moved from lccbuffer.h
#pragma intrinsic(memcpy)
#pragma intrinsic(memset)
#if !defined(ARM)
#pragma intrinsic(__rdtsc)
#endif 

#if DBG == 0
#pragma intrinsic (_InterlockedIncrement)
#pragma intrinsic (_InterlockedDecrement)
#define InterlockedIncrement _InterlockedIncrement
#define InterlockedDecrement _InterlockedDecrement
#endif

#pragma intrinsic(_bittestandreset)
#pragma intrinsic(_bittestandset)
#pragma intrinsic(_bittest)
#define bittestandreset _bittestandreset
#define bittestandset _bittestandset
#define bittest _bittest

#define RTCPAL_MAX_COMPUTERNAME_LENGTH  MAX_COMPUTERNAME_LENGTH

// Some goodies from POSIX
//  "man getopt" on MAC/Unix system on how to use these.
int
getopt(int argc, char * const argv[], const char *optstring);

RTC_DATA_LINKAGE extern char *optarg;
RTC_DATA_LINKAGE extern int optind, opterr, optopt, optreset;

struct option {
    const char * name;
    int has_arg;
    int * flag;
    int val;
};

int
getopt_long(int argc, char * const argv[],
        const char *optstring, const struct option *longopts, int *longindex);

int
getopt_long_only(int argc, char * const argv[],
        const char *optstring, const struct option *longopts, int *longindex);

#if (defined(_M_IX86) && (_MSC_FULL_VER > 13009037)) || ((defined(_M_AMD64) || defined(_M_IA64)) && (_MSC_FULL_VER > 13009175)) || defined(_M_ARM)
    unsigned short __cdecl _byteswap_ushort(_In_ unsigned short);
    unsigned long  __cdecl _byteswap_ulong (_In_ unsigned long);
    uint64_t __cdecl _byteswap_uint64(_In_ uint64_t);

    #pragma intrinsic(_byteswap_ushort)
    #pragma intrinsic(_byteswap_ulong)
    #pragma intrinsic(_byteswap_uint64)

    #define RTCPAL_BYTESWAP_UINT16  _byteswap_ushort
    #define RTCPAL_BYTESWAP_UINT32  _byteswap_ulong
    #define RTCPAL_BYTESWAP_UINT64  _byteswap_uint64

    #ifndef __cplusplus_cli
        // Media Stack Core use these routines, which are defined in WS2_32.DLL.
        // WS2_32.DLL is not "blessed" (not WINRT compliant).
        // Redefine them to PAL version
        #undef htonl
        #undef ntohl
        #undef htons
        #undef ntohs

        // byteswap is always required since Windows is little endian across
        // every platform which it has been ported to.
        #define htonl   RTCPAL_BYTESWAP_UINT32
        #define ntohl   RTCPAL_BYTESWAP_UINT32
        #define htons   RTCPAL_BYTESWAP_UINT16
        #define ntohs   RTCPAL_BYTESWAP_UINT16
    #else
        // This is for managed media stack, which doesn't need to run inside WINRT.
        // managed C++ can't use intrinsic in a managed function.
        // Let it use WS2_32.DLL.
    #endif
#else
    #error "Unsupported compiler version"
#endif

#if !defined(_M_IX86) && !defined(_M_IA64) && !defined(_M_AMD64) && !defined(_M_ARM)
    // decorated name used in /alternatename depends on the architecture.
    #error Unsupported platform
#endif

#if WINAPI_FAMILY_PARTITION(WINAPI_PARTITION_APP)
// Immersive SDK blocks these defines for Reg APIs, but since our PAL APIs are
// are using the same signature (but mapped internally to modern versions), we
// need the definitions here locally.
typedef ACCESS_MASK REGSAM;
#define HKEY_CLASSES_ROOT                   (( HKEY ) (ULONG_PTR)((LONG)0x80000000) )
#define HKEY_CURRENT_USER                   (( HKEY ) (ULONG_PTR)((LONG)0x80000001) )
#define HKEY_LOCAL_MACHINE                  (( HKEY ) (ULONG_PTR)((LONG)0x80000002) )
#define HKEY_USERS                          (( HKEY ) (ULONG_PTR)((LONG)0x80000003) )
#define HKEY_PERFORMANCE_DATA               (( HKEY ) (ULONG_PTR)((LONG)0x80000004) )
#define HKEY_PERFORMANCE_TEXT               (( HKEY ) (ULONG_PTR)((LONG)0x80000050) )
#define HKEY_PERFORMANCE_NLSTEXT            (( HKEY ) (ULONG_PTR)((LONG)0x80000060) )
#define HKEY_CURRENT_CONFIG                 (( HKEY ) (ULONG_PTR)((LONG)0x80000005) )
#define HKEY_DYN_DATA                       (( HKEY ) (ULONG_PTR)((LONG)0x80000006) )
#define HKEY_CURRENT_USER_LOCAL_SETTINGS    (( HKEY ) (ULONG_PTR)((LONG)0x80000007) )
#endif /* WINAPI_PARTITION_APP */

RTCPAL_ERROR
RTCAPI
RtcPalRegCreateKeyExA (
    __in HKEY hKey,
    __in LPCSTR lpSubKey,
    __reserved DWORD Reserved,
    __in_opt LPSTR lpClass,
    __in DWORD dwOptions,
    __in REGSAM samDesired,
    __in_opt CONST LPSECURITY_ATTRIBUTES lpSecurityAttributes,
    __out PHKEY phkResult,
    __out_opt LPDWORD lpdwDisposition
    );
#pragma comment(linker, "/alternatename:_RtcPalRegCreateKeyExA@36=_RegCreateKeyExA@36")
#pragma comment(linker, "/alternatename:RtcPalRegCreateKeyExA=RegCreateKeyExA")

RTCPAL_ERROR
RTCAPI
RtcPalRegCreateKeyExW (
    __in HKEY hKey,
    __in LPCWSTR lpSubKey,
    __reserved DWORD Reserved,
    __in_opt LPWSTR lpClass,
    __in DWORD dwOptions,
    __in REGSAM samDesired,
    __in_opt CONST LPSECURITY_ATTRIBUTES lpSecurityAttributes,
    __out PHKEY phkResult,
    __out_opt LPDWORD lpdwDisposition
    );
#pragma comment(linker, "/alternatename:_RtcPalRegCreateKeyExW@36=_RegCreateKeyExW@36")
#pragma comment(linker, "/alternatename:RtcPalRegCreateKeyExW=RegCreateKeyExW")

RTCPAL_ERROR
RTCAPI
RtcPalRegOpenKeyExA (
    __in HKEY hKey,
    __in_opt LPCSTR lpSubKey,
    __in_opt DWORD ulOptions,
    __in REGSAM samDesired,
    __out PHKEY phkResult
    );
#pragma comment(linker, "/alternatename:_RtcPalRegOpenKeyExA@20=_RegOpenKeyExA@20")
#pragma comment(linker, "/alternatename:RtcPalRegOpenKeyExA=RegOpenKeyExA")

RTCPAL_ERROR
RTCAPI
RtcPalRegOpenKeyExW (
    __in HKEY hKey,
    __in_opt LPCWSTR lpSubKey,
    __in_opt DWORD ulOptions,
    __in REGSAM samDesired,
    __out PHKEY phkResult
    );
#pragma comment(linker, "/alternatename:_RtcPalRegOpenKeyExW@20=_RegOpenKeyExW@20")
#pragma comment(linker, "/alternatename:RtcPalRegOpenKeyExW=RegOpenKeyExW")

RTCPAL_ERROR
RTCAPI
RtcPalRegQueryValueExA(
    __in HKEY hKey,
    __in_opt LPCSTR lpValueName,
    __reserved LPDWORD lpReserved,
    __out_opt LPDWORD lpType,
    __out_bcount_part_opt(*lpcbData, *lpcbData) __out_data_source(REGISTRY) LPBYTE lpData,
    __inout_opt LPDWORD lpcbData
    );
#pragma comment(linker, "/alternatename:_RtcPalRegQueryValueExA@24=_RegQueryValueExA@24")
#pragma comment(linker, "/alternatename:RtcPalRegQueryValueExA=RegQueryValueExA")

RTCPAL_ERROR
RTCAPI
RtcPalRegQueryValueExW(
    __in HKEY hKey,
    __in_opt LPCWSTR lpValueName,
    __reserved LPDWORD lpReserved,
    __out_opt LPDWORD lpType,
    __out_bcount_part_opt(*lpcbData, *lpcbData) __out_data_source(REGISTRY) LPBYTE lpData,
    __inout_opt LPDWORD lpcbData
    );
#pragma comment(linker, "/alternatename:_RtcPalRegQueryValueExW@24=_RegQueryValueExW@24")
#pragma comment(linker, "/alternatename:RtcPalRegQueryValueExW=RegQueryValueExW")

RTCPAL_ERROR
RTCAPI
RtcPalRegSetValueExA (
    __in HKEY hKey,
    __in_opt LPCSTR lpValueName,
    __reserved DWORD Reserved,
    __in DWORD dwType,
    __in_bcount_opt(cbData) CONST BYTE* lpData,
    __in DWORD cbData
    );
#pragma comment(linker, "/alternatename:_RtcPalRegSetValueExA@24=_RegSetValueExA@24")
#pragma comment(linker, "/alternatename:RtcPalRegSetValueExA=RegSetValueExA")

RTCPAL_ERROR
RTCAPI
RtcPalRegSetValueExW (
    __in HKEY hKey,
    __in_opt LPCWSTR lpValueName,
    __reserved DWORD Reserved,
    __in DWORD dwType,
    __in_bcount_opt(cbData) CONST BYTE* lpData,
    __in DWORD cbData
    );
#pragma comment(linker, "/alternatename:_RtcPalRegSetValueExW@24=_RegSetValueExW@24")
#pragma comment(linker, "/alternatename:RtcPalRegSetValueExW=RegSetValueExW")

RTCPAL_ERROR
RTCAPI
RtcPalRegCloseKey (
    __in HKEY hKey
    );
#pragma comment(linker, "/alternatename:_RtcPalRegCloseKey@4=_RegCloseKey@4")
#pragma comment(linker, "/alternatename:RtcPalRegCloseKey=RegCloseKey")

BOOL
RTCAPI
RtcPalMoveFileW(
    _In_ LPCWSTR lpExistingFileName,
    _In_ LPCWSTR lpNewFileName
    );
#pragma comment(linker, "/alternatename:_RtcPalMoveFileW@8=_MoveFileW@8")
#pragma comment(linker, "/alternatename:RtcPalMoveFileW=MoveFileW")

BOOL
RTCAPI
RtcPalMoveFileA(
    _In_ LPCSTR lpExistingFileName,
    _In_ LPCSTR lpNewFileName
    );
#pragma comment(linker, "/alternatename:_RtcPalMoveFileA@8=_MoveFileA@8")
#pragma comment(linker, "/alternatename:RtcPalMoveFileA=MoveFileA")

_Ret_maybenull_
HANDLE
WINAPI
RtcPalCreateEventW(
    _In_opt_ LPSECURITY_ATTRIBUTES lpEventAttributes,
    _In_ BOOL bManualReset,
    _In_ BOOL bInitialState,
    _In_opt_ LPCWSTR lpName
    );
#pragma comment(linker, "/alternatename:_RtcPalCreateEventW@16=_CreateEventW@16")
#pragma comment(linker, "/alternatename:RtcPalCreateEventW=CreateEventW")

_Ret_maybenull_
HANDLE
WINAPI
RtcPalCreateMutexW(
    _In_opt_ LPSECURITY_ATTRIBUTES lpMutexAttributes,
    _In_ BOOL bInitialOwner,
    _In_opt_ LPCWSTR lpName
    );
#pragma comment(linker, "/alternatename:_RtcPalCreateMutexW@12=_CreateMutexW@12")
#pragma comment(linker, "/alternatename:RtcPalCreateMutexW=CreateMutexW")

void
RTCAPI
RtcPalSleep(
    __in uint32_t dwMilliseconds
    );
#pragma comment(linker, "/alternatename:_RtcPalSleep@4=_Sleep@4")
#pragma comment(linker, "/alternatename:RtcPalSleep=Sleep")

DWORD
RTCAPI
RtcPalGetTickCount(
    void
    );
#pragma comment(linker, "/alternatename:_RtcPalGetTickCount@0=_GetTickCount@0")
#pragma comment(linker, "/alternatename:RtcPalGetTickCount=GetTickCount")

HINSTANCE
RTCAPI
RtcPalLoadLibraryW(
    LPCWSTR pszPath
    );
#pragma comment(linker, "/alternatename:_RtcPalLoadLibraryW@4=_LoadLibraryW@4")
#pragma comment(linker, "/alternatename:RtcPalLoadLibraryW=LoadLibraryW")


#include <windows/rtcpal_wpp.h>

#ifdef __cplusplus
};
#endif


